/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.hibara.attachecase.utility.Utility;

import com.apple.eawt.AppEvent.OpenFilesEvent;
import com.apple.eawt.Application;
import com.apple.eawt.OpenFilesHandler;

/**
 * Attache Case 実行用クラスです。
 * 
 * @author Masanori Uozumi
 * @version 2012.08.03<br>
 *          2012.08.30 二重起動のチェックを追加<br>
 *          2012.09.07 Mac上でファイルをダブルクリックしての起動をサポート<br>
 */
public class AttacheCase implements Runnable {
	private String[] args = null;

	private AttacheCase(String[] args) {
		super();
		this.args = args;
	}

	/**
	 * mainメソッド
	 * 
	 * @param args
	 *            コマンドライン引数
	 * @throws Exception
	 *             例外が投げられる可能性あり
	 */
	public static void main(String[] args) throws Exception {
		Options options = new Options(args);
		if (options.isCui()) {
			AttacheCase4CUI obj = new AttacheCase4CUI(options);
			obj.run();
		} else {
			// 二重起動禁止の場合は，そのチェックを行う
			if (options.isNotMulipleStarts() && isMultipleStarts()) {
				return;
			}
			SwingUtilities.invokeLater(new AttacheCase(args));
		}
	}

	@Override
	public void run() {
		if (Utility.isMac()) {
			System.setProperty("apple.laf.useScreenMenuBar", "true");
			System.setProperty("com.apple.mrj.application.apple.menu.about.name",
					Confirm.resources.getString("APP_FRAME_NAME"));
		}
		Options options = new Options(args);
		final AttacheCase4GUI gui = new AttacheCase4GUI();
		gui.setOptions(options); // 引数が存在していればその処理を行う
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			// Mac上で動作させることを考慮する
			if (Utility.isMac()) {
				GuiForMac.setUp(gui);
				// atcファイルをダブルクリックした際の処理を追加
				Application app = Application.getApplication();
				app.setOpenFileHandler(new OpenFilesHandler() {
					@Override
					public void openFiles(OpenFilesEvent evt) {
						List<File> files = evt.getFiles();
						gui.setFileList(files);
						gui.run();
					}
				});
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		// 画面表示
		gui.setVisible(true);
	}

	/**
	 * 二重起動をチェックします。
	 * 
	 * @return すでに起動している場合はtrue
	 */
	public static boolean isMultipleStarts() {
		try {
			// 起動チェック
			final File lockFile = new File(Utility.getStartupPath(), "lock_attachecase");
			final FileOutputStream fos = new FileOutputStream(lockFile);
			final FileChannel fc = fos.getChannel();
			final FileLock lock = fc.tryLock();
			if (lock == null) {
				return true;
			}

			// ロック開放処理を登録
			Runtime.getRuntime().addShutdownHook(new Thread() {
				public void run() {
					try {
						if (lock != null && lock.isValid()) {
							lock.release();
						}
						fc.close();
						fos.close();
						lockFile.delete(); // ロック用ファイルを削除
					} catch (IOException ex) {
						ex.printStackTrace();
					}
				}
			});
		} catch (IOException e) {
			// 何もしない
		}
		return false;
	}
}
