/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ResourceBundle;

import org.hibara.attachecase.encrypt.Encrypt;
import org.hibara.attachecase.encrypt.FileDecrypt2;
import org.hibara.attachecase.encrypt.FileDecrypt3;
import org.hibara.attachecase.encrypt.FileEncrypt;
import org.hibara.attachecase.encrypt.FileEncrypt3;
import org.hibara.attachecase.entity.ShaEntity;
import org.hibara.attachecase.exception.PasswordException;
import org.hibara.attachecase.utility.Digest;
import org.hibara.attachecase.utility.Utility;

/**
 * CUIで暗号化・復号を実行するクラスです。
 * 
 * @author Masanori Uozuomi
 * @version 2017.01.20<br>
 *          2012.08.06 復号処理をprivateメソッドとして独立<br>
 *          2012.08.18 ATCファイルを複数指定できるように変更。それに伴ってパスワードの指定方法を変更。復号時はATCファイル毎にパスワードの入力を行う。<br>
 *          2013.03.12 コマンドライン動作時のパスワード設定ロジックにバグがあったので修正<br>
 *          2017.01.20 アタッシェケース#3に対応<br>
 */
public class AttacheCase4CUI implements AttacheCaseConstant {

	private Options options = null;
	private Confirm confirm = null;

	/**
	 * AttacheCase4CUIを構成します。
	 * 
	 * @param options
	 *            オプション
	 */
	public AttacheCase4CUI(Options options) {
		this.options = options;
	}

	/**
	 * 暗号化・復号処理を実行します。
	 */
	public void run() {
		if (options.getRunMode() == VERSION_MODE) {
			showVersion();
			return;
		}
		confirm = new ConsoleConfirm();
		try {
			if (options.getRunMode() == DECRYPT_MODE) {
				decrypt();
			} else if (options.getRunMode() == ENCRYPT_MODE) {
				encrypt();
			}
		} catch (FileNotFoundException e) {
			// 暗号ファイルが存在しない
			Object[] replaceMessages = { e.getMessage() };
			String msg = Utility.replacePlaceHolder(
					ResourceBundle.getBundle(RESOURCE).getString(MSG_ERROR_NOT_FILE_EXIST), replaceMessages);
			confirm.showErrorMessage(msg);
		} catch (IOException e) {
			// パスワードファイルが存在しない
			Object[] replaceMessages = { (options.getRunMode() == Options.ENCRYPT_MODE)
					? options.getEncryptPasswordFileName() : options.getDecryptPasswordFileName() };
			String msg = Utility.replacePlaceHolder(
					ResourceBundle.getBundle(RESOURCE).getString(MSG_ERROR_NOT_FILE_EXIST), replaceMessages);
			confirm.showErrorMessage(msg);
		} catch (PasswordException e) {
			// パスワードが入力されていない。
			confirm.showErrorMessageByBundle(MSG_ERROR_ILLEGAL_PASSWORD);
		}
	}

	private void encrypt() throws IOException, PasswordException {
		Encrypt encrypt = null;
		if (options.isUseVersion2()) {
			encrypt = new FileEncrypt(false, options);
		} else {
			encrypt = new FileEncrypt3(false, options);
		}
		// パスワードの設定
		// 暗号化の場合は最初にパスワードを入力する。
		// 複数のファイルを暗号化する場合でも，パスワードの入力は1回だけ
		setPassword(encrypt);
		encrypt.run(null);
		if (encrypt.isComplete()) {
			// 後処理が必要であれば…
		}
	}

	/**
	 * 復号処理
	 * 
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws PasswordException
	 */
	private void decrypt() throws FileNotFoundException, IOException, PasswordException {
		for (String path : options.getDecryptFileNames()) {
			int version = Utility.getAtcVersion(new File(path));
			Encrypt decrypt = null;
			switch (version) {
			case 2:
				// 復号実行
				decrypt = new FileDecrypt2(false, options);
				break;
			case 3:
				// 復号実行
				decrypt = new FileDecrypt3(false, options);
				break;
			}
			// 復号の場合はファイル毎にパスワードを入力する。したがってパスワードの入力は復号処理の途中で行う
			if (decrypt != null) {
				decrypt.run(null);
				if (decrypt.isComplete()) {
					// 後処理が必要であれば…
				}
			}
		}
	}

	/**
	 * バージョン表示
	 */
	private void showVersion() {
		String s = ResourceBundle.getBundle(RESOURCE).getString("MSG_VERSION");
		Object[] param = { VERSION };
		System.out.printf("\n %s", Utility.replacePlaceHolder(s, param));
	}

	/**
	 * 暗号化時のパスワードを設定します。
	 * 
	 * <pre>
	 * パスワードファイルを利用する設定の場合は，ファイルを読み込んでパスワードを設定します。
	 * 入力する必要がある場合は，キーボードから入力を求めます。
	 * </pre>
	 * 
	 * @param encrypt
	 *            暗号化オブジェクト
	 * @throws IOException
	 *             ファイルが存在しない場合
	 * @throws PasswordException
	 *             パスワード入力でエラーが発生した場合
	 */
	private void setPassword(Encrypt encrypt) throws IOException, PasswordException {
		if (!options.isEncryptPasswordRequired() && options.isUsePasswordOption()) {
			// コマンドラインで指定
			encrypt.setPassword(options.getDecryptPassword());
		} else if (!options.isEncryptPasswordRequired() && options.getEncryptPasswordFileName() != null) {
			// ファイルが指定されている場合
			Digest digest = new Digest();
			ShaEntity sha = digest.getHash(options.getEncryptPasswordFileName());
			encrypt.setSha(sha);
		} else if (!options.isEncryptPasswordRequired() && options.getEncryptPassword() != null) {
			// 元々設定されているパスワード
			encrypt.setPassword(
					Utility.decrypt(Confirm.resources.getString("APP_FRAME_NAME"), options.getEncryptPassword()));
		} else {
			// 未入力時
			// 暗号化時のパスワード入力は2回
			String password = confirm.inputPassword(2);
			if (password == null) {
				throw new PasswordException();
			}
			options.setEncryptPasswardRequired(false);
			// options.setEncyptPassword(password);
			encrypt.setPassword(password);
		}
	}
}
