/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import org.hibara.attachecase.encrypt.Encrypt;
import org.hibara.attachecase.encrypt.FileDecrypt2;
import org.hibara.attachecase.encrypt.FileDecrypt3;
import org.hibara.attachecase.encrypt.FileEncrypt;
import org.hibara.attachecase.encrypt.FileEncrypt3;
import org.hibara.attachecase.entity.ShaEntity;
import org.hibara.attachecase.exception.PasswordException;
import org.hibara.attachecase.gui.ButtonPanel;
import org.hibara.attachecase.gui.IconButton;
import org.hibara.attachecase.gui.OptionDialog;
import org.hibara.attachecase.io.AtcFileFilter;
import org.hibara.attachecase.utility.AbstractProgressListener;
import org.hibara.attachecase.utility.Digest;
import org.hibara.attachecase.utility.Utility;

/**
 * GUIで暗号化・復号を実行するクラスです。
 * 
 * @author Masanori Uozuomi
 * @version 2017.01.17<br>
 *          2012.08.31 パスワードの設定メソッドを変更<br>
 *          2012.09.07 Mac上でファイルをダブルクリックして復号する処理をサポートするためにメソッドを追加<br>
 *          2017.01.17 アタッシェケース#3に対応<br>
 */

public class AttacheCase4GUI extends JFrame implements AttacheCaseConstant {
	private static final long serialVersionUID = 2612224766701174329L;
	private static AttacheCase4GUI frame = null;
	private ResourceBundle bandle = Confirm.resources;
	private final Action encryptAction = new EncryptAction();
	private final Action decryptAction = new DecryptAction();
	private final Action settingAction = new SettingAction();
	private final Action quitAction = new QuitAction();

	private Button cancelButton;
	private JProgressBar progressBar = null;
	private JLabel statusText = null;
	private List<File> fileList = null;
	private SwingWorker<Void, Integer> encryptWorker = null;

	private Options options = null;
	private DialogConfirm confirm = null;

	private int version = -1;

	/**
	 * AttacheCase4GUIを構成します。
	 */
	public AttacheCase4GUI() {
		this(new Options());
	}

	/**
	 * AttacheCase4GUIを構成します。
	 * 
	 * @param opt
	 *            オプション
	 */
	public AttacheCase4GUI(Options opt) {
		if (opt == null) {
			opt = new Options();
		}
		options = opt;

		// ダイアログの大きさは固定
		setPreferredSize(new Dimension(440, 320));
		setMinimumSize(new Dimension(440, 320));
		setMaximumSize(new Dimension(440, 320));
		setResizable(false);
		setAlwaysOnTop(options.isAlwaysOnTop());

		frame = this;
		this.confirm = new DialogConfirm(this);

		setTitle(bandle.getString("APP_FRAME_NAME"));

		JMenuBar menuBar = new JMenuBar();
		menuBar.setBounds(0, 0, 132, 22);
		this.setJMenuBar(menuBar);

		JMenu menu = new JMenu(bandle.getString("MENU_FILE"));
		if (!Utility.isMac()) {
			menu.setText(menu.getText() + "(F)");
			menu.setMnemonic(KeyEvent.VK_F);
		}
		menuBar.add(menu);

		JMenuItem encryptMenuItem = new JMenuItem(bandle.getString("MENU_ENCRYPT_FILE"));
		encryptMenuItem.setAction(encryptAction);
		if (!Utility.isMac()) {
			encryptMenuItem.setText(encryptMenuItem.getText() + "(O)");
			encryptMenuItem.setMnemonic(KeyEvent.VK_O);
		}
		encryptMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, AttacheCaseConstant.SHORT_CUT_KEY));
		menu.add(encryptMenuItem);

		JMenuItem decryptMenuItem = new JMenuItem(bandle.getString("MENU_DECRYPT_FILE"));
		decryptMenuItem.setAction(decryptAction);
		if (!Utility.isMac()) {
			decryptMenuItem.setText(menu.getText() + "(D)");
			decryptMenuItem.setMnemonic(KeyEvent.VK_D);
		}
		decryptMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, AttacheCaseConstant.SHORT_CUT_KEY));
		menu.add(decryptMenuItem);

		// Mac以外の場合に表示
		JMenuItem quitMenuItem = new JMenuItem(bandle.getString("MENU_QUIT"));
		if (!Utility.isMac()) {
			// セパレーター
			menu.addSeparator();
			quitMenuItem.setText(menu.getText() + "(C)");
			quitMenuItem.setMnemonic(KeyEvent.VK_C);
			menu.add(quitMenuItem);
		}

		JMenu optionMenu = new JMenu(bandle.getString("MENU_OPTION"));
		if (!Utility.isMac()) {
			optionMenu.setText(optionMenu.getText() + "(O)");
			optionMenu.setMnemonic(KeyEvent.VK_O);
		}
		menuBar.add(optionMenu);

		JMenuItem settingMenuItem = new JMenuItem(bandle.getString("MENU_SETTING"));
		if (!Utility.isMac()) {
			settingMenuItem.setText(settingMenuItem.getText() + "(T)...Ctrl+W");
			settingMenuItem.setMnemonic(KeyEvent.VK_W);
		} else {
			settingMenuItem.setText(settingMenuItem.getText() + "...\u2318+W");
		}
		settingMenuItem.setAction(settingAction);
		settingMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, AttacheCaseConstant.SHORT_CUT_KEY));
		optionMenu.add(settingMenuItem);

		// ウィンドウイベントのハンドリング
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				quit();
			}
		});
		// 画面中央に表示
		setLocationRelativeTo(null);

		Panel panel = new Panel();
		panel.setPreferredSize(new Dimension(300, 120));
		panel.setMinimumSize(new Dimension(300, 120));
		getContentPane().add(panel, BorderLayout.SOUTH);
		panel.setLayout(new GridLayout(3, 1));

		JPanel progressPanel = new JPanel();
		progressPanel.setBackground(Color.WHITE);
		progressPanel.setName("progressPanel");
		progressPanel.setBorder(new EmptyBorder(5, 5, 5, 5)); // プログレスバーの間に隙間を入れるため
		panel.add(progressPanel);
		progressPanel.setLayout(new BorderLayout(0, 0));

		progressBar = new JProgressBar();
		progressBar.setVisible(false);
		progressBar.setStringPainted(true);
		progressPanel.add(progressBar, BorderLayout.CENTER);

		Panel buttonPanel = new Panel();
		buttonPanel.setBackground(Color.WHITE);
		buttonPanel.setName("buttonPanel");
		panel.add(buttonPanel);

		cancelButton = new Button(bandle.getString("MENU_QUIT"));
		cancelButton.addActionListener(quitAction);
		if (!Utility.isMac()) {
			cancelButton.setLabel(cancelButton.getLabel() + "(C)");
			optionMenu.setMnemonic(KeyEvent.VK_C);
		}
		buttonPanel.add(cancelButton);

		Panel statusPanel = new Panel();
		statusPanel.setBackground(UIManager.getColor("Button.background"));
		panel.add(statusPanel);
		statusPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 10));

		statusText = new JLabel("");
		statusText.setPreferredSize(new Dimension(390, 20));
		statusText.setHorizontalAlignment(SwingConstants.LEFT);
		statusPanel.add(statusText);

		ButtonPanel iconButtonPanel = new ButtonPanel();
		// ファイルをドラッグ＆ドロップしたときの処理を追加
		iconButtonPanel.setDrop(new DropTargetAdapter() {
			@Override
			@SuppressWarnings("unchecked")
			public void drop(DropTargetDropEvent dtde) {
				frame.toFront();
				dtde.acceptDrop(DnDConstants.ACTION_COPY);
				Transferable trans = dtde.getTransferable();
				try {
					// LinuxでDrag&Dropに失敗していたので対策する 2013/05/28
					if (dtde.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
						fileList = (List<File>) trans.getTransferData(DataFlavor.javaFileListFlavor);
					} else if (dtde.isDataFlavorSupported(DataFlavor.stringFlavor)) {
						String fileNames = (String) trans.getTransferData(DataFlavor.stringFlavor);
						StringTokenizer st = new StringTokenizer(fileNames, "\n");
						fileList = new ArrayList<File>();
						while (st.hasMoreTokens()) {
							String fileScheme = st.nextToken();
							fileList.add(new File(fileScheme.substring("file://".length()).trim()));
						}
					}
					dtde.dropComplete(fileList.size() != 0);
					run();
				} catch (UnsupportedFlavorException e) {
					// e.printStackTrace();
					confirm.showErrorMessage(e.getMessage());
				} catch (IOException e) {
					// e.printStackTrace();
					confirm.showErrorMessage(e.getMessage());
				}
			}
		});
		iconButtonPanel.setBackground(Color.WHITE);
		getContentPane().add(iconButtonPanel, BorderLayout.CENTER);
		iconButtonPanel.setLayout(new GridLayout(1, 3));

		EmptyBorder emptyBoarder = new EmptyBorder(V_GAP_FOR_ICON, 0, 0, 0);
		// 暗号化
		JPanel encryptPanel = new JPanel();
		encryptPanel.setBackground(Color.WHITE);
		encryptPanel.setBorder(emptyBoarder);
		iconButtonPanel.add(encryptPanel);
		encryptPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

		JLabel encryptLabel = new JLabel(bandle.getString("LABEL_ENCRYPT"));
		encryptLabel.setPreferredSize(new Dimension(100, 20));
		encryptLabel.setHorizontalAlignment(SwingConstants.CENTER);
		encryptPanel.add(encryptLabel);
		IconButton encryptButton = new IconButton("/1b72.png", "/1a72.png") {
			private static final long serialVersionUID = 1L;

			@Override
			public void mouseClicked(MouseEvent arg0) {
				encryptAction.actionPerformed(null);
			}
		};
		encryptButton.setBackground(Color.WHITE);
		encryptPanel.add(encryptButton);

		// 復号
		JPanel decryptPanel = new JPanel();
		decryptPanel.setBackground(Color.WHITE);
		decryptPanel.setBorder(emptyBoarder);
		iconButtonPanel.add(decryptPanel);
		decryptPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

		JLabel decryptLabel = new JLabel(bandle.getString("LABEL_DECRYPT"));
		decryptLabel.setPreferredSize(new Dimension(130, 20));
		decryptLabel.setHorizontalAlignment(SwingConstants.CENTER);
		decryptPanel.add(decryptLabel);

		IconButton decryptButton = new IconButton("/2b72.png", "/2a72.png") {
			private static final long serialVersionUID = -5935519443745942443L;

			@Override
			public void mouseClicked(MouseEvent arg0) {
				decryptAction.actionPerformed(null);
			}
		};
		decryptPanel.add(decryptButton);

		// 動作設定
		JPanel optionPanel = new JPanel();
		optionPanel.setBackground(Color.WHITE);
		optionPanel.setBorder(emptyBoarder);
		iconButtonPanel.add(optionPanel);
		optionPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

		JLabel optionLabel = new JLabel(bandle.getString("LABEL_OPTION"));
		optionLabel.setPreferredSize(new Dimension(100, 20));
		optionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		optionPanel.add(optionLabel);

		IconButton optionButton = new IconButton("/3b72.png", "/3a72.png") {
			private static final long serialVersionUID = -5935519443745942443L;

			@Override
			public void mouseClicked(MouseEvent arg0) {
				settingAction.actionPerformed(null);
			}
		};
		optionPanel.add(optionButton);
	}

	/**
	 * 終了処理
	 */
	protected void quit() {
		// JOptionPane.showMessageDialog(this, "終了します");
		options.saveToFile();
		System.exit(0);
	}

	/**
	 * 暗号化処理を行うためのActionクラスです。
	 * 
	 * @author Masanori Uozumi
	 */
	private class EncryptAction extends AbstractAction {
		private static final long serialVersionUID = 3743129248681047677L;
		private AbstractAction cancelAction = null;

		/**
		 * EncryptActionを構成します。
		 */
		public EncryptAction() {
			putValue(NAME, bandle.getString("MENU_ENCRYPT_FILE"));
			putValue(SHORT_DESCRIPTION, bandle.getString("MENU_ENCRYPT_FILE_SHORT_DESCRIPTION"));
		}

		/**
		 * Actionを実行します。
		 */
		public void actionPerformed(ActionEvent e) {
			// options.setEncryptPassword(null);
			// ファイルがすでに設定されているかどうか。
			// Drag & Drop された場合はすでにファイルが設定されている。
			List<String> fileNameList = new ArrayList<String>();
			if (fileList == null || fileList.size() == 0) {
				// ファイルが設定されていない場合は，ファイル選択ダイアログを開く
				fileList = confirm.getFiles(null, bandle.getString("DIALOG_TITLE_ENCRYPT_FILE"),
						JFileChooser.FILES_ONLY, null);
				// ファイル選択ダイアログをキャンセルで抜けてきた場合
				if (fileList.size() == 0) {
					return;
				}
			}
			for (File f : fileList) {
				fileNameList.add(f.getAbsolutePath());
			}
			options.setEncryptFileNames(fileNameList);
			// 終了ボタンをキャンセルボタンに入れ替え
			cancelAction = new AbstractAction() {
				private static final long serialVersionUID = -1244369977167704186L;

				public void actionPerformed(ActionEvent e) {
					if (encryptWorker != null) {
						encryptWorker.cancel(true);
					}
				}
			};
			cancelButton.removeActionListener(quitAction);
			cancelButton.addActionListener(cancelAction);
			cancelButton.setLabel(bandle.getString("SELECT_OPTION_CANCEL"));
			encryptWorker = new SwingWorker<Void, Integer>() {
				private Encrypt encrypt = null;
				private String oldFileName = "";

				@Override
				protected Void doInBackground() throws Exception {
					progressBar.setVisible(true);
					progressBar.setValue(0);
					// versionに応じて処理を変更
					if (options.isUseVersion2()) {
						encrypt = new FileEncrypt(false, options, confirm);
					} else {
						encrypt = new FileEncrypt3(false, options, confirm);
					}
					setPassword(encrypt);
					encrypt.run(new AbstractProgressListener() {

						@Override
						public void publishProgress(int value) {
							publish(value);
						}

						@Override
						public void publishFileName(String fileName) {
							firePropertyChange(CURRENT_FILE_NAME_KEY, oldFileName, fileName);
							oldFileName = fileName;
						}

						@Override
						public boolean isCancel() {
							return isCancelled();
						}
					});
					return null;
				}

				@Override
				protected void done() {
					if (isCancelled() || encrypt != null && !encrypt.isComplete()) {
						confirm.showMessageByBundle("MSG_STOPPED");
					} else if (encrypt != null && encrypt.isComplete()) {
						// 後処理が必要であれば…
					}
					progressBar.setVisible(false);
					if (fileList != null) {
						fileList = null;
					}
					;
					statusText.setText(bandle.getString("MSG_ENCRYPT_END"));
					// 終了ボタンを元に戻す
					cancelButton.removeActionListener(cancelAction);
					cancelButton.addActionListener(quitAction);
					cancelButton.setLabel(bandle.getString("MENU_QUIT"));
					// 終了するかどうか
					if (options.isQuitAfterEncrypt()) {
						quit();
					}
				}

				@Override
				protected void process(List<Integer> chunks) {
					for (Integer i : chunks) {
						setProgress(i);
					}
				}
			};

			encryptWorker.addPropertyChangeListener(new PropertyChangeListener() {
				@Override
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						progressBar.setValue((Integer) evt.getNewValue());
					} else if (CURRENT_FILE_NAME_KEY.equals(evt.getPropertyName())) {
						statusText.setText("" + evt.getNewValue());
					}
				}
			});
			statusText.setText(bandle.getString("MSG_ENCRYPT_START"));
			encryptWorker.execute();
		}
	}

	/**
	 * 復号処理を行うためのActionです。
	 * 
	 * @author Masanori Uozumi
	 */
	private class DecryptAction extends AbstractAction {
		private static final long serialVersionUID = 8121598904661486893L;
		private AbstractAction cancelAction = null;

		/**
		 * DecryptActionを構成します。
		 */
		public DecryptAction() {
			putValue(NAME, bandle.getString("MENU_DECRYPT_FILE"));
			putValue(SHORT_DESCRIPTION, bandle.getString("MENU_DECRYPT_FILE_SHORT_DESCRIPTION"));
		}

		/**
		 * Actionを実行します。
		 */
		public void actionPerformed(ActionEvent e) {
			if (!frame.isVisible())
				frame.setVisible(true);
			// options.setDecryptPassword(null);
			// ファイルがすでに設定されているかどうか。
			// Drag & Drop された場合はすでにファイルが設定されている。
			List<String> fileNameList = new ArrayList<String>();
			if (fileList == null || fileList.size() == 0) {
				// ファイルが設定されていない場合は，ファイル選択ダイアログを開く
				fileList = confirm.getFiles(null, "ファイル選択", JFileChooser.FILES_ONLY, new AtcFileFilter());
				// ファイル選択ダイアログをキャンセルで抜けてきた場合
				if (fileList.size() == 0) {
					return;
				}
			}
			for (File f : fileList) {
				fileNameList.add(f.getAbsolutePath());
			}
			options.setDecryptFileNames(fileNameList);
			cancelAction = new AbstractAction() {
				private static final long serialVersionUID = -1244369977167704186L;

				public void actionPerformed(ActionEvent e) {
					if (encryptWorker != null) {
						encryptWorker.cancel(true);
					}
				}
			};

			cancelButton.removeActionListener(quitAction);
			cancelButton.addActionListener(cancelAction);
			cancelButton.setLabel(bandle.getString("SELECT_OPTION_CANCEL"));
			encryptWorker = new SwingWorker<Void, Integer>() {
				private Encrypt encrypt = null;
				private String oldFileName = "";

				@Override
				protected Void doInBackground() throws Exception {
					progressBar.setVisible(true);
					progressBar.setValue(0);
					if (version == 2) {
						encrypt = new FileDecrypt2(false, options, confirm);
					} else {
						encrypt = new FileDecrypt3(false, options, confirm);
					}
					encrypt.run(new AbstractProgressListener() {
						@Override
						public void publishProgress(int value) {
							publish(value);
						}

						@Override
						public void publishFileName(String fileName) {
							firePropertyChange(CURRENT_FILE_NAME_KEY, oldFileName, fileName);
							oldFileName = fileName;
						}

						@Override
						public boolean isCancel() {
							return isCancelled();
						}
					});
					return null;
				}

				@Override
				protected void done() {
					if (isCancelled() || (encrypt != null && !encrypt.isComplete())) {
						confirm.showMessageByBundle("MSG_STOPPED");
					} else if (encrypt != null && encrypt.isComplete()) {
						// 後処理が必要であれば…
					}
					progressBar.setVisible(false);
					if (fileList != null) {
						fileList = null;
					}
					;
					statusText.setText(bandle.getString("MSG_DECRYPT_END"));
					// 終了ボタン
					cancelButton.removeActionListener(cancelAction);
					cancelButton.addActionListener(quitAction);
					cancelButton.setLabel(bandle.getString("MENU_QUIT"));
					// 終了するかどうか
					if (options.isQuitAfterEncrypt()) {
						quit();
					}
				}

				@Override
				protected void process(List<Integer> chunks) {
					for (Integer i : chunks) {
						setProgress(i);
					}
				}
			};
			encryptWorker.addPropertyChangeListener(new PropertyChangeListener() {
				@Override
				public void propertyChange(PropertyChangeEvent evt) {
					if ("progress".equals(evt.getPropertyName())) {
						progressBar.setValue((Integer) evt.getNewValue());
					} else if (CURRENT_FILE_NAME_KEY.equals(evt.getPropertyName())) {
						statusText.setText("" + evt.getNewValue());
					}
				}
			});
			statusText.setText(bandle.getString("MSG_DECRYPT_START"));
			encryptWorker.execute();
		}
	}

	/**
	 * 終了処理を行うためのActionです。
	 * 
	 * @author Masanori Uozumi
	 */
	private class QuitAction extends AbstractAction {
		private static final long serialVersionUID = -3178962834401876853L;

		public QuitAction() {
			putValue(NAME, bandle.getString("MENU_QUIT"));
			putValue(SHORT_DESCRIPTION, bandle.getString("MENU_QUIT_DESCRIPTION"));
		}

		/**
		 * Actionを実行します。
		 */
		public void actionPerformed(ActionEvent e) {
			quit();
		}
	}

	/**
	 * 設定を行うためのActionです。
	 * 
	 * @author Masanori Uozumi
	 */
	private class SettingAction extends AbstractAction {
		private static final long serialVersionUID = -1213442520167632698L;

		public SettingAction() {
			putValue(NAME, bandle.getString("MENU_SETTING"));
			putValue(SHORT_DESCRIPTION, bandle.getString("MENU_SETTING_DESCRIPTION"));
		}

		/**
		 * Actionを実行します。
		 */
		public void actionPerformed(ActionEvent e) {
			OptionDialog dialog = new OptionDialog(frame, options);
			dialog.setVisible(true);
			if (!dialog.isCanceled()) {
				options = dialog.getOptions();
			}
		}
	}

	/**
	 * 暗号化時のパスワードを設定します。
	 * 
	 * <pre>
	 * パスワードファイルを利用する設定の場合は，ファイルを読み込んでパスワードを設定します。
	 * 入力する必要がある場合は，キーボードから入力を求めます。
	 * 【注意】このメソッドを利用するのは暗号化の時だけです。
	 * </pre>
	 * 
	 * @param encrypt
	 *            暗号化オブジェクトまたは復号オブジェクト
	 * @throws IOException
	 *             ファイルが存在しない場合
	 * @throws PasswordException
	 *             パスワード入力でエラーが発生した場合
	 */
	public void setPassword(Encrypt encrypt) throws IOException, PasswordException {
		// パスワード設定
		if (options.isEncryptPasswordRequired()) { // パスワード入力が必要
			// 暗号化時のパスワード入力は2回
			String password = confirm.inputPassword(null, 2, options.isShowPassword());
			if (password == null) {
				throw new PasswordException();
			}
			options.setEncryptPasswardRequired(false);
			encrypt.setPassword(password);
		} else {
			if (options.isUsePasswordFile() && options.getEncryptPasswordFileName() != null
					&& options.getEncryptPasswordFileName().length() > 0) { // ファイルが指定されている場合
				Digest digest = new Digest();
				ShaEntity sha = digest.getHash(options.getEncryptPasswordFileName());
				encrypt.setSha(sha);
			} else { // 元々設定されているパスワード
				encrypt.setPassword(options.getEncryptPassword());
			}
		}
	}

	public static AttacheCase4GUI getFrame() {
		return frame;
	}

	public void setFileList(List<File> fileList) {
		this.fileList = fileList;
	}

	public Action getDecryptAction() {
		return decryptAction;
	}

	public Options getOptions() {
		try {
			return options.clone();
		} catch (CloneNotSupportedException e) {
			// 何もしない
		}
		return null;
	}

	public void setOptions(Options options) {
		this.options = options;
	}

	/**
	 * 渡されたファイルを確認して，暗号化または復号の処理を行います。<br>
	 * 渡されたファイルが暗号化ファイルの場合は復号処理を，それ以外の場合は暗号化処理を行います。<br>
	 */
	public void run() {
		if (fileList != null && fileList.size() != 0) {
			version = Utility.getAtcVersion(fileList.get(0));
			// 処理を選択するか，ファイルの種類によって自動実行するか
			if (options.isAskEncryptOrDecrypt()) {
				// ダイアログを表示して選択
				String[] buttons = (Utility.isMac()) ? // MacとWindowsでボタンの並びを変える。
						new String[] { bandle.getString("LABEL_DECRYPT_SHORT"), bandle.getString("LABEL_ENCRYPT"),
								bandle.getString("LABEL_CANCEL") }
						: new String[] { bandle.getString("LABEL_CANCEL"), bandle.getString("LABEL_DECRYPT_SHORT"),
								bandle.getString("LABEL_ENCRYPT") };
				int result = JOptionPane.showOptionDialog(frame, bandle.getString("MSG_SELECT_ACTION"),
						bandle.getString("ENCRYPT_DECRYPT_SELECT_DIALOG"), JOptionPane.YES_NO_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, new ImageIcon(getClass().getResource("/icon_help.png")), buttons,
						(Utility.isMac()) ? buttons[1] : buttons[2]);
				if (result == -1) { // ダイアログをクローズされた場合は何もしない
					return;
				}
				String buttonLabel = buttons[result];
				if (buttonLabel.equals(bandle.getString("LABEL_CANCEL"))) { // キャンセルの場合も何もしない
					return;
				} else {
					boolean mode = buttonLabel.equals(bandle.getString("LABEL_ENCRYPT"));
					if (mode) {
						version = -1;
					}
				}
			}
			if (version == -1) {
				encryptAction.actionPerformed(null);
			} else {
				decryptAction.actionPerformed(null);
			}
		}
	}
}
