/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */

package org.hibara.attachecase;

import java.io.File;
import java.util.ResourceBundle;

/**
 * 確認メソッド用インターフェースです。
 * 
 * @author Masanori Uozumi
 * @version 2012.07.27<br>
 *          2012.08.09 リソースバンドル用変数を追加<br>
 *          2012.08.18 パスワード入力時にファイル名を表示できるようメソッドを追加<br>
 */
public interface Confirm {

	/**
	 * YES
	 */
	int YES = 1;

	/**
	 * NO
	 */
	int NO = -1;

	/**
	 * CANCEL
	 */
	int CANCEL = -2;

	/**
	 * ALL. すべて承認
	 */
	int ALL = 2;

	/**
	 * リソース
	 */
	ResourceBundle resources = ResourceBundle.getBundle(AttacheCaseConstant.RESOURCE);

	/**
	 * ユーザーにYesまたはNoを確認します。
	 * 
	 * @param s
	 *            リソースバンドルまたは表示するメッセージ
	 * @param bundle
	 *            ture の場合は引数 s をリソースバンドルとして扱う
	 * @return YES または NO の値
	 */
	public int confirmYesNo(String s, boolean bundle);

	/**
	 * ユーザーにYes，NoまたはAllを確認します。
	 * 
	 * @param s
	 *            リソースバンドルまたは表示するメッセージ
	 * @param bundle
	 *            true の場合は引数 s をリソースバンドルとして扱う
	 * @return YES または NO または ALL の値
	 */
	public int confirmYesNoAll(String s, boolean bundle);

	/**
	 * ユーザーにYes，NoまたはCancelを確認します。
	 * 
	 * @param s
	 *            リソースバンドルまたは表示するメッセージ
	 * @param bundle
	 *            true の場合は引数 s をリソースバンドルとして扱う
	 * @return YES または NO または Cancel の値
	 */
	public int confirmYesNoCancel(String s, boolean bundle);

	/**
	 * ユーザーにYes，No, All, Cancelを確認します。
	 * 
	 * @param s
	 *            リソースバンドルまたは表示するメッセージ
	 * @param bundle
	 *            true の場合は引数 s をリソースバンドルとして扱う
	 * @return YES または NO または All または Cancel の値
	 */
	public int confirmYesNoAllCancel(String s, boolean bundle);

	/**
	 * パスワードを入力します。
	 * 
	 * <pre>
	 * 入力した文字は画面上に表示されません。
	 * </pre>
	 * 
	 * @param loop
	 *            パスワードの入力回数 1また2
	 * @return 入力された文字列
	 */
	public String inputPassword(int loop);

	/**
	 * ファイル名を表示してパスワードを入力します。
	 * 
	 * <pre>
	 * 入力した文字は画面上に表示されません。
	 * </pre>
	 * 
	 * @param fileName
	 *            ファイル名
	 * @param loop
	 *            入力回数 1または2
	 * @return 入力された文字列
	 */
	public String inputPassword(String fileName, int loop);

	/**
	 * ファイル名を表示してパスワードを入力します。
	 * 
	 * <pre>
	 * 入力した文字が画面上に表示するかどうかは，引数passwordによって設定します
	 * </pre>
	 * 
	 * @param fileName
	 *            ファイル名
	 * @param loop
	 *            入力回数 1または2
	 * @param showPassword
	 *            入力された文字を表示する場合はtrue
	 * @return 入力された文字列
	 */
	public String inputPassword(String fileName, int loop, boolean showPassword);

	/**
	 * ディレクトリ名を入力します。
	 * 
	 * @param f
	 *            ベースとなるファイル情報
	 * @return 入力されたディレクトリ名
	 */
	public String inputDirName(File f);

	/**
	 * ファイル名を入力します。
	 * 
	 * @param f
	 *            ベースとなるファイル情報
	 * @return 入力されたファイル名
	 */
	public String inputFileName(File f);

	/**
	 * リソースバンドルで指定された文字列を表示します。
	 * 
	 * @param bundle
	 *            リソースバンドル
	 */
	public void showMessageByBundle(String bundle);

	/**
	 * リソースバンドルで指定された文字列を標準エラー出力に表示します。
	 * 
	 * @param bundle
	 *            リソースバンドル
	 */
	public void showErrorMessageByBundle(String bundle);

	/**
	 * 指定された文字列を表示します。
	 * 
	 * @param str
	 *            表示する文字列
	 */
	public void showMessage(String str);

	/**
	 * 指定された文字列を標準エラー出力に表示します。
	 * 
	 * @param str
	 *            表示する文字列
	 */
	public void showErrorMessage(String str);

}
