/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */

package org.hibara.attachecase;

import java.io.Console;
import java.io.File;

import org.hibara.attachecase.utility.Utility;

/**
 * コンソールを用いて確認を行う。
 * 
 * @author maruuo
 * @version 2012.08.19<br>
 *          2012.08.18 メソッド追加<br>
 *          2012.08.19 パスワード入力時の回数を指定できるように変更(パスワードの確認のため)<br>
 */

public class ConsoleConfirm implements Confirm, AttacheCaseConstant {

	private static Console console = System.console();
	private static final String YES_NO = "Yes/No";
	private static final String YES_NO_CANCEL = "Yes/No/Cancel";
	private static final String YES_NO_ALL = "Yes/No/All";
	private static final String YES_NO_ALL_CANCEL = "Yes/No/All/Cancel";

	/**
	 * ユーザーにYesまたはNoを確認します。
	 * <p>
	 * 正しく入力されなかった場合，再入力を10回繰り返します。10回以内に正しく入力されなかった場合は，No が返ります。
	 * </p>
	 */
	@Override
	public int confirmYesNo(String s, boolean bundle) {
		String str = inputFromConsole(s, bundle, YES_NO, 10);
		if (str.toUpperCase().startsWith("Y") || str.toUpperCase().equals("YES")) {
			return YES;
		}
		return NO;
	}

	/**
	 * ユーザーにYes，NoまたはCancelを確認します。
	 * <p>
	 * 正しく入力されなかった場合，再入力を10回繰り返します。10回以内に正しく入力されなかった場合は，No が返ります。
	 * </p>
	 */
	@Override
	public int confirmYesNoCancel(String s, boolean bundle) {
		String str = inputFromConsole(s, bundle, YES_NO_CANCEL, 10);
		if (str.toUpperCase().startsWith("Y") || str.toUpperCase().equals("YES")) {
			return YES;
		} else if (str.toUpperCase().startsWith("C") || str.toUpperCase().equals("CANCEL")) {
			return CANCEL;
		}
		return NO;
	}

	/**
	 * ユーザーにYes，NoまたはCancelを確認します。
	 * <p>
	 * 正しく入力されなかった場合，再入力を10回繰り返します。10回以内に正しく入力されなかった場合は，No が返ります。
	 * </p>
	 */
	@Override
	public int confirmYesNoAll(String s, boolean bundle) {
		String str = inputFromConsole(s, bundle, YES_NO_ALL, 10);
		if (str.toUpperCase().startsWith("Y") || str.toUpperCase().equals("YES")) {
			return YES;
		} else if (str.toUpperCase().startsWith("A") || str.toUpperCase().equals("ALL")) {
			return ALL;
		}
		return NO;
	}

	/**
	 * ユーザーにYes，No, AllまたはCancelを確認します。
	 * <p>
	 * 正しく入力されなかった場合，再入力を10回繰り返します。10回以内に正しく入力されなかった場合は，No が返ります。
	 * </p>
	 */
	@Override
	public int confirmYesNoAllCancel(String s, boolean bundle) {
		String str = inputFromConsole(s, bundle, YES_NO_ALL_CANCEL, 10);
		if (str.toUpperCase().startsWith("Y") || str.toUpperCase().equals("YES")) {
			return YES;
		} else if (str.toUpperCase().startsWith("A") || str.toUpperCase().equals("ALL")) {
			return ALL;
		} else if (str.toUpperCase().startsWith("C") || str.toUpperCase().equals("CANCEL")) {
			return CANCEL;
		}
		return NO;
	}

	@Override
	public String inputPassword(int loop) {
		return inputPassword(null, loop);
	}

	@Override
	public String inputPassword(String fileName, int loop) {
		// ループ回数
		if (loop < 0)
			loop = 1;
		if (loop > 2)
			loop = 2;

		String[] inputPasswords = new String[2]; // 入力されたパスワードを格納する配列
		int count = 0;
		// 確認も含めて2回入力
		while (count < loop) {
			StringBuilder sb = new StringBuilder();
			if (fileName != null) {
				sb.append('[').append(fileName).append("] ");
			}
			sb.append(resources.getString((count == 0) ? "MSG_INPUT_PASSWORD" : "MSG_INPUT_PASSWORD_AGAIN"));
			char[] passwordChars = console.readPassword("%s", sb.toString());
			// 未入力の場合は再入力
			if (passwordChars == null || passwordChars.length == 0) {
				continue;
			}
			String password = new String(passwordChars);
			if (count == 1 && !inputPasswords[0].equals(password)) {
				showErrorMessageByBundle("MSG_ERROR_PASSWORD_DIFFERENCE");
			} else {
				inputPasswords[count++] = password;
			}
		}
		return new String(inputPasswords[0]);
	}

	@Override
	public String inputPassword(String fileName, int loop, boolean showPassword) {
		return inputPassword(fileName, loop);
	}

	@Override
	public void showMessageByBundle(String bundle) {
		if (bundle != null) {
			showMessage(resources.getString(bundle));
		}
	}

	@Override
	public void showMessage(String str) {
		console.printf("%s", str);
	}

	@Override
	public void showErrorMessageByBundle(String bundle) {
		if (bundle != null) {
			showErrorMessage(resources.getString(bundle));
		}
	}

	@Override
	public void showErrorMessage(String str) {
		System.err.printf("%s", str);
	}

	/**
	 * コンソールから文字列を入力します。
	 * 
	 * @param s
	 *            メッセージとして表示する文字列。 bundle が true の場合はリソースバンドルとして扱われる
	 * @param bundle
	 *            true の場合，引数 s はリソースバンドルとして扱われます。
	 * @param allowStr
	 *            入力値として許される文字列を指定します。各文字列は /(半角スラッシュ) で区切って指定します。
	 * @param loop
	 *            再入力の回数を指定します。マイナスの値が指定された場合は，事実上無制限に繰り返すことになります。
	 * @return 入力した文字列
	 */
	private String inputFromConsole(String s, boolean bundle, String allowStr, int loop) {
		if (loop < 0) {
			loop = Integer.MAX_VALUE;
		}
		String msg = allowStr + " : ";
		if (bundle && s != null) {
			s = resources.getString(s);
		}
		if (s != null) {
			msg = s + msg;
		}
		String[] allow = allowStr.split("/");
		if (loop < 0) {
			loop = Integer.MAX_VALUE;
		}
		String inputStr = "";
		for (int i = 0; i < loop; i++) {
			inputStr = console.readLine("%s", msg);
			if (isContained(allow, inputStr)) {
				break;
			}
		}
		return inputStr;
	}

	/**
	 * 指定した文字列が， 配列に含まれているかどうかを判定します。
	 * 
	 * @param array
	 *            比較される文字列
	 * @param s
	 *            比較する文字列
	 * @return array に s が含まれている場合には true
	 */
	private boolean isContained(String[] array, String s) {
		boolean contained = false;
		if (s.length() == 0) {
			return contained;
		}
		for (int i = 0; i < array.length; i++) {
			String upper = array[i].toUpperCase();
			if (upper.equals(s.toUpperCase()) || upper.startsWith(s.toUpperCase())) {
				contained = true;
				break;
			}
		}
		return contained;
	}

	@Override
	/**
	 * ディレクトリ名を入力します。
	 * <p>
	 * 入力された名前のディレクトリがすでに存在している場合は，再入力を促します。
	 * </p>
	 */
	public String inputDirName(File f) {
		String path = null;
		while (path == null) {
			path = console.readLine("%s", resources.getString(MSG_INPUT_NEW_DIR));
			if (path.length() == 0 || path.equals("\n")) {
				path = null;
				continue;
			}
			path = Utility.includeTrailingPathDelimiter(path);
			File file = new File(path);
			if (file.exists() || file.isFile()) {
				String msg = resources.getString((file.isFile()) ? MSG_ERROR_NO_DIR_NAME : MSG_ERROR_DIR_EXIST);
				if (msg.endsWith("\n")) {
					msg = msg.substring(0, msg.indexOf('\n') - 1) + resources.getString(MSG_INPUT_NEW_DIR);
				}
				console.printf("%s", msg);
				path = null;
			} else {
				path = file.getAbsolutePath();
			}
		}
		return path;
	}

	/**
	 * ファイル名を入力します。
	 * <p>
	 * 入力された名前のファイルがすでに存在している場合は，再入力を促します。
	 * </p>
	 */
	@Override
	public String inputFileName(File f) {
		String path = null;
		while (path == null) {
			path = console.readLine("%s", resources.getString(MSG_INPUT_NEW_FILE));
			path = Utility.rejectPathTraversal(path);
			if (path.length() == 0 || path.equals("\n")) {
				path = null;
				continue;
			}
			File file = new File(path);
			if (file.exists() || file.isDirectory()) {
				String msg = resources.getString((file.isDirectory()) ? MSG_ERROR_NO_FILE_NAME : MSG_ERROR_FILE_EXIST);
				if (msg.endsWith("\n")) {
					msg = msg.substring(0, msg.indexOf('\n') - 1) + resources.getString(MSG_INPUT_NEW_FILE);
				}
				console.printf("%s", msg);
				path = null;
			} else {
				path = file.getAbsolutePath();
			}
		}
		return path;
	}
}
