/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.hibara.attachecase.io.FileDeletion;
import org.hibara.attachecase.utility.Utility;
import org.ini4j.Wini;

import com.jgoodies.binding.beans.Model;
import com.townleyenterprises.command.CommandOption;
import com.townleyenterprises.command.CommandParser;
import com.townleyenterprises.command.DefaultCommandListener;
import com.townleyenterprises.command.RepeatableCommandOption;

/**
 * このクラスは，アタッシェケースの動作をコントロールするための情報を保持します。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.20<br>
 *          2012.08.04 不正なオプションを指定すると，そこで処理が止まるように変更。<br>
 *          2012.08.10 圧縮レートオプションを追加。<br>
 *          2012.08.11
 *          サイレントモード(処理メッセージを表示しないモード。CUIの場合にのみ有効)追加。暗号化するファイルを複数指定可能に変更<br>
 *          2012.08.18 復号オプションも複数指定できるように変更。<br>
 *          2012.08.23 JGoodies Modelクラスのサブクラスに変更。<br>
 *          2013.03.12 パスワードオプションが指定されたかどうかを判定するための変数を追加。<br>
 *          2014.04.20 パスワードをBase64でエンコードして保存するように変更。<br>
 *          2014.07.24 暗号化および復号用パスワードを保存する処理のバグを修正。<br>
 *          2017.01.20 暗号化の際のファイル形式指定を追加（アタッシェケース#3に対応）。<br>
 */
public class Options extends Model implements AttacheCaseConstant, Cloneable {
	private static final long serialVersionUID = 5622094508320788533L;

	// リソース取得
	private static ResourceBundle resources = Confirm.resources;

	/*
	 * 暗号化オプション
	 */
	private RepeatableCommandOption encryptOption = new RepeatableCommandOption("encrypt", 'e',
			resources.getString("HELP_ENCRYPT_OPTION"), resources.getString("HELP_ENCRYPT_MSG"), true, null) {
		@Override
		@SuppressWarnings("unchecked")
		public void execute() throws Exception {
			setEncryptFileNames((List<String>) getArgs());
			if (runMode == 0) {
				runMode = ENCRYPT_MODE;
			}
		}
	};

	/*
	 * 復号オプション
	 */
	private RepeatableCommandOption decryptOption = new RepeatableCommandOption("decrypt", 'd',
			resources.getString("HELP_DECRYPT_OPTION"), resources.getString("HELP_DECRYPT_MSG"), true, null) {
		@Override
		@SuppressWarnings("unchecked")
		public void execute() throws Exception {
			setDecryptFileNames((List<String>) getArgs());
			runMode = DECRYPT_MODE; // 復号処理を優先（暗号化と復号が同時に指定されるかも…）
		}
	};

	/*
	 * パスワードオプション
	 */
	private CommandOption passwordOption = new CommandOption("password", 'p', true,
			resources.getString("HELP_PASSWORD_OPTION"), resources.getString("HELP_PASSWORD_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			setDecryptPassword(this.getArg());
			setEncryptPassword(this.getArg());
			decryptPasswordRequired = encryptPasswordRequired = false;
			usePasswordOption = true;
		}
	};

	/*
	 * パスワードファイルオプション
	 */
	private CommandOption passwordFileOption = new CommandOption("passwordfile", 'P', true,
			resources.getString("HELP_PASSWORD_FILE_OPTION"), resources.getString("HELP_PASSWORD_FILE_MSG"), true,
			null) {
		@Override
		public void execute() throws Exception {
			encryptPasswordFileName = decryptPasswordFileName = getArg();
			encryptPasswordRequired = decryptPasswordRequired = false;
		}
	};

	/*
	 * 削除オプション
	 */
	private CommandOption deleteOption = new CommandOption("delete", 'D', true,
			resources.getString("HELP_DELETE_OPTION"), resources.getString("HELP_DELETE_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			String arg = getArg();
			try {
				int value = Integer.parseInt(arg);
				if (value >= FileDeletion.NO_DELETE && value <= FileDeletion.TRASH_DELETE) {
					deleteMode = value;
					deleteOriginalFile = deleteEncryptoFile = true;
				}
			} catch (NumberFormatException e) {
				deleteMode = FileDeletion.NO_DELETE;
				deleteOriginalFile = deleteEncryptoFile = false;
			}
		}
	};

	/*
	 * 上書きオプション
	 */
	private CommandOption overwriteOption = new CommandOption("overwrite", 'o', false,
			resources.getString("HELP_OVERWRITE_OPTION"), resources.getString("HELP_OVERWRITE_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			confirmOverwirte = false;
		}
	};

	/*
	 * ランダム値の書き込み回数
	 */
	private CommandOption randomOption = new CommandOption("random", 'R', true,
			resources.getString("HELP_RANDOM_OPTION"), resources.getString("HELP_RANDOM_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			String arg = getArg();
			if (arg == null || arg.length() == 0) {
				throw new Exception();
			}
			try {
				int value = Integer.parseInt(arg);
				randomWriteCount = (value > 0) ? value : 1;
			} catch (NumberFormatException e) {
				randomWriteCount = 1;
				throw e;
			}
		}
	};

	/*
	 * 0の書き込み回数
	 */
	private CommandOption zeroOption = new CommandOption("zero", 'Z', true, resources.getString("HELP_ZERO_OPTION"),
			resources.getString("HELP_ZERO_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			String arg = getArg();
			try {
				int value = Integer.parseInt(arg);
				zeroWriteCount = (value > 0) ? value : 1;
			} catch (NumberFormatException e) {
				zeroWriteCount = 1;
			}
		}
	};

	/*
	 * CUI動作を設定
	 */
	private CommandOption cuiOption = new CommandOption("cui", 'c', false, resources.getString("HELP_CUI_OPTION"),
			resources.getString("HELP_CUI_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			cui = true;
		}
	};

	/*
	 * Versionを表示
	 */
	private CommandOption versionOption = new CommandOption("version", 'v', false,
			resources.getString("HELP_VERSION_OPTION"), resources.getString("HELP_VERSION_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			cui = true;
			runMode = VERSION_MODE;
		}
	};

	/*
	 * 暗号化時にバージョン2形式で暗号化する
	 */
	private CommandOption useVersion2Option = new CommandOption("version2", '2', false,
			resources.getString("HELP_VERSION2_OPTION"), resources.getString("HELP_VERSION2_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			cui = true;
			useVersion2 = true;
		}
	};

	/*
	 * 圧縮率の設定
	 */
	private CommandOption compressRateOption = new CommandOption("compress", 'C', true,
			resources.getString("HELP_COMPRESS_OPTION"), resources.getString("HELP_COMPRESS_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			String arg = getArg();
			try {
				int value = Integer.parseInt(arg);
				compressRate = (value >= 0 && value < 10) ? value : Z_DEFAULT_COMPRESSION;
			} catch (NumberFormatException e) {
				compressRate = Z_DEFAULT_COMPRESSION;
			}
		}
	};

	/*
	 * Silentモードの設定（コンソールに何も表示しない）
	 */
	private CommandOption silentOption = new CommandOption("silent", 's', false,
			resources.getString("HELP_SILENT_OPTION"), resources.getString("HELP_SILENT_MSG"), true, null) {
		@Override
		public void execute() throws Exception {
			silent = true;
		}
	};

	/*
	 * 隠し属性のファイルを含めるかどうか
	 */
	private CommandOption hiddenIncludeOption = new CommandOption("hidden", 'h', false,
			resources.getString("HELP_HIDDEN_INCLUDE_OPTION"), resources.getString("HELP_HIDDEN_INCLUDE_MSG"), true,
			null) {
		@Override
		public void execute() throws Exception {
			includeHidden = true;
		}
	};

	/*
	 * 暗号化時のフォルダ指定 指定しなければ暗号ファイルのフォルダ
	 */
	private CommandOption saveFolderOption = new CommandOption("saveTo", 'S', true,
			resources.getString("HELP_DESTINATION_FOLDER_OPTION"), resources.getString("HELP_DESTINATION_FOLDER_MSG"),
			true, null) {
		@Override
		public void execute() throws Exception {
			String arg = getArg();
			encodeToSameFolder = true;
			encodeFolderPath = arg;
		}
	};

	/*
	 * 復号時のフォルダ指定 指定しなければ暗号ファイルのフォルダ
	 */

	/*
	 * オプション設定用配列
	 */
	private CommandOption[] commandOptions = { encryptOption, decryptOption, passwordOption, passwordFileOption,
			deleteOption, overwriteOption, randomOption, zeroOption, cuiOption, versionOption, compressRateOption,
			silentOption, hiddenIncludeOption, saveFolderOption, useVersion2Option };
	// アプリケーション情報
	private String appPath; // 本体がインストールされたフルパス
	private int versionNum; // バージョン
	private boolean changed = false; // 変更されたかどうか
	private boolean usePasswordOption = false; // パスワードオプションが指定されたかどうか

	// 暗号化が必要なプロパティ
	private static String[] ENCRYPT_PROPERTIES = { "decryptPassword", "encryptPassword", };

	// プロパティ情報
	private static String[][] PROPERTIES = {
			{ "BASIC", "memoryEncryptPassword", "encryptPassword", "memoryDecryptPassword", "decryptPassword",
					"encryptWithoutConfirmation", "openDirectoryAfterDecrypt", "openFile", "quitAfterEncrypt",
					"alwaysOnTop", "showPassword", "askEncryptOrDecrypt", "notMulipleStarts", "includeHidden",
					"useVersion2" },
			{ "SAVE", "encodeToSameFolder", "encodeFolderPath", "decodeToSameFolder", "decodeFolderPath",
					"confirmOverwirte", "allFilePacked", "notMakeParentFolder", "keepTimestamp", "timestampNow",
					"includeExtension", "autoNameFormatText", "addFileName", },
			{ "DELETE", "deleteOriginalFile", "deleteEncryptoFile", "deleteMode", "randomWriteCount",
					"zeroWriteCount" },
			{ "ADVANCED", "usePasswordFile", "encryptPasswordFileName", "decryptPasswordFileName", "fileBrakeOption",
					"maxMistake", "camouflageExtention", "extensionString", "compressRate", } };

	// 【動作設定】
	private List<String> encryptFileNames = null;
	private List<String> decryptFileNames = null;
	private int runMode = 0; // 1:復号，2:暗号化
	private boolean cui = DEFAULT_MODE; // CUI or GUI
	private boolean silent = false; // サイレントモード
	private boolean includeHidden = false; // 暗号化する際に隠し属性のファイルを含めるかどうか
	private boolean quitAfterEncrypt = true;// 処理後，アタッシェケースを終了するかどうか
	private boolean alwaysOnTop = true; // デスクトップで最前面にウィンドウを表示するかどうか
	private boolean openDirectoryAfterDecrypt = false; // ディレクトリの場合，複合後に開くかどうか
	private boolean askEncryptOrDecrypt = false; // 暗号/復号処理かを問い合わせる
	private boolean notMulipleStarts = true;// 複数起動を許さない
	private boolean showPassword = true; // パスワードを*で隠さずに表示する
	private boolean useVersion2 = false; // 暗号化時にバージョン2形式を使用

	// 【基本設定】
	private String encryptPassword; // 暗号化用パスワード
	private String decryptPassword; // 復号用パスワード
	private boolean memoryEncryptPassword = false; // 暗号化用パスワードを記憶するかどうか
	private boolean memoryDecryptPassword = false; // 復号用パスワードを記憶するかどうか
	private boolean encryptWithoutConfirmation = false; // 記憶しているパスワードで確認なく実行する。パスワードファイル設定時はそちらを優先
	private boolean encryptPasswordRequired = true; // 暗号化用パスワードの入力が必要かどうか
	private boolean decryptPasswordRequired = true; // 復号用パスワードの入力が必要かどうか
	private boolean openFile = false; // 復号したファイルを関連付けされたソフトで開く

	// 【保存設定】
	private boolean decodeToSameFolder = false; // 常に同じ場所へファイルを復号するか
	private String decodeFolderPath = null; // その保存場所
	private boolean encodeToSameFolder = false; // 常に同じ場所へファイルを暗号化するか
	private String encodeFolderPath = null; // その保存場所
	private boolean confirmOverwirte = true; // 同名ファイルの上書きを確認するか
	private boolean timestampNow = false; // 復号したファイルのタイムスタンプを復号した日時にする
	private boolean keepTimestamp = true; // 暗号化ファイルのタイムスタンプを元ファイルに合わせる
	private boolean allFilePacked = false; // 複数ファイルが指定されたときにまとめて暗号化するかどうか
	private boolean notMakeParentFolder = false;// 親フォルダを作成しない
	private boolean includeExtension = false; // 暗号化ファイルに元ファイルの拡張子を含める
	private boolean addFileName = false; // 自動でファイル名に付加するかどうか
	private String autoNameFormatText; // 自動で付加するファイル名書式

	// 【削除設定】
	private boolean deleteOriginalFile = false; // 暗号化した後、元ファイルを削除する
	private boolean deleteEncryptoFile = false; // 復号化した後、暗号化ファイルを削除する
	private int deleteMode = FileDeletion.NO_DELETE; // 削除モード
	private int randomWriteCount = 1; // 乱数の書き込み回数
	private int zeroWriteCount = 1; // ゼロの書き込み回数

	// 【動作設定】
	private int compressRate = Z_DEFAULT_COMPRESSION; // 圧縮率

	// 【高度な設定】
	private boolean usePasswordFile = false;
	private String encryptPasswordFileName = null; // 暗号化用パスワードファイル
	private String decryptPasswordFileName = null; // 復号用パスワードファイル
	private boolean fileBrakeOption = false; // ミスタイプでファイルを破壊するか否か
												// 暗号化ファイルに書き込まれる
	private int maxMistake = 3; // パスワードの入力ミス許容回数 暗号化ファイルに書き込まれる
	private boolean camouflageExtention = false;// 拡張子を偽装するかどうか
	private String extensionString = ".jpg"; // 偽装拡張子

	/**
	 * Optionオブジェクトを構成します。
	 * 起動ディレクトリに設定ファイル"attachecase.ini"が存在していれば，設定ファイルを読み込みます。<br>
	 */
	public Options() {
		super();
		addChangeListener();
		loadFromFile();
	}

	/**
	 * コマンドライン引数を渡してOptionオブジェクトを構成します。<br>
	 * 起動ディレクトリに設定ファイル"attachecase.ini"が存在していれば，設定ファイルをまず読み込みます。<br>
	 * その後，コマンドオプションの指定を設定します。つまりコマンドオプションの指定は，設定ファイルの指定より優先されます。
	 * 
	 * @param args
	 *            コマンドライン引数
	 */
	public Options(String[] args) {
		loadFromFile();
		CommandParser parser = new CommandParser("Attachecase");
		parser.setExitOnMissingArg(true, -1); // パラメータ未指定の場合はエラーで終了
		parser.setAbortExecuteOnError(false);
		parser.addCommandListener(new DefaultCommandListener("options", commandOptions));
		parser.parse(args); // コマンドオプション解析
		String[] largs = parser.getUnhandledArguments();

		// オプションとして解析できなかったコマンドライン引数
		for (String s : largs) {
			System.err.println("Unknown command-line argument: " + s);
		}
		// 解析できなかった引数がある場合には終了
		if (largs.length > 0) {
			System.exit(1);
		}

		// 存在しないオプションが指定されているかどうかをチェック。
		if (!validateOptions(args)) {
			System.exit(-1);
		}

		try {
			// コマンドライン・オプションを変数に設定
			parser.executeCommands();
		} catch (Exception e) {
			// 例外発生時は終了
			System.exit(-1);
		}
		addChangeListener();
	}

	/**
	 * プロパティが変更されたなら変数changedをtrueに変更するためのPropertyChangeListenerを追加します。
	 */
	public void addChangeListener() {
		this.addPropertyChangeListener(new PropertyChangeListener() {
			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				if (!changed) {
					Object oldValue = evt.getOldValue();
					Object newValue = evt.getNewValue();
					if (oldValue != null && newValue != null) {
						changed = !oldValue.equals(newValue);
					} else if (oldValue == null && newValue == null) {
						changed = false;
					} else {
						changed = true;
					}
				}
			}
		});
	}

	public boolean isIncludeHidden() {
		return includeHidden;
	}

	public boolean isSilent() {
		return silent;
	}

	public void setSilent(boolean silent) {
		boolean oldValue = this.silent;
		this.silent = silent;
		firePropertyChange("silent", oldValue, this.silent);
	}

	public boolean isIncludeExtension() {
		return includeExtension;
	}

	public void setIncludeExtension(boolean includeExtension) {
		boolean oldValue = this.includeExtension;
		this.includeExtension = includeExtension;
		firePropertyChange("includeExtension", oldValue, includeExtension);
	}

	public boolean isAllFilePacked() {
		return allFilePacked;
	}

	public void setAllFilePacked(boolean allFilePacked) {
		boolean oldValue = this.allFilePacked;
		this.allFilePacked = allFilePacked;
		firePropertyChange("allFilePacked", oldValue, this.allFilePacked);
	}

	public String getAutoNameFormatText() {
		return autoNameFormatText;
	}

	public void setAutoNameFormatText(String autoNameFormatText) {
		String oldValue = this.autoNameFormatText;
		this.autoNameFormatText = autoNameFormatText;
		firePropertyChange("autoNameFormatText", oldValue, this.autoNameFormatText);
	}

	public boolean isCamouflageExtention() {
		return camouflageExtention;
	}

	public void setCamouflageExtention(boolean camouflageExtention) {
		boolean oldValue = this.camouflageExtention;
		this.camouflageExtention = camouflageExtention;
		firePropertyChange("camouflageExtention", oldValue, this.camouflageExtention);
	}

	public String getExtensionString() {
		return extensionString;
	}

	public void setExtensionString(String extensionString) {
		String oldValue = this.extensionString;
		this.extensionString = extensionString;
		firePropertyChange("extensionString", oldValue, this.extensionString);
	}

	public int getCompressRate() {
		return compressRate;
	}

	public void setCompressRate(int compressRate) {
		if (compressRate < Z_MIN_COMPRESSION) {
			compressRate = Z_MIN_COMPRESSION;
		} else if (compressRate > Z_MAX_COMPRESSION) {
			compressRate = Z_MAX_COMPRESSION;
		}
		int oldValue = this.compressRate;
		this.compressRate = compressRate;
		firePropertyChange("compressRate", oldValue, this.compressRate);
	}

	public void setDecryptPasswardRequired(boolean passwardRequired) {
		boolean oldValue = this.decryptPasswordRequired;
		this.decryptPasswordRequired = passwardRequired;
		firePropertyChange("decryptPasswordRequired", oldValue, this.decryptPasswordRequired);
	}

	public void setEncryptPasswardRequired(boolean passwardRequired) {
		boolean oldValue = this.encryptPasswordRequired;
		this.encryptPasswordRequired = passwardRequired;
		firePropertyChange("encryptPasswordRequired", oldValue, this.encryptPasswordRequired);
	}

	public List<String> getEncryptFileNames() {
		return encryptFileNames;
	}

	public void setEncryptFileNames(List<String> fileNames) {
		this.encryptFileNames = fileNames;
	}

	public void setEncryptFilenames(String fileName) {
		if (encryptFileNames == null) {
			encryptFileNames = new ArrayList<String>();
		}
		// 引数がnullの場合は何もしない
		if (fileName == null) {
			return;
		}
		encryptFileNames.add(fileName);
	}

	public void setDecryptFileNames(List<String> fileNames) {
		this.decryptFileNames = fileNames;
	}

	public void setDecryptFilenames(String fileName) {
		if (decryptFileNames == null) {
			decryptFileNames = new ArrayList<String>();
		}
		decryptFileNames.add(fileName);
	}

	public List<String> getDecryptFileNames() {
		return decryptFileNames;
	}

	public boolean isDeleteOriginalFile() {
		return deleteOriginalFile;
	}

	public void setDeleteOriginalFile(boolean deleteOriginalFile) {
		boolean oldValue = this.deleteOriginalFile;
		this.deleteOriginalFile = deleteOriginalFile;
		firePropertyChange("deleteOriginalFile", oldValue, this.deleteOriginalFile);
	}

	public boolean isDeleteEncryptoFile() {
		return deleteEncryptoFile;
	}

	public void setDeleteEncryptoFile(boolean deleteEncryptoFile) {
		boolean oldValue = this.deleteEncryptoFile;
		this.deleteEncryptoFile = deleteEncryptoFile;
		firePropertyChange("deleteEncryptoFile", oldValue, this.deleteEncryptoFile);
	}

	public boolean isTimestampNow() {
		return timestampNow;
	}

	public void setTimestampNow(boolean timestampNow) {
		boolean oldValue = this.timestampNow;
		this.timestampNow = timestampNow;
		firePropertyChange("timestampNow", oldValue, this.timestampNow);
	}

	public boolean isKeepTimestamp() {
		return keepTimestamp;
	}

	public void setKeepTimestamp(boolean keepTimeStamp) {
		boolean oldValue = this.keepTimestamp;
		this.keepTimestamp = keepTimeStamp;
		firePropertyChange("keepTimestamp", oldValue, this.keepTimestamp);
	}

	public boolean isConfirmOverwirte() {
		return confirmOverwirte;
	}

	public void setConfirmOverwirte(boolean confirmOverwirte) {
		boolean oldValue = this.confirmOverwirte;
		this.confirmOverwirte = confirmOverwirte;
		firePropertyChange("confirmOverwirte", oldValue, this.confirmOverwirte);
	}

	public boolean isOpenFile() {
		return openFile;
	}

	public void setOpenFile(boolean openFile) {
		boolean oldValue = this.openFile;
		this.openFile = openFile;
		firePropertyChange("openFile", oldValue, this.openFile);
	}

	public String getAppPath() {
		return appPath;
	}

	public void setAppPath(String appPath) {
		String oldValue = this.appPath;
		this.appPath = appPath;
		firePropertyChange("appPath", oldValue, this.appPath);
	}

	public int getVersionNum() {
		return versionNum;
	}

	public void setVersionNum(int versionNum) {
		int oldValue = this.versionNum;
		this.versionNum = versionNum;
		firePropertyChange("versionNum", oldValue, this.versionNum);
	}

	public String getEncryptPassword() {
		return encryptPassword;
	}

	/**
	 * 暗号化用パスワードを暗号化して取得する
	 * 
	 * @return 暗号化されたパスワード
	 */
	public String getEncryptPasswordByEncryption() {
		if (encryptPassword == null) {
			return null;
		}
		return Utility.encrypt(Confirm.resources.getString("APP_FRAME_NAME"), encryptPassword);
	}

	public String getDecryptPassword() {
		return decryptPassword;
	}

	/**
	 * 復号用パスワードを暗号化して取得する
	 * 
	 * @return 暗号化された復号用パスワード
	 */
	public String getDecryptPasswordByEncryption() {
		if (decryptPassword == null) {
			return null;
		}
		return Utility.encrypt(Confirm.resources.getString("APP_FRAME_NAME"), decryptPassword);
	}

	public void setEncryptPassword(String password) {
		// null または空文字の場合はパスワードを再入力
		if (password == null || password.length() == 0) {
			setEncryptPasswordToNull(password);
			encryptPasswordRequired = true;
			return;
		}
		String oldValue = this.encryptPassword;
		encryptPassword = password;
		encryptPasswordRequired = false;
		firePropertyChange("encryptPassword", oldValue, this.encryptPassword);
	}

	public void setEncryptPasswordToNull(String password) {
		String oldValue = encryptPassword;
		encryptPassword = password;
		firePropertyChange("encryptPassword", oldValue, encryptPassword);
	}

	/**
	 * 暗号化された暗号化用パスワードが与えられるので，復号して格納する
	 * 
	 * @param encryptedPassword
	 *            暗号化された暗号化用パスワード
	 */
	public void setEncryptPasswordByEncryption(String encryptedPassword) {
		String password = Utility.decrypt(Confirm.resources.getString("APP_FRAME_NAME"), encryptedPassword);
		setEncryptPassword(password);
	}

	public void setDecryptPassword(String password) {
		// null または空文字の場合はパスワードを再入力
		if (password == null || password.length() == 0) {
			setDecryptPasswordToNull(password);
			setDecryptPasswordRequired(true);
			return;
		}
		String oldValue = decryptPassword;
		decryptPassword = password;
		setDecryptPasswordRequired(false);
		firePropertyChange("decryptPassword", oldValue, decryptPassword);
	}

	public void setDecryptPasswordToNull(String password) {
		String oldValue = decryptPassword;
		decryptPassword = password;
		firePropertyChange("decryptPassword", oldValue, decryptPassword);
	}

	/**
	 * 暗号化された復号用パスワードが与えられるので，復号して格納する
	 * 
	 * @param decryptedPassword
	 *            暗号化された復号用パスワード
	 */
	public void setDecryptPasswordByEncryption(String decryptedPassword) {
		String password = Utility.decrypt(Confirm.resources.getString("APP_FRAME_NAME"), decryptedPassword);
		setDecryptPassword(password);
	}

	public boolean isDecodeToSameFolder() {
		return decodeToSameFolder;
	}

	public void setDecodeToSameFolder(boolean decodeToSameFolder) {
		boolean oldValue = this.decodeToSameFolder;
		this.decodeToSameFolder = decodeToSameFolder;
		firePropertyChange("decodeToSameFolder", oldValue, this.decodeToSameFolder);
	}

	public String getDecodeFolderPath() {
		return decodeFolderPath;
	}

	public void setDecodeFolderPath(String decodeFolderPath) {
		String oldValue = this.decodeFolderPath;
		this.decodeFolderPath = decodeFolderPath;
		firePropertyChange("decodeFolderPath", oldValue, this.decodeFolderPath);
	}

	public int getRunMode() {
		return runMode;
	}

	public void setRunMode(int runMode) {
		this.runMode = runMode;
	}

	public String getEncryptPasswordFileName() {
		return encryptPasswordFileName;
	}

	public String getDecryptPasswordFileName() {
		return decryptPasswordFileName;
	}

	public void setDecryptPasswordFileName(String passwordFileName) {
		String oldValue = this.decryptPasswordFileName;
		this.decryptPasswordFileName = passwordFileName;
		setDecryptPasswardRequired(decryptPasswordFileName != null);
		firePropertyChange("decryptPasswordFileName", oldValue, this.decryptPasswordFileName);
	}

	public void setEncryptPasswordFileName(String passwordFileName) {
		String oldValue = this.encryptPasswordFileName;
		this.encryptPasswordFileName = passwordFileName;
		setEncryptPasswardRequired(encryptPasswordFileName != null);
		firePropertyChange("encryptPasswordFileName", oldValue, this.encryptPasswordFileName);
	}

	public int getDeleteMode() {
		return deleteMode;
	}

	public void setDeleteMode(int deleteMode) {
		int oldValue = this.deleteMode;
		this.deleteMode = deleteMode;
		firePropertyChange("deleteMode", oldValue, this.deleteMode);
	}

	public int getRandomWriteCount() {
		return randomWriteCount;
	}

	public void setRandomWriteCount(int randomWriteCount) {
		int oldValue = this.randomWriteCount;
		this.randomWriteCount = randomWriteCount;
		firePropertyChange("randomWriteCount", oldValue, this.randomWriteCount);
	}

	public int getZeroWriteCount() {
		return zeroWriteCount;
	}

	public void setZeroWriteCount(int zeroWriteCount) {
		int oldValue = this.zeroWriteCount;
		this.zeroWriteCount = zeroWriteCount;
		firePropertyChange("zeroWriteCount", oldValue, this.zeroWriteCount);
	}

	public boolean isCui() {
		return cui;
	}

	public boolean isFileBrakeOption() {
		return fileBrakeOption;
	}

	public void setFileBrakeOption(boolean fileBrakeOption) {
		boolean oldValue = this.fileBrakeOption;
		this.fileBrakeOption = fileBrakeOption;
		firePropertyChange("fileBrakeOption", oldValue, this.fileBrakeOption);
	}

	public int getMaxMistake() {
		return maxMistake;
	}

	public void setMaxMistake(int maxMistake) {
		int oldValue = this.maxMistake;
		this.maxMistake = maxMistake;
		firePropertyChange("maxMistake", oldValue, this.maxMistake);
	}

	/**
	 * 正しいオプションかどうかを判定します。 te-code command
	 * でパースすると，不正なオプションを指定してもメッセージが出るだけでストップしないので，ストップさせるために追加しました。
	 * 
	 * @param args
	 * @return
	 */
	private boolean validateOptions(String[] args) {
		ArrayList<String> optionArgs = new ArrayList<String>();
		// オブション指定だけを取り出す
		for (String s : args) {
			if (s.startsWith("--") || s.startsWith("-")) {
				optionArgs.add(s);
			}
		}
		boolean valid = true;
		for (String s : optionArgs) {
			if (!valid) {
				break;
			}
			if (s.startsWith("--")) {
				String arg = s.substring(2);
				valid = isValidOption(arg);
			} else {
				String arg = s.substring(1);
				for (int i = 0; i < arg.length(); i++) {
					valid = isValidOption(arg.charAt(i), i == 0);
				}
			}
		}
		return valid;
	}

	/**
	 * 指定したオプションが正しいオプションかどうか
	 * 
	 * @param arg
	 *            オプション文字列
	 * @return 正しい場合はtrue
	 */
	private boolean isValidOption(String arg) {
		boolean valid = false;
		for (CommandOption opt : commandOptions) {
			if (arg.equals(opt.getName())) {
				valid = true;
				break;
			}
		}
		return valid;
	}

	/**
	 * 指定したオプションが正しいオプションかどうか
	 * 
	 * @param arg
	 *            オプション文字列
	 * @param top
	 *            先頭かどうか
	 * @return 正しい場合はtrue
	 */
	private boolean isValidOption(char arg, boolean top) {
		boolean valid = false;
		for (CommandOption opt : commandOptions) {
			if (top) {
				if (arg == opt.getShortName()) {
					valid = true;
					break;
				}
			} else {
				if (arg == opt.getShortName() && !opt.getExpectsArgument()) {
					valid = true;
					break;
				}
			}
		}
		return valid;
	}

	public boolean isNotMulipleStarts() {
		return notMulipleStarts;
	}

	public void setNotMulipleStarts(boolean notMulipleStarts) {
		boolean oldValue = this.notMulipleStarts;
		this.notMulipleStarts = notMulipleStarts;
		firePropertyChange("notMulipleStarts", oldValue, this.notMulipleStarts);
	}

	public boolean isEncryptPasswordRequired() {
		// パスワードファイル使用時は入力する必要なし
		if (usePasswordFile && encryptPasswordFileName != null && encryptPasswordFileName.length() > 0) {
			return false;
		}
		if (memoryEncryptPassword && encryptPassword != null && encryptPassword.length() > 0) {
			return false;
		}
		return encryptPasswordRequired;
	}

	public void setEncryptPasswordRequired(boolean encryptPasswordRequired) {
		boolean oldValue = this.encryptPasswordRequired;
		this.encryptPasswordRequired = encryptPasswordRequired;
		firePropertyChange("encryptPasswordRequired", oldValue, this.encryptPasswordRequired);
	}

	public boolean isDecryptPasswordRequired() {
		// パスワードファイル使用時は入力する必要がない
		if (usePasswordFile && decryptPasswordFileName != null && decryptPasswordFileName.length() > 0) {
			return false;
		}
		// パスワードが設定されている場合も入力する必要がない
		if (memoryDecryptPassword && decryptPassword != null && decryptPassword.length() > 0) {
			return false;
		}
		return decryptPasswordRequired;
	}

	public void setDecryptPasswordRequired(boolean decryptPasswordRequired) {
		boolean oldValue = this.decryptPasswordRequired;
		this.decryptPasswordRequired = decryptPasswordRequired;
		firePropertyChange("decryptPasswordRequired", oldValue, this.decryptPasswordRequired);
	}

	public void setIncludeHidden(boolean includeHidden) {
		boolean oldValue = this.includeHidden;
		this.includeHidden = includeHidden;
		firePropertyChange("includeHidden", oldValue, this.includeHidden);
	}

	public boolean isQuitAfterEncrypt() {
		return quitAfterEncrypt;
	}

	public void setQuitAfterEncrypt(boolean quitAfterEncrypt) {
		boolean oldValue = this.quitAfterEncrypt;
		this.quitAfterEncrypt = quitAfterEncrypt;
		firePropertyChange("quitAfterEncrypt", oldValue, this.quitAfterEncrypt);
	}

	public boolean isAlwaysOnTop() {
		return alwaysOnTop;
	}

	public void setAlwaysOnTop(boolean alwaysOnTop) {
		boolean oldValue = this.alwaysOnTop;
		this.alwaysOnTop = alwaysOnTop;
		firePropertyChange("alwaysOnTop", oldValue, this.alwaysOnTop);
	}

	public boolean isOpenDirectoryAfterDecrypt() {
		return openDirectoryAfterDecrypt;
	}

	public void setOpenDirectoryAfterDecrypt(boolean openDirectoryAfterDecrypt) {
		boolean oldValue = this.openDirectoryAfterDecrypt;
		this.openDirectoryAfterDecrypt = openDirectoryAfterDecrypt;
		firePropertyChange("openDirectoryAfterDecrypt", oldValue, this.openDirectoryAfterDecrypt);
	}

	public boolean isAskEncryptOrDecrypt() {
		return askEncryptOrDecrypt;
	}

	public void setAskEncryptOrDecrypt(boolean askEncryptOrDecrypt) {
		boolean oldValue = askEncryptOrDecrypt;
		this.askEncryptOrDecrypt = askEncryptOrDecrypt;
		firePropertyChange("askEncryptOrDecrypt", oldValue, this.askEncryptOrDecrypt);
	}

	public boolean isMemoryEncryptPassword() {
		return memoryEncryptPassword;
	}

	public void setMemoryEncryptPassword(boolean memoryEncryptPassword) {
		boolean oldValue = this.memoryEncryptPassword;
		this.memoryEncryptPassword = memoryEncryptPassword;
		firePropertyChange("memoryEncryptPassword", oldValue, this.memoryEncryptPassword);
	}

	public boolean isMemoryDecryptPassword() {
		return memoryDecryptPassword;
	}

	public void setMemoryDecryptPassword(boolean memoryDecryptPassword) {
		boolean oldValue = this.memoryDecryptPassword;
		this.memoryDecryptPassword = memoryDecryptPassword;
		firePropertyChange("memoryDecryptPassword", oldValue, this.memoryDecryptPassword);
	}

	public boolean isEncryptWithoutConfirmation() {
		return encryptWithoutConfirmation;
	}

	public void setEncryptWithoutConfirmation(boolean encryptWithoutConfirmation) {
		boolean oldValue = this.encryptWithoutConfirmation;
		this.encryptWithoutConfirmation = encryptWithoutConfirmation;
		firePropertyChange("encryptWithoutConfirmation", oldValue, this.encryptWithoutConfirmation);
		setEncryptPasswordRequired(encryptPassword != null);
		setDecryptPasswardRequired(decryptPassword != null);
	}

	public boolean isShowPassword() {
		return showPassword;
	}

	public void setShowPassword(boolean showPassword) {
		boolean oldValue = this.showPassword;
		this.showPassword = showPassword;
		firePropertyChange("showPassword", oldValue, this.showPassword);
	}

	public boolean isEncodeToSameFolder() {
		return encodeToSameFolder;
	}

	public void setEncodeToSameFolder(boolean encodeToSameFolder) {
		boolean oldValue = this.encodeToSameFolder;
		this.encodeToSameFolder = encodeToSameFolder;
		firePropertyChange("encodeToSameFolder", oldValue, this.encodeToSameFolder);
	}

	public String getEncodeFolderPath() {
		return encodeFolderPath;
	}

	public void setEncodeFolderPath(String encodeFolderPath) {
		String oldValue = this.encodeFolderPath;
		this.encodeFolderPath = encodeFolderPath;
		firePropertyChange("encodeFolderPath", oldValue, this.encodeFolderPath);
	}

	public boolean isNotMakeParentFolder() {
		return notMakeParentFolder;
	}

	public void setNotMakeParentFolder(boolean notMakeParentFolder) {
		boolean oldValue = this.notMakeParentFolder;
		this.notMakeParentFolder = notMakeParentFolder;
		firePropertyChange("notMakeParentFolder", oldValue, this.notMakeParentFolder);
	}

	public boolean isUsePasswordFile() {
		return usePasswordFile;
	}

	public void setUsePasswordFile(boolean usePasswordFile) {
		boolean oldValue = this.usePasswordFile;
		this.usePasswordFile = usePasswordFile;
		firePropertyChange("usePasswordFile", oldValue, this.usePasswordFile);
	}

	@Override
	public Options clone() throws CloneNotSupportedException {
		return (Options) super.clone();
	}

	public boolean isChanged() {
		return changed;
	}

	public void setChanged(boolean changed) {
		this.changed = changed;
	}

	public boolean isAddFileName() {
		return addFileName;
	}

	public void setAddFileName(boolean addFileName) {
		this.addFileName = addFileName;
	}

	private String getIniFileDefault() {
		StringBuilder name = new StringBuilder(Utility.includeTrailingPathDelimiter(System.getProperty("user.dir")));
		name.append(AttacheCaseConstant.INI_FILE_NAME);
		return name.toString();
	}

	private String getIniFileNameFromResource() {
		File dir = null;

		URL url = Options.class.getResource("Options.class");
		String protocol = url.getProtocol();
		if (protocol.equals("jar")) {
			Pattern p = Pattern.compile("(file:/.*)!/");
			Matcher m = p.matcher(url.toString());
			if (!m.find()) {
				return getIniFileDefault();
			}

			File jarFile = null;
			try {
				jarFile = new File(new URI(m.group(1)));
			} catch (URISyntaxException e) {
				e.printStackTrace();
				return getIniFileDefault();
			}
			dir = jarFile.getParentFile();
		} else if (protocol.equals("file")) {
			File classFile = null;
			try {
				classFile = new File(new URI(url.toString()));
			} catch (URISyntaxException e) {
				e.printStackTrace();
				return getIniFileDefault();
			}

			Package pack = Options.class.getPackage();
			if (pack == null) {
				dir = classFile.getParentFile();
			} else {
				String packName = pack.getName();
				String[] dirs = packName.split("\\.");
				dir = classFile.getParentFile();
				for (int i = 0; i < dirs.length; i++) {
					dir = dir.getParentFile();
				}
			}
		} else {
			return getIniFileDefault();
		}
		File iniFile = new File(dir, AttacheCaseConstant.INI_FILE_NAME);
		return iniFile.getAbsolutePath();
	}

	private String getIniFileName() {
		if (Utility.isMac()) {
			return MAC_INI_FILE_PATH;
		}
		
		String fileName = null;
		String cp = System.getProperty("java.class.path");
		int pos = cp.indexOf(':');
		if (pos >= 0 && pos != 1) {
			cp = cp.substring(pos + 1);
		}
		if (cp.endsWith("attachecase.jar")) {
			String dirPath = cp.substring(0, cp.lastIndexOf(File.separator) + 1);
			fileName = dirPath + INI_FILE_NAME;
		} else {
			fileName = getIniFileNameFromResource();
		}
		return fileName;
	}

	/**
	 * iniファイルから設定を取得します。
	 */
	public void loadFromFile() {
		Wini ini = null;
		File f = new File(getIniFileName());
		if (!f.exists()) {
			return;
		}
		try {
			ini = new Wini(f);
			BeanInfo beanInfo = Introspector.getBeanInfo(Options.class);
			PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
			for (String[] properties : PROPERTIES) {
				String sectionName = properties[0]; // セクション名
				for (int i = 1; i < properties.length; i++) {
					String value = ini.get(sectionName, properties[i]);
					if (value == null)
						continue;
					String propertyName = properties[i];
					if (isEncryptProperty(propertyName)) {
						propertyName = propertyName + "ByEncryption";
					}
					PropertyDescriptor pd = findPropertyDescriptor(propertyDescriptors, propertyName);
					if (pd == null)
						continue;
					Method method = pd.getWriteMethod();
					if (method == null)
						continue;
					// boolean型かどうか
					if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false")) {
						method.invoke(this, new Object[] { new Boolean(value) });
					} else {
						try {
							method.invoke(this, new Object[] { new Integer(value) });
						} catch (NumberFormatException e) {
							method.invoke(this, new Object[] { value });
						}
					}
				}
			}
		} catch (IllegalArgumentException e) {
			//e.printStackTrace();
		} catch (IllegalAccessException e) {
			//e.printStackTrace();
		} catch (InvocationTargetException e) {
			//e.printStackTrace();
		} catch (IntrospectionException e) {
			//e.printStackTrace();
		} catch (IOException e) {
			//e.printStackTrace();
		} finally {
			ini = null;
		}
	}

	/**
	 * iniファイルに設定を保存します。
	 */
	public void saveToFile() {
		Wini ini = null;

		try {
			File file = new File(getIniFileName());
			if (!file.exists()) {
				file.createNewFile();
			}
			ini = new Wini(file);
			BeanInfo beanInfo = Introspector.getBeanInfo(Options.class);
			PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
			for (String[] properties : PROPERTIES) {
				String sectionName = properties[0]; // セクション名
				for (int i = 1; i < properties.length; i++) {
					String propertyName = properties[i];
					if (isEncryptProperty(propertyName)) {
						propertyName = propertyName + "ByEncryption";
					}
					PropertyDescriptor pd = findPropertyDescriptor(propertyDescriptors, propertyName);
					if (pd == null)
						continue;
					Method method = pd.getReadMethod();
					if (method != null) {
						ini.put(sectionName, properties[i], method.invoke(this, (Object[]) null));
					}
				}
			}
			ini.store();
		} catch (IntrospectionException e) {
			//e.printStackTrace();
		} catch (IOException e) {
			//e.printStackTrace();
		} catch (InvocationTargetException e) {
			//e.getCause().printStackTrace();
			// e.printStackTrace();
		} catch (IllegalAccessException e) {
			//e.printStackTrace();
		} catch (Exception e) {
			//e.printStackTrace();
		} finally {
			ini = null;
		}
	}

	static private PropertyDescriptor findPropertyDescriptor(PropertyDescriptor[] descriptors, String propertyName) {
		for (PropertyDescriptor pd : descriptors) {
			if (pd.getName().equals(propertyName)) {
				return pd;
			}
		}
		return null;
	}

	public boolean isUsePasswordOption() {
		return usePasswordOption;
	}

	public void setUsePasswordOption(boolean usePasswordOption) {
		this.usePasswordOption = usePasswordOption;
	}

	/**
	 * 与えられたプロパティ名のプロパティが暗号化が必要かどうかをチェック
	 * 
	 * @param propertyName
	 * @return
	 */
	boolean isEncryptProperty(String propertyName) {
		for (String property : ENCRYPT_PROPERTIES) {
			if (property.equals(propertyName)) {
				return true;
			}
		}
		return false;
	}

	public boolean isUseVersion2() {
		return useVersion2;
	}

	public void setUseVersion2(boolean useVersion2) {
		this.useVersion2 = useVersion2;
	}
}
