/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.encrypt;

import java.io.UnsupportedEncodingException;
import java.util.Arrays;

import org.bouncycastle.crypto.BlockCipher;
import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.ShaEntity;
import org.hibara.attachecase.utility.AbstractProgressListener;

/**
 * 暗号化，また復号に必要な情報とメソッドを提供する抽象クラスです。
 * 
 * <pre>
 * 以下の機能を提供します。
 * ●パスワードを保持するための変数とアクセッサメソッド
 * 
 * このクラスは抽象クラスなので，サブクラスはrunメソッドをオーバーライドする必要があります。
 * </pre>
 * 
 * @author Masanori Uozumi
 * @version 2014.06.18 2012.08.06 メンバー変数を追加。<br>
 *          2012.08.07 FileDecrypt2からメンバー変数を追加。<br>
 *          2012.08.12 Runnableインターフェースをインプリメント。<br>
 *          2012.08.20 GUIでの動作時にプログレスバーなどを表示できるように変更。<br>
 *          2013.01.28 Rijndaelライブラリを変更。GNU CryptをBouncyCastleに変更<br>
 *          2014.06.18 Rijndael型をBlockCipherに変更。Blowfishに対応することを踏まえての変更<br>
 *          2017.02.19 パスワードファイルからのパスワード設定の扱いを変更<br>
 */
public abstract class Encrypt implements AttacheCaseConstant, Runnable {

	/**
	 * GUIで進捗状況を表示するために使用
	 */
	protected AbstractProgressListener listener = null;

	/**
	 * 暗号化・復号処理の終了
	 */
	protected boolean complete = false;

	/**
	 * オプション
	 */
	protected Options options = null;

	// パスワードキー
	/**
	 * 暗号化キーです。
	 */
	protected byte key[] = new byte[BUF_SHA1_SIZE]; // C++ではchar型の配列

	/**
	 * 旧版用暗号化キーです。
	 */
	protected byte oldKey[] = new byte[BUF_SHA1_SIZE]; // 下位互換維持のため。C++ではchar型の配列

	// SHA-1
	/**
	 * SHA-1格納用変数です。
	 */
	protected ShaEntity sha;

	/**
	 * 進捗状況を示す変数です。
	 */
	protected int progress; // 復号の進捗状況 %

	/**
	 * 現在処理中のファイルです。
	 */
	protected String currentFileName; // 現在処理中のファイル名

	/**
	 * 確認画面出力用
	 */
	protected Confirm confirm;

	/**
	 * 暗号化・復号用クラス
	 */
	protected BlockCipher blockCipher = null;

	/**
	 * 暗号化・復号用キー
	 */
	protected Object rijndaelKey = null;

	/**
	 * サイレントモード
	 */
	protected boolean silent = false;

	/**
	 * スリープする時間
	 */
	protected int sleepTime;

	/**
	 * 暗号化キー
	 */
	protected byte[] aesKey = new byte[BUF_SIZE_32]; // 暗号化キー

	/**
	 * 初期ベクトル
	 */
	protected byte[] aesIV = new byte[BUF_SIZE_32]; // 初期ベクトル

	/**
	 * 確認画面出力用オブジェクトを取得します。
	 * 
	 * @return 確認画面出力用オブジェクト
	 */
	public Confirm getConfirm() {
		return confirm;
	}

	/**
	 * 確認画面出力用オブジェクトを設定します。
	 * 
	 * @param confirm
	 *            確認画面出力用オブジェクト
	 */
	public void setConfirm(Confirm confirm) {
		this.confirm = confirm;
	}

	public boolean isComplete() {
		return complete;
	}

	/**
	 * SHA-1ハッシュを設定します。
	 * 
	 * <pre>
	 * SHA-1を設定します。パスワードとしてファイルデータを用いる場合に使用します。
	 * </pre>
	 * 
	 * @param sha
	 *            SHA-1ハッシュ
	 */
	public void setSha(ShaEntity sha) {
		this.sha = sha;

		for (int i = 0; i < key.length; i++) {
			key[i] = oldKey[i] = 0;
		}

		// keyの設定
		for (int i = 0; i < 32; i++) {
			key[i] = sha.getHashArray()[i];
			// C言語の文字列扱いで処理しているので0がくるとNULL文字としてそこで打ち切られる
			if (key[i] == 0) {
				break;
			}
		}

		// オールドキーの設定
		for (int i = 0; i < sha.getHashArray().length; i++) {
			oldKey[i] = sha.getHashArray()[i];
		}
	}

	/**
	 * パスワードを設定します。
	 * 
	 * <pre>
	 * パスワードを設定します。パスワードとしてプレーンテキストを用いる場合に使用します。
	 * </pre>
	 * 
	 * @param planePassword
	 *            復号用パスワード
	 */
	public void setPassword(String planePassword) {
		for (int i = 0; i < key.length; i++) {
			oldKey[i] = key[i] = 0;
		}

		// Windows上で，漢字のパスワードが設定されていることを考慮
		try {
			byte[] password = planePassword.getBytes("Windows-31J");
			int len = (key.length < password.length) ? key.length : password.length;
			for (int i = 0; i < len; i++) {
				oldKey[i] = key[i] = password[i];
			}
		} catch (UnsupportedEncodingException e) {
			// 何もしない
		}
	}

	/**
	 * パスワードを設定します。
	 * 
	 * <pre>
	 * パスワードを設定します。＃3でパスワードファイルを使用する場合に呼び出します
	 * </pre>
	 * 
	 * @param hash
	 *            復号用パスワード（ファイルのSHA256ハッシュ）
	 */
	public void setPassword(byte[] hash) {
		int length = Math.min(hash.length, key.length);
		for (int i = 0; i < length; i++) {
			key[i] = hash[i];
		}
	}

	/**
	 * パスワードを設定します。
	 * 
	 * <pre>
	 * パスワードを設定します。パスワードとしてプレーンテキストを用いる場合に使用します。
	 * 文字列ははUTF-8のままで扱われます。
	 * </pre>
	 * 
	 * @param planePassword
	 *            復号用パスワード
	 */
	public void setPasswordByUTF8(String planePassword) {
		for (int i = 0; i < key.length; i++) {
			oldKey[i] = key[i] = 0;
		}

		// Windows上で，漢字のパスワードが設定されていることを考慮
		try {
			byte[] password = planePassword.getBytes("UTF-8");
			int len = (key.length < password.length) ? key.length : password.length;
			for (int i = 0; i < len; i++) {
				oldKey[i] = key[i] = password[i];
			}
		} catch (UnsupportedEncodingException e) {
			// 何もしない
		}
	}

	/**
	 * 格納されているSHAを取得します。
	 * 
	 * @return SHA-1 Entity
	 */
	public ShaEntity getSha() {
		return sha;
	}

	/**
	 * 復号処理，または暗号化処理を行います。
	 * 
	 * @param listener
	 *            GUIのプログレスバー更新用リスナー
	 */
	public void run(AbstractProgressListener listener) {
		this.listener = listener;
		run();
	}

	/**
	 * 復号処理，または暗号化処理を行います。
	 */
	public abstract void run();

	/**
	 * 進捗状況を取得します。
	 * 
	 * <pre>
	 * 復号の進捗状況をパーセントで取得します。
	 * </pre>
	 * 
	 * @return 進捗状況(%)
	 */
	public int getProgress() {
		return progress;
	}

	/**
	 * 現在，復号中のファイルを取得します。
	 * 
	 * @return 復号中のファイル名
	 */
	public String getCurrentFileName() {
		return currentFileName;
	}

	public Options getOptions() {
		return options;
	}

	public void setOptions(Options options) {
		this.options = options;
		if (this.options.isSilent() || !this.options.isCui()) {
			silent = true;
		} else {
			silent = false;
		}
	}

	/**
	 * パスワードをchar型の配列で取得する
	 * 
	 * @return
	 */
	protected char[] getKeyByCharArray() {
		char[] chars = new char[this.key.length];
		for (int i = 0; i < key.length; i++) {
			chars[i] = (char) ((short) key[i] & 0xff);
		}
		return chars;
	}

}
