/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.encrypt;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;
import java.util.zip.DataFormatException;
import java.util.zip.Inflater;

import org.apache.commons.io.FilenameUtils;
import org.bouncycastle.crypto.engines.RijndaelEngine;
import org.bouncycastle.crypto.params.KeyParameter;
import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.ConsoleConfirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.Buffers;
import org.hibara.attachecase.entity.ByteBufferManager;
import org.hibara.attachecase.entity.FileAttribute;
import org.hibara.attachecase.entity.FileInformation;
import org.hibara.attachecase.entity.InfoTable;
import org.hibara.attachecase.entity.ShaEntity;
import org.hibara.attachecase.exception.AttachecaseException;
import org.hibara.attachecase.exception.DiskSpaceException;
import org.hibara.attachecase.exception.FileDestroyedException;
import org.hibara.attachecase.exception.LowerVersionException;
import org.hibara.attachecase.exception.NotEncryptFileException;
import org.hibara.attachecase.exception.PasswordException;
import org.hibara.attachecase.exception.ProcessBreakException;
import org.hibara.attachecase.exception.UpperVersionException;
import org.hibara.attachecase.io.FileDeletion;
import org.hibara.attachecase.io.RandomAccessFile;
import org.hibara.attachecase.utility.Digest;
import org.hibara.attachecase.utility.Utility;

/**
 * このクラスはAttachecaseで暗号化されたファイルの復号を行います。
 * 
 * <pre>
 * オプションの指定にもとづいてファイルの削除も行います。
 * 注意：クラス名の最後に数字の2が付けられていますが，Attachecase1は存在していません。
 * </pre>
 * 
 * @author Masanori Uozuomi
 * @version 2017.01.17<br>
 *          2012.07.28 ディレクトリの日付設定ロジックを変更。<br>
 *          2012.07.29 ファイル属性の設定ロジック追加。<br>
 *          2012.08.01
 *          プレーン・パスワードに漢字が用いられている場合のことを考慮。プレーンパスワードを設定する配列をkeyに変更。<br>
 *          2012.08.02 上書き確認の処理がフラグと逆になっていたバグを修正。<br>
 *          2012.08.03 Encryptクラスのサブクラスに変更<br>
 *          2012.08.07 メンバ変数の一部をスーパークラス Encrypt に移動。ファイルパスの冗長部分の削除処理を追加。<br>
 *          2012.08.18 暗号ファイルを複数指定できるように変更<br>
 *          2012.08.21 復号メソッドのバグFIX<br>
 *          2012.12.22 同名ディレクトリが存在した時に，上書きせず異なるフォルダに復号するときの処理にバグがあったのを修正<br>
 *          2012.12.23 復号時，パスワードを間違えた場合，入力を複数回繰り返すように変更<br>
 *          2013.01.28 Rijndaelライブラリを変更。GNU CryptをBouncyCastleに変更<br>
 *          2013.03.12 メッセージの出力を暗号化時と合わせる。<br>
 *          2013.06.03 getSavePathメソッドのロジックを見直し<br>
 *          2017.01.17 アタッシェケース#3に対応するため変数とメソッドのスコープを変更<br>
 */
public class FileDecrypt2 extends Encrypt {
	protected static final boolean DEBUG = false;
	protected static int typeLimits = 0;

	// ストリーム
	protected RandomAccessFile fsIn;

	// ファイルから読み込んだ諸設定値
	protected InfoTable infoTable;

	// 同名ファイル・フォルダーはすべて上書きして復号する
	// （ユーザーがダイアログで「すべてはい」を選択したとき = true ）
	protected boolean overwirteYesToAll;

	protected int countOfTrials = 0; // パスワードの試行回数

	protected String atcFilePath; // 入力する暗号化ファイルパス

	protected boolean compare; // コンペア
	// private List<String> compareFileList; //コンペアする元ファイルリスト

	protected boolean failPasswordFile = false;

	/**
	 * 復号用クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended
	 *            生成を一時中断するかどうか
	 */
	public FileDecrypt2(boolean createSuspended) {
		this(createSuspended, new Options(), new ConsoleConfirm());
	}

	/**
	 * 復号用クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended
	 *            生成を一時中断するかどうか
	 * @param options
	 *            オプション
	 */
	public FileDecrypt2(boolean createSuspended, Options options) {
		this(createSuspended, options, new ConsoleConfirm());
	}

	/**
	 * 復号用クラスを準備します。
	 * 
	 * @param createSuspended
	 *            作成を一時的にストップするか
	 * @param options
	 *            オプション
	 * @param confirm
	 *            ユーザー確認オブジェクト （問い合わせをコンソールで行うか，ダイアログで行うか）
	 */
	public FileDecrypt2(boolean createSuspended, Options options, Confirm confirm) {
		for (int i = 0; i < key.length; i++) {
			key[i] = 0;
		}
		setOptions(options);
		this.confirm = confirm;
		// sleepTime = (options.isCui()) ? SLEEP_TIME_FOR_CUI :
		// SLEEP_TIME_FOR_GUI;
	}

	public static int getTypeLimits() {
		return typeLimits;
	}

	public static void setTypeLimits(int typeLimits) {
		FileDecrypt2.typeLimits = typeLimits;
	}

	public String getAtcFilePath() {
		return atcFilePath;
	}

	public void setAtcFilePath(String atcFilePath) {
		this.atcFilePath = atcFilePath;
	}

	/**
	 * エラーなどでrunメソッドを抜ける際の後始末メソッド
	 */
	protected void destroy() {
		try {
			if (fsIn != null) {
				fsIn.close();
				fsIn = null;
			}
		} catch (IOException e) {
			// e.printStackTrace();
			confirm.showErrorMessage(e.getMessage());
		}
	}

	@Override
	protected void finalize() throws Throwable {
		destroy();
		super.finalize();
	}

	/**
	 * 復号します。
	 * 
	 * <pre>
	 * このメソッドが呼び出される前に，以下のデータが設定されている必要があります。
	 * ● 復号するアタッシェケースファイル名
	 * ● パスワードまたはパスワード用ハッシュ
	 * </pre>
	 */
	@Override
	public void run() {
		complete = false;
		if (!silent) {
			System.out.println(ResourceBundle.getBundle(RESOURCE).getString("MSG_DECRYPT_START"));
		}
		boolean ignoreFileNotExist = false;
		/*
		 * // パスワードファイルが設定されておらず，複数ファイルが指定されている場合はその都度パスワードを入力させる。 if
		 * (options.getDecryptFileNames().size() > 1 &&
		 * options.getDecryptPasswordFileName() == null) {
		 * options.setDecryptPasswardRequired(true); }
		 */

		try {
			for (String fname : options.getDecryptFileNames()) {
				atcFilePath = fname;
				File f = new File(atcFilePath);
				if (!f.exists() && ignoreFileNotExist) {
					// 存在しないけど無視。
					continue;
				} else if (!f.exists() && !ignoreFileNotExist) {
					// 存在しないけど，無視もしない。
					Object[] values = { atcFilePath };
					String msg = Utility.replacePlaceHolder(
							ResourceBundle.getBundle(RESOURCE).getString(MSG_ERROR_NOT_FILE_EXIST), values);
					msg += ResourceBundle.getBundle(RESOURCE).getString(MSG_CONFIRM_CONTINUE);
					int result = confirm.confirmYesNoAllCancel(msg, false);
					if (result == Confirm.ALL) {
						ignoreFileNotExist = true;
						continue;
					} else if (result == Confirm.YES) {
						continue;
					} else {
						throw new FileNotFoundException(msg);
					}
				}
				FileInformation fileInfomation = null;
				List<String> fileNameList = null;
				String mode = "rw"; // 常に読み書きモード
				fsIn = new RandomAccessFile(atcFilePath, mode);
				// パスワードチェックループ

				while (true) {
					// 通常の処理
					this.setPassword();
					// String mode = (options.isFileBrakeOption() && numOfTrials
					// > typeLimits) ? "rw" : "r";

					infoTable = new InfoTable(fsIn);
					infoTable.readFromFile();
					if (!infoTable.isMatchToken()) {
						destroy();
						if (infoTable.isMatchToken(DESTROY_TOKEN_STRING)) {
							throw new FileDestroyedException();
						} else {
							throw new NotEncryptFileException();
						}
					}
					// -----------------------------------
					// データバージョンチェック
					// -----------------------------------
					int dataVersion = infoTable.getDataVersion();
					// System.out.println("DataVersion = " + dataVersion);
					if (dataVersion > ATC_DATA_FILE_VERSION && dataVersion < 200) {
						throw new UpperVersionException();
					} else if (dataVersion <= 103) {
						throw new LowerVersionException();
					}
					// -----------------------------------
					// 復号の準備
					// -----------------------------------
					blockCipher = new RijndaelEngine(RIJNDAEL_BLOCK_SIZE * 8);
					// rijndaelKey = getKey();

					blockCipher.init(false, new KeyParameter(getKeyArray()));
					fileNameList = getFileNameList(fsIn);
					fileInfomation = new FileInformation(fileNameList);
					// -----------------------------------
					// 復号正否（復号できたか）
					// -----------------------------------
					boolean success = isSuccess(fileNameList);
					if (success) {
						break;
					} else if (infoTable.getTypeLimits() <= countOfTrials) {
						// 復号処理に失敗
						throw new PasswordException();
					}
					fsIn.seek(0);
					if (options.isUsePasswordFile()) {
						failPasswordFile = true;
					}
				}
				long driveFreeSize = Utility.getDiskFreeSpace(getSavePath());
				if (fileInfomation.getAllTotalSize() > driveFreeSize) {
					throw new DiskSpaceException(driveFreeSize, fileInfomation.getAllTotalSize());
				}
				// -----------------------------------
				// 複数フォルダ/ファイルを開こうとしているので確認
				// -----------------------------------
				if (options != null && options.isOpenFile() && fileInfomation.getFileListSize() > 4
						&& compare == false) {
					// '5つ以上のファイルを復号後に開こうとしていますが、処理を続けますか？'+#13+
					// '「いいえ」を選択すると、開かず復号します。';
					int result = confirm.confirmYesNoCancel(MSG_CONFIRM_OPEN_DECRYPTED_FILES, true);
					if (result == Confirm.NO) {
						options.setOpenFile(false);
					} else if (result == Confirm.CANCEL) {
						confirm.showMessageByBundle("MSG_PROCESS_CANCELED");
						return;
					}
				}
				// -----------------------------------
				// 復号開始
				// -----------------------------------
				decrypt(fsIn, fileInfomation);

				// 復号終了を表示
				if (!silent) {
					System.out.print(Confirm.resources.getString("MSG_DECRYPT_END"));
				}
				// -----------------------------------
				// 暗号化ファイルの削除
				// -----------------------------------
				if (options.isDeleteEncryptoFile()) { // 削除しないのであればこれで終了
					// ファイルの削除
					FileDeletion deletion = new FileDeletion(options);
					deletion.setDeleteFile(atcFilePath);
					if (!silent) {
						confirm.showMessageByBundle("MSG_DELETE_START");
					}
					deletion.run(listener);
					if (!silent) {
						// System.out.printf(Confirm.resources.getString("MSG_DECRYPT_FILE_DELETE"),
						// f.getName());
						confirm.showMessageByBundle("MSG_DELETE_END");
					}
				}
			}
			complete = true;
		} catch (IOException e) {
			confirm.showErrorMessage(e.getMessage());
			if (DEBUG) {
				e.printStackTrace();
			}
		} catch (InvalidKeyException e) {
			// 何もしない
		} catch (DiskSpaceException e) {
			// ディスク容量が足りない
			confirm.showErrorMessage(e.getErrorMessage());
		} catch (ProcessBreakException e) {
			// 何もしない
		} catch (AttachecaseException e) {
			// 各種エラーメッセージ
			confirm.showErrorMessageByBundle(e.getBundle());
			// パスワードエラーの場合
			if (e instanceof PasswordException) {

			}
		} catch (Exception e) {
			confirm.showErrorMessageByBundle(e.getMessage());
			if (DEBUG) {
				e.printStackTrace();
			}
		} finally {
			destroy();
		}
	}

	/**
	 * アタッシェケースファイルから暗号化ファイルに含まれているファイルの一覧を取得します。
	 * 
	 * @param fs
	 *            暗号化ファイルのRandomAccessFileオブジェクト
	 * @return 暗号化ファイルに含まれているファイルの一覧
	 * @throws IOException
	 * @throws InvalidKeyException
	 */
	private List<String> getFileNameList(RandomAccessFile fs) throws IOException, InvalidKeyException {
		// if (DEBUG) System.out.println("DataVersion = " +
		// infoTable.getDataVersion());
		// パスワードセット
		/*
		 * if (rijndaelKey == null) { rijndaelKey = getKey(); }
		 */
		ArrayList<String> fileNames = new ArrayList<String>();
		int len = 0; // 読み込んだバイト数
		byte[] chainBuffer = new byte[BUF_SIZE_32];
		fs.read(chainBuffer);
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		// System.out.println("EncryptHeaderSize:" +
		// infoTable.getEncryptHeaderSize());
		while (len < infoTable.getEncryptHeaderSize()) {
			byte[] sourceBuffer = new byte[BUF_SIZE_32];
			len += fs.read(sourceBuffer);
			byte[] tempBuffer = sourceBuffer.clone(); // あとのxorのためによけておく
			byte[] outBuffer = new byte[BUF_SIZE_32];
			blockCipher.processBlock(sourceBuffer, 0, outBuffer, 0);

			for (int i = 0; i < BUF_SIZE_32; i++) {
				outBuffer[i] ^= chainBuffer[i];
				chainBuffer[i] = tempBuffer[i]; // CBC
			}

			os.write(outBuffer);
		}
		// もともとがWindows用ソフトなので，文字コードもWindowsに合わせる
		String fileNamesString = new String(os.toByteArray(), "Windows-31J");
		// System.out.println("文字列:" + fileNamesString);
		os.close();
		// 改行文字で切り出す
		for (String name : fileNamesString.split("(\n|\r)")) {
			if (name.length() != 0) {
				fileNames.add(name);
				// System.out.println(name);
			}
		}
		return fileNames;
	}

	/**
	 * 復号の成否をチェックします。
	 * 
	 * @param list
	 * @return 復号に成功している場合は，true が返ります。
	 * @throws IOException
	 */
	private boolean isSuccess(List<String> list) throws IOException {
		countOfTrials++; // 試行回数インクリメント
		boolean success = true;
		// -----------------------------------
		// 復号正否（復号できたか）
		// -----------------------------------
		// TODO numOfTrials
		if (list == null || list.size() == 0 || list.get(0).indexOf("AttacheCase") == -1) {
			success = false;
			// 試行回数のチェック
			if (infoTable.isFileDestroy() && countOfTrials >= infoTable.getTypeLimits()) {
				// 試行回数が設定されたミスタイプ数を超えたのでIVを破壊する
				fsIn.write(new byte[BUF_SIZE_32]);
				// 遡ってヘッダ部分のIVも破壊する
				fsIn.seekFromCurrent((long) (-BUF_SIZE_32 - SIZE_OF_INT * 3 - 16));
				fsIn.write(DESTROY_TOKEN_STRING.getBytes()); // "_Atc_Broken_Data"
			}
		}

		return success;
	}

	/**
	 * 復号したファイルの保存ディレクトリを取得します。
	 * 
	 * @return 保存ディレクトリ
	 */
	protected String getSavePath() {
		String path = (options != null) ? options.getDecodeFolderPath() : null;

		if (options == null || !options.isDecodeToSameFolder() || path == null || path.length() == 0) {
			File f = new File(atcFilePath);
			path = f.getAbsoluteFile().getParent();
			// System.out.println(path);
		}
		return Utility.includeTrailingPathDelimiter(FilenameUtils.normalize(path));
	}

	/**
	 * 復号用キーを取得します。
	 * 
	 * @return 復号用キー
	 * @throws InvalidKeyException
	 */
	/*
	 * private Object getKey() throws InvalidKeyException { byte[] keys =
	 * this.key; //パスワードセット if (infoTable.getDataVersion() == 105 && oldKey[0]
	 * != 0) { //ver.2.75 && 古い形式のパスワードファイルを使っている keys = oldKey; } return
	 * rijndael.makeKey(keys, RIJNDAEL_BLOCK_SIZE); }
	 */

	/**
	 * 暗号用キーを取得します。
	 * 
	 * @return 暗号用キー
	 */
	protected byte[] getKeyArray() {
		byte[] keys = Arrays.copyOf(this.key, this.key.length);
		// パスワードセット
		/*
		 * if (infoTable.getDataVersion() == 105 && oldKey[0] != 0) { //
		 * ver.2.75 // && // 古い形式のパスワードファイルを使っている keys = oldKey; }
		 */
		/*
		 * for (int i = 0; i < keys.length; i++) { System.out.printf("%02x ",
		 * keys[i]); } System.out.println();
		 */

		return keys;
	}

	/**
	 * データを読み出す
	 * 
	 * @param fs
	 * @param buffers
	 * @return 読み出したバイト数の累計
	 * @throws IOException
	 */
	protected long inputBuffer(RandomAccessFile fs, Buffers buffers) throws IOException {
		long size = buffers.getTotalSize();
		// 読み出し用バッファの初期化
		buffers.clearSourceBuffer();
		// 入力ファイルが開かれていない
		if (fs == null) {
			buffers.setLength(0);
			return size;
		}
		// 最後のブロックを越えている
		if (size >= buffers.getAllTotalSize()) {
			buffers.setLength(0);
			return size;
		}

		// 暗号化されたデータブロックの読み出し
		byte[] sourceBuffer = buffers.getSourceBuffer();
		size += fsIn.read(sourceBuffer);
		byte[] tempBuffer = sourceBuffer.clone(); // あとのxorのためによけておく

		// 復号処理
		byte[] outBuffer = new byte[BUF_SIZE_32];
		blockCipher.processBlock(sourceBuffer, 0, outBuffer, 0);
		// xor
		byte[] chainBuffer = buffers.getChainBuffer();
		for (int i = 0; i < BUF_SIZE_32; i++) {
			sourceBuffer[i] = (byte) (outBuffer[i] ^ chainBuffer[i]);
		}
		buffers.setChainBuffer(tempBuffer.clone());

		// 最後のブロックの境界... 読み込みすぎた場合の処理
		int length = BUF_SIZE_32;
		if (size > buffers.getAllTotalSize()) {
			// Check PKCS #7 pading num
			boolean pkcs = false;
			int paddingNum = sourceBuffer[BUF_SIZE_32 - 1];
			for (int i = 0; i < paddingNum; i++) {
				if (sourceBuffer[BUF_SIZE_32 - 1 - i] == paddingNum) {
					pkcs = true;
				} else {
					pkcs = false;
					break;
				}
			}
			if (pkcs == true) {
				length = BUF_SIZE_32 - paddingNum;
			}
		}
		buffers.setLength(length);
		buffers.setTotalSize(size);
		return size;
	}

	/**
	 * 復号処理を行います。 引数として渡したファイル情報に基づいて，復号し，ファイルに書き込みます。
	 * 
	 * @param fs
	 *            暗号化ファイル
	 * @param info
	 *            復号するファイルに関する情報を格納したリスト。
	 * @throws IOException
	 *             ファイルの読み書きで発生した例外
	 * @throws AttachecaseException
	 *             暗号ファイルが壊れている場合，またキャンセル発生時
	 */
	protected void decrypt(RandomAccessFile fs, FileInformation info) throws IOException, AttachecaseException {
		Inflater decompresser = new Inflater(false); // 伸張用クラス
		byte[] chainBuffer = new byte[BUF_SIZE_32]; // chainバッファ
		byte[] sourceBuffer = new byte[BUF_SIZE_32]; // ソースバッファ
		byte[] decompBuffer = new byte[FILE_BUFFER_SIZE * 2]; // 伸張用バッファ
		byte[] writeBuffer = new byte[FILE_BUFFER_SIZE]; // ファイル書き込み用バッファ
		long allTotalSize = fs.length() - fs.getFilePointer(); // 総バイト数
		long totalSize = fs.read(chainBuffer); // 初期化ベクトルの読み出し，読み込んだバイト数
		String saveDirPath = getSavePath(); // 保存ディレクトリ
		// 保存ディレクトリが存在するかどうかをチェックし，存在しない場合はディレクトリを作成する。
		makeSaveDir(saveDirPath);
		// ディレクトリの日付を元の日付に更新するために，ディレクトリの情報だけを取得する
		List<FileAttribute> dirList = new ArrayList<FileAttribute>(); // ディレクトリ内のファイルをいじるとディレクトリの更新日時が変更される
		// バッファ管理用
		Buffers buffers = new Buffers(sourceBuffer, chainBuffer, totalSize, allTotalSize);
		ByteBufferManager expandBuffer = new ByteBufferManager(BUF_SIZE_256); // 256バイトを一区切り
		// 復号するファイルに関する情報
		Iterator<FileAttribute> it = info.getFiles().iterator();
		FileAttribute attr = null;
		// ファイルに書き込むためのStream
		BufferedOutputStream os = null;
		long writeLength = 0; // 個々のファイルに書き込んだバイト数
		int errorCount = 0; // ファイルが壊れているなどで終了しないなどの事例に対応。閾値以上になったら強制終了
		String writeFileName = null;// 実際に書き込んだファイルの名前（同名ファイルの存在などで名前が変更されているかもしれないので，変数を一個作っておく）
		try {
			while (it.hasNext() || os != null) {
				// 32バイトデータをファイルから読み込んで復号
				totalSize = inputBuffer(fsIn, buffers); // inputBufferメソッド内でtotalSizeとallTotalSizeを比較して読込を制御
				if (totalSize == allTotalSize) { // すでに読込が終了している場合
					errorCount++; // 読込が終了しているのにループを抜けられないのはファイルがおかしい可能性大
				}
				// 進捗状況の更新
				progress = (int) (100 * totalSize / allTotalSize);
				// System.out.printf("AllTotal : %d Total : %d\n",
				// allTotalSize, totalSize);
				if (listener != null) {
					listener.publishProgress(progress);
					if (listener.isCancel()) { // キャンセルボタンが押されたか
						throw new ProcessBreakException();
					}
				}
				// 伸張処理
				if (buffers.getLength() >= 0) { // TODO 0以上でいいのか検証
					decompresser.setInput(buffers.getSourceBuffer(), 0, buffers.getLength());
					while (!decompresser.finished()) {
						try {
							int count = decompresser.inflate(decompBuffer);
							if (count == 0) {
								break;
							}
							expandBuffer.add(decompBuffer, count);
						} catch (DataFormatException ex) {
							// 何もしない
						}
					}
				}
				// ファイルへの書き込み準備
				// 現在書き込み中のファイルが存在しないなら，新しいファイル情報を取り出して処理
				if (os == null) {
					attr = it.next(); // 次のファイル情報
					currentFileName = attr.getFileNameOnly(); // 現在処理中のファイル名
					// 処置中のファイル名をGUIに表示
					if (listener != null) {
						listener.publishFileName(Confirm.resources.getString("MSG_DECRYPT_CONTINUE") + currentFileName);
					}
					String fileName = saveDirPath + currentFileName; // 保存先ファイル名（フルパス）
					// System.out.printf("saveDirPath:%s fileName:%s \n",
					// saveDirPath, currentFileName);
					File f = new File(fileName);
					// ディレクトリの場合
					if (attr.isDirectory()) {
						// ディレクトリが存在するかどうかを確認
						if (f.exists() && options.isConfirmOverwirte() && !overwirteYesToAll) {
							Object[] values = { currentFileName };
							String msg = Utility.replacePlaceHolder(
									ResourceBundle.getBundle(RESOURCE).getString(MSG_CONFIRM_OVER_WRITE_SAME_DIR),
									values);
							int result = confirm.confirmYesNoAllCancel(msg, false);
							switch (result) {
							case Confirm.CANCEL:
								throw new ProcessBreakException();
							case Confirm.ALL:
								overwirteYesToAll = true;
								break;
							case Confirm.NO:
								// 新しい名前をsaveDirPathに変更
								saveDirPath = Utility
										.includeTrailingPathDelimiter(confirm.inputDirName(f.getParentFile()));
								fileName = saveDirPath + currentFileName;
								// System.out.println(fileName);
								f = new File(fileName);
							}
						}
						f.mkdirs(); // ディレクトリ作成
						if (!options.isTimestampNow()) { // タイムスタンプを元ファイルと合わせる場合
							dirList.add(attr);
						}
						continue;
					}
					// ファイルの場合
					// 同一ファイルが存在する場合
					if (f.exists() && options.isConfirmOverwirte() && !overwirteYesToAll) {
						Object[] values = { currentFileName };
						String msg = Utility.replacePlaceHolder(
								ResourceBundle.getBundle(RESOURCE).getString(MSG_CONFIRM_OVER_WRITE_SAME_FILE), values);
						int result = confirm.confirmYesNoAllCancel(msg, false);
						switch (result) {
						case Confirm.CANCEL:
							throw new ProcessBreakException();
						case Confirm.ALL:
							overwirteYesToAll = true;
							break;
						case Confirm.NO:
							currentFileName = Utility.includeTrailingPathDelimiter(confirm.inputFileName(f));
							// ファイル名がフルパスかどうかを確認
							// GUIで入力するとフルパスで返ってくる。CUIではフィアル名のみ
							if (new File(currentFileName).isAbsolute()) {
								fileName = currentFileName;
							} else {
								fileName = saveDirPath + currentFileName;
								fileName = FilenameUtils.normalize(fileName);
							}
							// System.out.println("fileName : " + fileName);
						}
					}
					// 書き込むためにOutputStreamを作成
					if (fileName.indexOf("../") != -1) {
						throw new AttachecaseException();
					}
					os = new BufferedOutputStream(new FileOutputStream(fileName));
					writeFileName = fileName;
					writeLength = 0;
				}
				// 書き込み処理。書き込みデータが存在する
				while (!expandBuffer.isFinished()) {
					int length = (writeLength + FILE_BUFFER_SIZE < attr.getFileSize()) ? FILE_BUFFER_SIZE
							: (int) (attr.getFileSize() - writeLength);
					length = expandBuffer.read(writeBuffer, length);
					os.write(writeBuffer, 0, length); // ファイル書き込み
					// System.out.printf("%s に %dバイト / %d を書き込み\n",
					// currentFileName, writeLength + length,
					// attr.getFileSize());
					writeLength += length; // ファイルに書き込んだバイト数
					// 書き込み完了かどうかのチェック
					if (writeLength >= attr.getFileSize()) {
						os.close();
						os = null;
						// 日付の変更
						File f = new File(writeFileName);
						if (f.exists() && !options.isTimestampNow()) {
							Utility.setFileTimeStamp(writeFileName, attr, false);
						}
						Utility.setFileAttribute(writeFileName, attr);
						// 復号終了を表示
						if (!silent) {
							System.out.printf(ResourceBundle.getBundle(RESOURCE).getString("MSG_DECRYPT_FILE"),
									currentFileName);
						}
						break;
					}
					// 書き込みがあった場合はエラーチェック用変数を0に戻す
					errorCount = 0;
				}
				// エラーが閾値を超えたら例外を発生
				if (errorCount > 30) {
					throw new FileDestroyedException();
				}
			}
			// ディレクトリの日付を変更する
			for (FileAttribute attribute : dirList) {
				String fileName = saveDirPath + attribute.getFileNameOnly(); // ディレクトリ
				Utility.setFileTimeStamp(fileName, attribute, false);
				Utility.setFileAttribute(fileName, attribute);
			}
		} catch (IOException e) {
			// e.printStackTrace();
			throw e;
		} finally {
			if (os != null) {
				try {
					os.close();
				} catch (IOException ex) {
					// 何もしない。
				}
			}
		}

	}

	/**
	 * 指定されたディレクトリが存在するかどうかをチェックし，存在しなければ作成します。
	 * 
	 * @param path
	 *            ディレクトリ
	 */
	protected void makeSaveDir(String path) {
		File f = new File(path);
		if (f.exists()) {
			return;
		}
		f.mkdir();
	}

	/**
	 * パスワードを設定します。
	 * 
	 * <pre>
	 * パスワードファイルを利用する設定の場合は，ファイルを読み込んでパスワードを設定します。
	 * 入力する必要がある場合は，キーボードから入力を求めます。
	 * </pre>
	 * 
	 * @param encrypt
	 *            暗号化オブジェクトまたは復号オブジェクト
	 * @throws IOException
	 *             ファイルが存在しない場合
	 * @throws PasswordException
	 *             パスワード入力でエラーが発生した場合
	 */
	private void setPassword() throws IOException, PasswordException {
		// パスワード設定
		if (options.isDecryptPasswordRequired() || failPasswordFile) {
			File f = new File(atcFilePath);
			// 復号する際のパスワード入力は1回だけ
			String password = confirm.inputPassword(f.getName(), 1);
			if (password == null) {
				throw new PasswordException();
			}
			setPassword(password);
		} else {
			if (options.isUsePasswordFile() && options.getDecryptPasswordFileName() != null
					&& options.getDecryptPasswordFileName().length() > 0) { // ファイルが指定されている場合
				Digest digest = new Digest();
				ShaEntity shaEntity = digest.getHash(options.getDecryptPasswordFileName());
				setSha(shaEntity);
			} else {
				setPassword(options.getDecryptPassword());
			}
		}
	}
}
