/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.encrypt;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.ResourceBundle;
import java.util.zip.Deflater;

import javax.crypto.KeyGenerator;

import org.apache.commons.io.FilenameUtils;
import org.bouncycastle.crypto.engines.RijndaelEngine;
import org.bouncycastle.crypto.params.KeyParameter;
import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.ConsoleConfirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.ByteBufferManager;
import org.hibara.attachecase.entity.FileAttribute;
import org.hibara.attachecase.exception.AttachecaseException;
import org.hibara.attachecase.exception.DiskSpaceException;
import org.hibara.attachecase.exception.ProcessBreakException;
import org.hibara.attachecase.io.ByteArrayOutputStream;
import org.hibara.attachecase.io.FileDeletion;
import org.hibara.attachecase.utility.Utility;

/**
 * このクラスはファイルの暗号化を行います。<br>
 * 暗号化したファイルには，デフォルトでは".atc"という拡張子が付きます。<br>
 * 
 * @author Masanori Uozumi
 * @version 2017.01.20<br>
 *          2012.08.11 暗号化するフォルダが絶対パスで指定された場合のことを考慮。<br>
 *          2012.08.20 GUIでの動作時にプログレスバーなどを表示できるように変更。<br>
 *          2012.09.01 同じ名前で暗号化フィアルを作成しようとした場合の処理を追加。<br>
 *          2013.01.08
 *          Ver2.8.2が正式版になったことに合わせて，ヘッダ部分をUnicodeとシフトJIS両方のデータを持つように変更(addFileListメソッド)<br>
 *          2013.01.28 Rijndaelライブラリを変更。GNU CryptをBouncyCastleに変更<br>
 *          2017.01.20 アタッシェケース#3に対応
 */
public class FileEncrypt extends Encrypt {

	protected String encryptFileName; // 暗号化ファイル名
	protected long allTotalSize; // すべてのファイルの合計サイズ

	// 同名ファイル・フォルダーはすべて上書きして暗号する
	// （ユーザーがダイアログで「すべてはい」を選択したとき = true ）
	protected boolean overwirteYesToAll;

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended
	 *            生成を中止
	 */
	public FileEncrypt(boolean createSuspended) {
		this(createSuspended, new Options(), new ConsoleConfirm());
	}

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended 生成を中止
	 * @param options オプション
	 */
	public FileEncrypt(boolean createSuspended, Options options) {
		this(createSuspended, options, new ConsoleConfirm());
	}

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * @param createSuspended 生成を中止
	 * @param options
	 *            オプション
	 * @param confirm
	 *            ユーザー確認オブジェクト （問い合わせをコンソールで行うか，ダイアログで行うか）
	 */
	public FileEncrypt(boolean createSuspended, Options options, Confirm confirm) {
		setOptions(options);
		this.confirm = confirm;
		// sleepTime = (options.isCui()) ? SLEEP_TIME_FOR_CUI :
		// SLEEP_TIME_FOR_GUI;
	}

	@Override
	public void run() {
		BufferedOutputStream fos = null;
		if (!silent) {
			System.out.println(ResourceBundle.getBundle(RESOURCE).getString("MSG_ENCRYPT_START"));
		}
		try {
			// -----------------------------------
			// 暗号化の準備
			// -----------------------------------
			blockCipher = new RijndaelEngine(RIJNDAEL_BLOCK_SIZE * 8);
			blockCipher.init(true, new KeyParameter(this.key));
			// rijndaelKey = getKey();
			/*
			 * 暗号化するファイル情報を取得（複数のファイルが指定されることを想定
			 */
			List<List<FileAttribute>> lists = getFileLists();
			// リスト毎に処理
			for (List<FileAttribute> list : lists) {
				encryptFileName = getEncryptFileName(list);
				if (encryptFileName == null) {
					continue;
				}
				// 暗号化ファイルごとに暗号化ファイルが存在するかをチェック。存在する場合は別のファイル名に変換
				checkFileNotExist(); // 暗号化ファイルと同じ名前のファイルが存在しかどうかをチェック。存在している場合は中止される場合も有り。
				if (isExistMyself(list, encryptFileName)) {
					confirm.showErrorMessageByBundle("MSG_ERROR_WRITE_TO_MYSELF");
					throw new ProcessBreakException();
				}
				// 暗号化ファイル毎にヘッダファイルを求める
				ByteArrayOutputStream bos = createHeader();
				// ファイルリストをバイト列にして書き込む
				addFileList(bos, list);
				// ディスクの空き容量チェック
				allTotalSize = getAllTotalSize(list);
				long freeSpaceSize = Utility.getDiskFreeSpace(encryptFileName);
				// System.out.println(encryptFileName);
				// System.out.printf("総容量 %d 空き容量 %d\n", allTotalSize,
				// freeSpaceSize);
				if (allTotalSize > freeSpaceSize) {
					throw new DiskSpaceException(freeSpaceSize, allTotalSize);
				}
				// ファイルの書き込み

				fos = new BufferedOutputStream(new FileOutputStream(encryptFileName));
				// -----------------------------------
				// ヘッダ情報の描き込み
				// -----------------------------------
				fos.write(bos.toByteArray());
				bos.close();
				// -----------------------------------
				// ファイルへのデータ書き込み
				// -----------------------------------
				blockCipher.reset();
				writeToAtcFile(fos, list);
				fos.close();
				fos = null;
				// ATCファイルの日付を変更する？
				if (options.isKeepTimestamp()) {
					File orignal = new File(list.get(0).getFileNameOnly());
					File atc = new File(encryptFileName);
					atc.setLastModified(orignal.lastModified());
				}
				// ファイルの削除を実行?
				if (options.isDeleteOriginalFile()) {
					// System.out.println("削除実行");
					if (!silent) {
						confirm.showMessageByBundle("MSG_DELETE_START");
					}
					FileDeletion deletion = new FileDeletion(options);
					deletion.setFileListByAttribute(list);
					deletion.run(listener);
					if (!silent) {
						confirm.showMessageByBundle("MSG_DELETE_END");
					}
				}
			}
			complete = true;
		} catch (InvalidKeyException e) {
			e.printStackTrace();
			confirm.showErrorMessage(e.getMessage());
		} catch (ProcessBreakException e) {
			// ここで中止のダイアログは出さない。
			// アイコン用画像の読み込みが中止されたというエラーメッセージが出るため。
			e.printStackTrace();
		} catch (DiskSpaceException e) {
			e.printStackTrace();
			confirm.showErrorMessage(e.getErrorMessage());
		} catch (AttachecaseException e) {
			e.printStackTrace();
			// 各種エラーメッセージ
			confirm.showErrorMessageByBundle(e.getBundle());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			// ATCファイルを作成できなかった場合
			String msg = Confirm.resources.getString(MSG_ERROR_OUT_FILE_OPEN);
			Object[] obj = { encryptFileName };
			confirm.showErrorMessage(Utility.replacePlaceHolder(msg, obj));
		} catch (IOException e) {
			e.printStackTrace();
			confirm.showErrorMessageByBundle(MSG_ERROR_OUT_FILE_WRITE);
		} catch (SecurityException e) {
			confirm.showErrorMessage(e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			confirm.showErrorMessage(e.getMessage());
		} finally {
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					// 何もしない
				}
			}
		}
		if (!complete) {
			File deleteFile = new File(encryptFileName);
			deleteFile.delete();
		}
	}

	/**
	 * 指定した出力ストリームに対して，ファイルを暗号化して書き込みます。
	 * 
	 * @param os
	 *            出力するストリーム
	 * @param list
	 *            暗号化するファイルの情報
	 * @throws Exception
	 *             書き込みに失敗した場合
	 */
	protected void writeToAtcFile(OutputStream os, List<FileAttribute> list) throws Exception {
		long totalSize = 0;
		// rijndaelをまず初期化
		blockCipher.reset();
		// 初期化ベクトルを生成
		byte[] chainBuffer = getInitalizationVector();
		byte[] encryptBuffer = new byte[BUF_SIZE_32];
		// 圧縮の準備
		Deflater comp = new Deflater(options.getCompressRate(), false);
		ByteBufferManager readBufferManger = new ByteBufferManager(FILE_BUFFER_SIZE); // ファイルのデータを格納しておき，圧縮用に取り出す
		ByteBufferManager compBufferManger = new ByteBufferManager(BUF_SIZE_32); // 圧縮後のデータを格納しておき，32バイト毎に取り出す
		int length = 0; // 読込バイト数
		BufferedInputStream ios = null;
		FileAttribute attr = null;
		try {
			// 初期化ベクトルの書き込み
			os.write(chainBuffer);
			byte[] readBuffer = new byte[FILE_BUFFER_SIZE];
			byte[] compBuffer = new byte[BUF_SIZE_256];
			byte[] sourceBuffer = new byte[BUF_SIZE_32];
			for (int i = 0; i < list.size(); i++) {
				attr = list.get(i);
				if (attr.isDirectory()) {
					continue;
				}
				if (listener != null) {
					listener.publishFileName(
							Confirm.resources.getString("MSG_ENCRYPT_CONTINUE") + attr.getFileNameOnly());
					// キャンセルされている場合は中止
					if (listener.isCancel()) {
						listener.publishFileName(Confirm.resources.getString("MSG_ENCRYPT_STOPPED"));
						throw new ProcessBreakException();
					}
				}
				// System.out.println(attr.getFileNameOnly());
				ios = new BufferedInputStream(new FileInputStream(attr.getFileNameOnly()));
				// ファイルを読み込む
				while ((length = ios.read(readBuffer)) != -1) {
					totalSize += length;
					progress = (int) (100 * totalSize / allTotalSize);
					if (listener != null) {
						listener.publishProgress(progress);
					}
					readBufferManger.add(readBuffer, length);
					// 圧縮用に取り出し可能な場合
					while (readBufferManger.isAvailable()) {
						// System.out.println("圧縮するために放り込む");
						readBufferManger.read(readBuffer);
						comp.setInput(readBuffer);
					}
					// 圧縮されたデータを取り出す
					while (!comp.finished()) {
						int count = comp.deflate(compBuffer);
						if (count == 0) {
							break;
						}
						// System.out.printf("圧縮されたデータを取り出す %dバイト\n", count);
						compBufferManger.add(compBuffer, count);
					}
					// 圧縮データを暗号化してファイルに書き込む
					while (compBufferManger.isAvailable()) {
						// System.out.println("圧縮されたデータを暗号化してファイルに書き込む");
						int count = compBufferManger.read(sourceBuffer);
						for (int j = 0; j < count; j++) {
							sourceBuffer[j] ^= chainBuffer[j];
						}
						blockCipher.processBlock(sourceBuffer, 0, encryptBuffer, 0);
						os.write(encryptBuffer); // ファイルへの書き込み
						System.arraycopy(encryptBuffer, 0, chainBuffer, 0, count);
						Arrays.fill(sourceBuffer, (byte) 0);
					}
				}
				// ファイルを閉じる
				ios.close();
				if (!silent) {
					System.out.printf(ResourceBundle.getBundle(RESOURCE).getString("MSG_ENCRYPT_FILE"),
							attr.getFileNameOnly());
				}
			}
			// 残っているデータを処理
			while (!readBufferManger.isFinished()) {
				readBufferManger.read(readBuffer);
				// 書き込む長さにデータの個数を用いると，Ver2.8以前のバージョンでおしりが切れる。
				// 後のデータは本来のデータではないが，そのまま圧縮に渡す。
				comp.setInput(readBuffer, 0, readBuffer.length);
			}
			comp.finish();
			while (!comp.finished()) {
				int count = comp.deflate(compBuffer);
				compBufferManger.add(compBuffer, count);
			}
			while (!compBufferManger.isFinished()) {
				int count = compBufferManger.read(sourceBuffer);
				if (count < BUF_SIZE_32) {
					byte padding = (byte) (BUF_SIZE_32 - count);
					for (int j = count; j < BUF_SIZE_32; j++) {
						sourceBuffer[j] = padding;
					}
				}
				for (int j = 0; j < BUF_SIZE_32; j++) {
					sourceBuffer[j] ^= chainBuffer[j];
				}
				blockCipher.processBlock(sourceBuffer, 0, encryptBuffer, 0);
				os.write(encryptBuffer); // ファイルへの書き込み
				System.arraycopy(encryptBuffer, 0, chainBuffer, 0, count);
				// Arrays.fill(sourceBuffer, (byte)0);
			}
		} catch (FileNotFoundException e) {
			// ファイル読み込み時に発生したエラーはこちらで処理
			String s = Confirm.resources.getString(MSG_ERROR_NOT_FILE_EXIST).replace('\n', ' ')
					+ Confirm.resources.getString(MSG_CONFIRM_CONTINUE);
			Object[] obj = { attr.getFileNameOnly() };
			int result = confirm.confirmYesNo(Utility.replacePlaceHolder(s, obj), false);
			if (result == Confirm.NO) {
				throw new Exception();
			}
		} catch (IOException e) {
			// ファイル書き込み時のエラー
		} finally {
			if (ios != null) {
				try {
					ios.close();
				} catch (IOException e) {
					// 何もしない
				}
			}
		}
		if (!silent) {
			confirm.showMessageByBundle("MSG_ENCRYPT_END");
		}
	}

	/**
	 * ファイルリストの情報を指定したByteArrayOutputStreamに書き込みます。
	 * 
	 * @param os
	 *            ByteArrayOutputStream
	 * @param list
	 *            ファイルリスト
	 * @throws Exception
	 *             書き込みに失敗した場合
	 */
	protected void addFileList(ByteArrayOutputStream os, List<FileAttribute> list) throws Exception {
		// 初期化ベクトル（IV）を生成して書き込む
		byte[] chainBuffer = getInitalizationVector();
		os.write(chainBuffer);
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		// 0:旧バージョン用に保存, 1:新バージョン用に保存
		for (int i = 0; i < 1; i++) {
			// パスコード
			bos.write(PASSCODE_ATTACHECASE.getBytes());
			// 日付
			bos.write(Utility.getDateTime(new Date()).getBytes());
			bos.write('\n');
			for (FileAttribute attr : list) {
				try {
					if (i == 1) {
						attr.setSuffix("U_");
					}
					String s = attr.toString();
					// System.out.println(s);
					bos.write(s.getBytes((i == 0) ? "Windows-31J" : "UTF-8"));
					bos.write('\r'); // 復帰
					bos.write('\n'); // 改行
				} catch (UnsupportedEncodingException e) {
					// 何もしない
				} catch (IOException e) {
					// TODO 例外発生時にどうするかは今後検討
				}
			}
		}
		// 暗号化して書き込んでゆく
		int pos = 0; // 処理対象の先頭の要素
		int headerSize = 0; // ヘッダのサイズ
		byte[] data = bos.toByteArray();
		bos.close();
		byte[] buffer = new byte[BUF_SIZE_32]; // 32バイトバッファ
		byte[] outBuffer = new byte[BUF_SIZE_32]; // 32バイトバッファ
		while (pos < data.length) { // データが存在する間繰り返す
			// バッファにコピーする長さを求める。
			int length = (pos + BUF_SIZE_32 > data.length) ? (data.length - pos) : BUF_SIZE_32;
			System.arraycopy(data, pos, buffer, 0, length); // バッファにコピー

			if (length < BUF_SIZE_32) {
				int padding = BUF_SIZE_32 - length;
				Arrays.fill(buffer, (byte) padding);
				System.arraycopy(data, pos, buffer, 0, length); // バッファにコピー
			}

			pos += BUF_SIZE_32; // 処理対象の先頭位置を移動させる
			// xor
			for (int i = 0; i < BUF_SIZE_32; i++) {
				buffer[i] ^= chainBuffer[i];
			}
			// 暗号化
			blockCipher.processBlock(buffer, 0, outBuffer, 0);
			// 書き込み
			os.write(outBuffer);
			headerSize += BUF_SIZE_32;
			// CBC＆バッファの初期化
			System.arraycopy(outBuffer, 0, chainBuffer, 0, BUF_SIZE_32);
			Arrays.fill(buffer, (byte) 0);

		}
		// ヘッダサイズの書き込み
		os.writeHeaderSize(headerSize);
		// System.out.printf("heder size = %d\n", headerSize);
	}

	/**
	 * 初期化ベクトルを指定したOutputStreamに書き込みます。<br>
	 * 初期化ベクトルの長さは256ビット（8ビット×32バイト）です。<br>
	 * 
	 * @param os
	 *            初期化ベクトルを書き込むOutputStream
	 * @return 初期化ベクトル256ビット（8ビット×32バイト）
	 */
	private byte[] getInitalizationVector() {
		byte[] buffer = null;
		try {
			KeyGenerator generator = KeyGenerator.getInstance("AES");
			SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
			generator.init(8 * BUF_SIZE_32, random);
			Key key = generator.generateKey();
			buffer = key.getEncoded();
		} catch (Exception e) {
			// 何もしない
		}
		return buffer;
	}

	/**
	 * 暗号化するファイルの情報を取得します。<br>
	 * 取得する情報は，暗号化ファイルごとにList<String>に格納されています。<br>
	 * 
	 * @return 暗号化するファイルの情報を格納したリストを要素とするリスト
	 */
	protected List<List<FileAttribute>> getFileLists() {
		List<List<FileAttribute>> lists = new ArrayList<List<FileAttribute>>();
		List<FileAttribute> list = null;
		// 暗号化するファイルの情報を取得する
		// 別々に暗号化するのか，まとめるのかはここで判断
		for (int i = 0; i < options.getEncryptFileNames().size(); i++) {
			if (i == 0 || !options.isAllFilePacked()) {
				if (list != null) {
					addIndexNumber(list);
					lists.add(list);
				}
				list = new ArrayList<FileAttribute>();
			}
			File f = new File(options.getEncryptFileNames().get(i));
			list.addAll(getFileInfomation(f, f.getParent()));
		}
		if (list != null) {
			addIndexNumber(list);
		}
		lists.add(list);
		return lists;
	}

	/**
	 * 与えられたリストの要素の先頭に「Fn_*」を追加する。
	 * 
	 * @param list
	 *            ファイルの情報が格納されたリスト
	 */
	private void addIndexNumber(List<FileAttribute> list) {
		// ファイル情報の先頭に「Fn_*」を付加する
		for (int i = 0; i < list.size(); i++) {
			list.get(i).setIndex(i);
		}
		/*
		 * if (DEBUG) { for (FileAttribute f : list) {
		 * System.out.println(f.toString()); } }
		 */
	}

	/**
	 * 暗号化ファイルが存在していないことを確認します。
	 * 
	 * @throws ProcessBreakException
	 *             確認画面でキャンセルが選択された場合
	 */
	private void checkFileNotExist() throws ProcessBreakException {
		// System.out.println(encryptFileName);
		File f = new File(encryptFileName);
		if (!f.exists()) {
			return;
		}
		if (options.isConfirmOverwirte() && !overwirteYesToAll) {
			Object[] values = { f.getAbsoluteFile() };
			String msg = Utility.replacePlaceHolder(
					ResourceBundle.getBundle(RESOURCE).getString(MSG_CONFIRM_OVER_WRITE_SAME_FILE), values);
			int result = confirm.confirmYesNoCancel(msg, false);
			switch (result) {
			case Confirm.CANCEL:
				throw new ProcessBreakException();
			case Confirm.ALL:
				overwirteYesToAll = true;
				break;
			case Confirm.NO:
				// 新しい名前をencryptFileNameに変更
				String anotherPath = confirm.inputFileName(f);
				if (anotherPath == null) {
					throw new ProcessBreakException();
				}
				encryptFileName = FilenameUtils.normalize(anotherPath);
			}
		}
		// System.out.println(encryptFileName);
	}

	private String getEncryptFileName(List<FileAttribute> list) {
		if (list.size() == 0) {
			return null;
		}
		// ATCファイル名を取得
		File f = new File(list.get(0).getFileNameOnly());
		String baseName = (options.isIncludeExtension()) ? f.getName() : FilenameUtils.getBaseName(f.getAbsolutePath());
		// 暗号化ファイル名にいろいろな情報を付加できるように
		// 拡張子の取得
		String extension = (options.isCamouflageExtention()) ? options.getExtensionString() : ATC_EXTENSION;
		// デフォルトの暗号化フォルダ
		File dir = f.getAbsoluteFile().getParentFile();
		// 常に同じフォルダーに暗号化するという指定が有る場合のことを考慮
		String encodeFolder = options.getEncodeFolderPath();
		if (options.isEncodeToSameFolder() && encodeFolder != null && encodeFolder.length() != 0) {
			dir = new File(encodeFolder);
		}
		f = new File(dir, baseName + extension);
		return FilenameUtils.normalize(f.getAbsolutePath());
	}

	/**
	 * ヘッダ情報が格納されたByteArrayOutputStreamを取得します。<br>
	 * 以下の情報が格納されています。 ●データサブバージョン<br>
	 * ●予約データ<br>
	 * ●ミスタイプ許容回数<br>
	 * ●ミスタイプ許容回数を超えた場合，ファイルを破壊するかどうか<br>
	 * ●トークン<br>
	 * ●データファイルバージョン<br>
	 * ●アルゴリズムタイプ<br>
	 * ●暗号化ヘッダーサイズ<br>
	 * 
	 * @return
	 */
	protected ByteArrayOutputStream createHeader() {
		ByteArrayOutputStream os = new ByteArrayOutputStream();

		// データサブバージョン : 1byte
		os.write(ATC_DATA_SUB_VERSION_6);
		// 予約データ(reserved） : 1byte
		os.write(0);
		// ミスタイプ回数 : 1byte
		os.write(options.getMaxMistake());
		// 破壊するか否か : 1byte
		os.write((options.isFileBrakeOption()) ? 1 : 0);
		// トークン : 16byte
		try {
			os.write(TOKEN_STRING.getBytes("Windows-31J"));
		} catch (Exception e) {
			// 何もしない
		}
		// データファイルバージョン : 4byte
		os.writeInt(ATC_DATA_FILE_VERSION);
		// アルゴリズムタイプ : 4byte
		os.writeInt(TYPE_ALGORISM_RIJNDAEL);
		// 暗号化部分のヘッダデータサイズ（先に確保しておく） ：4byte
		os.writeHeaderSize(0);

		return os;
	}

	/**
	 * ファイル情報を取得します。
	 * 
	 * @param f
	 *            ファイル情報を求めるファイルまたはフォルダ
	 * @return ファイル情報文字列のリスト
	 */
	protected List<FileAttribute> getFileInfomation(File f, String parentPath) {
		List<FileAttribute> list = new ArrayList<FileAttribute>();
		// 隠し属性のファイルの扱い
		if (f.isHidden() && !options.isIncludeHidden()) {
			return list;
		}
		FileAttribute attr = new FileAttribute();
		if (parentPath != null) {
			attr.setBase(Utility.includeTrailingPathDelimiter(parentPath));
		}
		if (f.isDirectory()) {
			attr.setFileSize(-1);
		} else {
			attr.setFileSize(f.length());
		}
		attr.setFileName(f.getPath());
		attr.setModifyedTimeStamp(f.lastModified());
		attr.setCreateTimeStamp(f.lastModified()); // 作成日時は取得できない
		// ファイル属性
		int permission = 0;
		if (!f.canWrite()) {
			permission = permission | 0x01;
		}
		if (f.getName().startsWith(".") && !Utility.isWindows()) {
			permission = permission | 0x02;
		}
		attr.setAttribute(permission);
		list.add(attr);
		// ディレクトリの場合は，その中の情報を追加する
		if (f.isDirectory()) {
			File[] files = f.listFiles();
			for (File dirFile : files) {
				String name = dirFile.getName();
				if (name.compareTo(".") == 0 || name.compareTo("..") == 0) {
					continue;
				}
				list.addAll(getFileInfomation(dirFile, parentPath));
			}
		}
		return list;
	}

	/**
	 * 復号用キーを取得します。
	 * 
	 * @return 復号用キー
	 * @throws InvalidKeyException
	 */
	/*
	 * private Object getKey() throws InvalidKeyException { byte[] keys =
	 * this.key; return rijndael.makeKey(keys, RIJNDAEL_BLOCK_SIZE); }
	 */

	/**
	 * 暗号化するファイルのサイズを取得します。
	 * 
	 * @param list
	 *            FileAttributeが格納されたリスト
	 * @return 総バイト数
	 */
	private long getAllTotalSize(List<FileAttribute> list) {
		long size = 0;
		for (FileAttribute attr : list) {
			// ディレクトリでなければ加算
			if (!attr.isDirectory()) {
				size += attr.getFileSize();
			}
		}
		return size;
	}

	private boolean isExistMyself(List<FileAttribute> list, String encryptFileName) {
		boolean exist = false;
		for (FileAttribute attr : list) {
			if (attr.getFileNameOnly().equals(encryptFileName)) {
				exist = true;
				break;
			}
		}
		return exist;
	}
}
