package org.hibara.attachecase.encrypt;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;

import org.bouncycastle.crypto.digests.GeneralDigest;
import org.bouncycastle.crypto.digests.SHA1Digest;
import org.bouncycastle.crypto.engines.RijndaelEngine;
import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
import org.bouncycastle.crypto.params.KeyParameter;
import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.ConsoleConfirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.ByteBufferManager;
import org.hibara.attachecase.entity.FileAttribute;
import org.hibara.attachecase.entity.FileAttribute3;
import org.hibara.attachecase.entity.ShaEntity;
import org.hibara.attachecase.exception.ProcessBreakException;
import org.hibara.attachecase.io.ByteArrayOutputStream;
import org.hibara.attachecase.io.ZipDeflateOutputStream;
import org.hibara.attachecase.utility.Digest;
import org.hibara.attachecase.utility.Utility;

/**
 * このクラスはアタッシェケース#3に対応した暗号化を行います。<br>
 * 暗号化したファイルには，デフォルトでは".atc"という拡張子が付きます。<br>
 * 
 * @author Masanori Uozumi
 * @version 2017.01.20<br>
 *          2017.02.19 パスワードファイルの扱いに関するバグを修正<br>
 *          2017.03.03 Windows上で文字化けが発生するバグを修正<br>
 */

public class FileEncrypt3 extends FileEncrypt {
	private int headerSizePosition = 0;
	private byte[] randomSalt = null;

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended
	 *            生成を中止
	 */
	public FileEncrypt3(boolean createSuspended) {
		super(createSuspended, new Options(), new ConsoleConfirm());
	}

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * <pre>
	 * ユーザーへの問い合わせはコンソールで行います。
	 * </pre>
	 * 
	 * @param createSuspended
	 *            生成を中止
	 * @param options
	 *            オプション
	 */
	public FileEncrypt3(boolean createSuspended, Options options) {
		super(createSuspended, options, new ConsoleConfirm());
	}

	/**
	 * 暗号化クラスを準備します。
	 * 
	 * @param createSuspended
	 *            生成を中止
	 * @param options
	 *            オプション
	 * @param confirm
	 *            ユーザー確認オブジェクト （問い合わせをコンソールで行うか，ダイアログで行うか）
	 */
	public FileEncrypt3(boolean createSuspended, Options options, Confirm confirm) {
		super(createSuspended, options, confirm);
	}

	/**
	 * ヘッダ情報が格納されたByteArrayOutputStreamを取得します。<br>
	 * 以下の情報が格納されています。 ●アプリケーションバージョン<br>
	 * ●ミスタイプ許容回数<br>
	 * ●ミスタイプ許容回数を超えた場合，ファイルを破壊するかどうか<br>
	 * ●トークン<br>
	 * ●データファイルバージョン<br>
	 * ●暗号化ヘッダーファイルサイズ<br>
	 * ●ランダムソルト<br>
	 * 
	 * @return
	 */
	@Override
	protected ByteArrayOutputStream createHeader() {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		byte[] header = { (byte) (ATTACHE_CASE_APP_VERSION & 0xff), (byte) ((ATTACHE_CASE_APP_VERSION >> 8) & 0xff),
				(byte) (options.getMaxMistake()), (byte) (options.isFileBrakeOption() ? 1 : 0), };

		try {
			// HEADER
			os.write(header);
			// TOKEN
			os.write(TOKEN_STRING.getBytes());
			// データファイルバージョン
			os.writeInt(ATC_DATA_FILE_VERSION_3);
			this.headerSizePosition = os.size();
			// 暗号化ヘッダーファイルサイズ
			os.writeInt(0);
			// ランダムソルト
			this.randomSalt = Utility.getRandomSalt(8);
			if (randomSalt.length == 8) {
				os.write(randomSalt);
			}
		} catch (Exception e) {
			// 何もしない
		}

		return os;
	}

	/**
	 * ファイル情報を取得します。
	 * 
	 * @param f
	 *            ファイル情報を求めるファイルまたはフォルダ
	 * @return ファイル情報文字列のリスト
	 */
	@Override
	protected List<FileAttribute> getFileInfomation(File f, String parentPath) {
		List<FileAttribute> list = new ArrayList<FileAttribute>();
		// 隠し属性のファイルの扱い
		if (f.isHidden() && !options.isIncludeHidden()) {
			return list;
		}
		FileAttribute3 attr = new FileAttribute3();
		if (parentPath != null) {
			attr.setBase(Utility.includeTrailingPathDelimiter(parentPath));
		}
		attr.setFileName(f.getPath());
		attr.setModifyedTimeStamp(f.lastModified());
		attr.setCreateTimeStamp(f.lastModified()); // 作成日時は取得できない
		if (f.isDirectory()) {
			attr.setDirectory();
		} else {
			attr.setFileSize(f.length());
		}
		// ファイル属性
		int permission = 0;
		if (!f.canWrite()) {
			permission = permission | 0x01;
		}
		if (f.getName().startsWith(".") && !Utility.isWindows()) {
			permission = permission | 0x02;
		}
		attr.setAttribute(permission);
		list.add(attr);
		// ディレクトリの場合は，その中の情報を追加する
		if (f.isDirectory()) {
			File[] files = f.listFiles();
			for (File dirFile : files) {
				String name = dirFile.getName();
				if (name.compareTo(".") == 0 || name.compareTo("..") == 0) {
					continue;
				}
				list.addAll(getFileInfomation(dirFile, parentPath));
			}
		}
		return list;
	}

	/**
	 * ファイルリストの情報を指定したByteArrayOutputStreamに書き込みます。
	 * 
	 * @param os
	 *            ByteArrayOutputStream
	 * @param list
	 *            ファイルリスト
	 * @throws Exception
	 *             書き込みに失敗した場合
	 */
	@Override
	protected void addFileList(ByteArrayOutputStream os, List<FileAttribute> list) throws Exception {
		
		//System.out.println(Arrays.toString(list.toArray()));
		blockCipher = new RijndaelEngine(RIJNDAEL_BLOCK_SIZE * 8);
		// rijndaelKey = getKey();

		// パスワードの生成
		GeneralDigest algorithm = new SHA1Digest();
		PKCS5S2ParametersGenerator gen = new PKCS5S2ParametersGenerator(algorithm);
		gen.init(this.key, this.randomSalt, 1000);
		byte[] dk = ((KeyParameter) gen.generateDerivedParameters(8 * 64)).getKey();
		this.aesKey = Arrays.copyOfRange(dk, 0, 32); // 前 半分
		this.aesIV = Arrays.copyOfRange(dk, 32, 64); // 後 半分
		blockCipher.init(true, new KeyParameter(this.aesKey));

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		// トークンの追加
		bos.write("atc3\n".getBytes());
		for (FileAttribute attr : list) {
			try {
				String s = attr.toString();
				bos.write(s.getBytes("UTF-8"));
				bos.write('\n'); // 改行
			} catch (UnsupportedEncodingException e) {
				// 何もしない
			} catch (IOException e) {
				// TODO 例外発生時にどうするかは今後検討
			}
		}
		// 暗号化して書き込んでゆく
		int pos = 0; // 処理対象の先頭の要素
		byte[] data = bos.toByteArray();
		bos.close();
		byte[] buffer = new byte[BUF_SIZE_32]; // 32バイトバッファ
		byte[] outBuffer = new byte[BUF_SIZE_32]; // 32バイトバッファ
		byte[] chainBuffer = Arrays.copyOf(this.aesIV, this.aesIV.length);
		while (pos < data.length) { // データが存在する間繰り返す
			// バッファにコピーする長さを求める。
			int length = (pos + BUF_SIZE_32 > data.length) ? (data.length - pos) : BUF_SIZE_32;

			if (length < BUF_SIZE_32) {
				// int padding = BUF_SIZE_32 - length;
				// Arrays.fill(buffer, (byte)padding);
				Arrays.fill(buffer, (byte) 0);
				System.arraycopy(data, pos, buffer, 0, length); // バッファにコピー
			} else {
				System.arraycopy(data, pos, buffer, 0, length); // バッファにコピー
			}

			pos += BUF_SIZE_32; // 処理対象の先頭位置を移動させる
			// xor
			for (int i = 0; i < BUF_SIZE_32; i++) {
				buffer[i] ^= chainBuffer[i];
			}
			// 暗号化
			blockCipher.processBlock(buffer, 0, outBuffer, 0);
			// 書き込み
			os.write(outBuffer);
			// CBC＆バッファの初期化
			System.arraycopy(outBuffer, 0, chainBuffer, 0, BUF_SIZE_32);
			Arrays.fill(buffer, (byte) 0);

		}
		// ヘッダサイズの書き込み
		os.writeIntToPosition(data.length, this.headerSizePosition);
	}

	/**
	 * 指定した出力ストリームに対して，ファイルを暗号化して書き込みます。
	 * 
	 * @param os
	 *            出力するストリーム
	 * @param list
	 *            暗号化するファイルの情報
	 * @throws Exception
	 *             書き込みに失敗した場合
	 */
	@Override
	protected void writeToAtcFile(OutputStream os, List<FileAttribute> list) throws Exception {
		long totalSize = 0;
		blockCipher = new RijndaelEngine(RIJNDAEL_BLOCK_SIZE * 8);
		blockCipher.init(true, new KeyParameter(this.aesKey));

		// 初期ベクトル設定
		byte[] chainBuffer = Arrays.copyOf(this.aesIV, this.aesIV.length);
		byte[] encryptBuffer = new byte[BUF_SIZE_32];
		// 圧縮の準備
		ByteBufferManager compBufferManger = new ByteBufferManager(BUF_SIZE_32); // 圧縮後のデータを格納しておき，32バイト毎に取り出す
		int length = 0; // 読込バイト数
		BufferedInputStream ios = null;
		FileAttribute attr = null;
		ZipDeflateOutputStream comp = null;
		try {
			comp = new ZipDeflateOutputStream(options.getCompressRate(), true);
			byte[] readBuffer = new byte[FILE_BUFFER_SIZE];
			byte[] sourceBuffer = new byte[BUF_SIZE_32];
			for (int i = 0; i < list.size(); i++) {
				attr = list.get(i);
				if (attr.isDirectory()) {
					continue;
				}
				if (listener != null) {
					listener.publishFileName(
							Confirm.resources.getString("MSG_ENCRYPT_CONTINUE") + attr.getFileNameOnly());
					// キャンセルされている場合は中止
					if (listener.isCancel()) {
						listener.publishFileName(Confirm.resources.getString("MSG_ENCRYPT_STOPPED"));
						throw new ProcessBreakException();
					}
				}
				ios = new BufferedInputStream(new FileInputStream(attr.getFileNameOnly()));
				// ファイルを読み込む
				while ((length = ios.read(readBuffer)) != -1) {
					totalSize += length;
					progress = (int) (100 * totalSize / allTotalSize);
					if (listener != null) {
						listener.publishProgress(progress);
					}
					comp.write(readBuffer, length);
					// 圧縮されたデータを取り出す
					while (comp.canRead()) {
						byte[] compressedData = comp.readAll();
						if (compressedData != null && compressedData.length != 0) {
							compBufferManger.add(compressedData);
						}
					}
					// 圧縮データを暗号化してファイルに書き込む
					while (compBufferManger.isAvailable()) {
						int count = compBufferManger.read(sourceBuffer);
						for (int j = 0; j < count; j++) {
							sourceBuffer[j] ^= chainBuffer[j];
						}
						blockCipher.processBlock(sourceBuffer, 0, encryptBuffer, 0);
						os.write(encryptBuffer); // ファイルへの書き込み
						Arrays.fill(sourceBuffer, (byte) 0);
						System.arraycopy(encryptBuffer, 0, chainBuffer, 0, count);
					}
				}
				// ファイルを閉じる
				ios.close();
				if (!silent) {
					System.out.printf(ResourceBundle.getBundle(RESOURCE).getString("MSG_ENCRYPT_FILE"),
							attr.getFileNameOnly());
				}
			}
			// 残っているデータを処理
			comp.finish();
			while (comp.canRead()) {
				byte[] compressedData = comp.readAll();
				if (compressedData.length != 0) {
					compBufferManger.add(compressedData);
				}
			}
			while (!compBufferManger.isFinished()) {
				int count = compBufferManger.read(sourceBuffer);
				if (count < BUF_SIZE_32) {
					// byte padding = (byte) (BUF_SIZE_32 - count);
					byte padding = (byte) 0;
					for (int j = count; j < BUF_SIZE_32; j++) {
						sourceBuffer[j] = padding;
					}
				}
				for (int j = 0; j < BUF_SIZE_32; j++) {
					sourceBuffer[j] ^= chainBuffer[j];
				}
				blockCipher.processBlock(sourceBuffer, 0, encryptBuffer, 0);
				os.write(encryptBuffer); // ファイルへの書き込み
				System.arraycopy(encryptBuffer, 0, chainBuffer, 0, count);
				// Arrays.fill(sourceBuffer, (byte)0);
			}
		} catch (FileNotFoundException e) {
			// ファイル読み込み時に発生したエラーはこちらで処理
			String s = Confirm.resources.getString(MSG_ERROR_NOT_FILE_EXIST).replace('\n', ' ')
					+ Confirm.resources.getString(MSG_CONFIRM_CONTINUE);
			Object[] obj = { attr.getFileNameOnly() };
			int result = confirm.confirmYesNo(Utility.replacePlaceHolder(s, obj), false);
			if (result == Confirm.NO) {
				throw new Exception();
			}
		} catch (IOException e) {
			// ファイル書き込み時のエラー
			// e.printStackTrace();
		} finally {
			if (ios != null) {
				try {
					ios.close();
				} catch (IOException e) {
					// 何もしない
				}
			}
			if (comp != null) {
				comp.close();
			}
		}
		if (!silent) {
			confirm.showMessageByBundle("MSG_ENCRYPT_END");
		}
	}

	/**
	 * パスワードの設定
	 */
	@Override
	public void setPassword(String password) {
		setPasswordByUTF8(password);
	}

	/**
	 * SHA-256のハッシュを作成しなおします。<br>
	 * 引数はダミーです。<br>
	 */
	@Override
	public void setSha(ShaEntity sha) {
		Digest digest = new Digest();
		try {
			setPassword(digest.getSha256(options.getDecryptPasswordFileName()));
		} catch (IOException e) {
			// 何もしない
		}
	}
}
