package org.hibara.attachecase.entity;

import java.util.Iterator;
import java.util.LinkedList;

import org.hibara.attachecase.AttacheCaseConstant;

/**
 * FIFOバッファを構成します。<br>
 * 
 * @author Masanori Uozumi
 * @version 2017.01.17 poolのクラスをLinkedListに変更
 *
 */
public class ByteBufferManager {

	private LinkedList<Byte> pool;
	private int maxLength = AttacheCaseConstant.BUF_SIZE_32;
	private int totalLength = 0;

	/**
	 * コンストラクタ
	 * 
	 * @param maxLength
	 *            一度に読み出す最大データ長を指定
	 */
	public ByteBufferManager(int maxLength) {
		this.maxLength = maxLength;
		pool = new LinkedList<Byte>();
	}

	/**
	 * データを追加します。
	 * 
	 * @param data
	 *            追加するデータを格納したバイト配列
	 */
	public void add(byte[] data) {
		add(data, data.length);
		totalLength += data.length;
	}

	/**
	 * データを追加します。
	 * 
	 * @param data
	 *            追加するデータを格納したバイト配列
	 * @param length
	 *            追加するデータ数
	 */
	public void add(byte[] data, int length) {
		for (int i = 0; i < length; i++) {
			pool.add(data[i]);
		}
		totalLength += length;
	}

	/**
	 * 引数で渡したバイト配列にデータを読み出します。<br>
	 * 読み出したデータはバッファーから削除されます。<br>
	 * 
	 * @param readBuffer
	 *            読み出したデータを格納するバイト配列
	 * @return 読み出したデータ数
	 */
	public int read(byte[] readBuffer) {
		int count = 0;
		Iterator<Byte> it = pool.iterator();
		while (it.hasNext()) {
			readBuffer[count++] = it.next();
			it.remove();
			if (count == maxLength) {
				break;
			}
		}
		return count;
	}

	/**
	 * 引数で渡したバイト配列に指定した長さのデータを読み出します。<br>
	 * 読み出したデータはバッファーから削除されます。<br>
	 * 
	 * @param readBuffer
	 *            読み出したデータを格納するバイト配列
	 * @param length
	 *            読み出すデータ数。maxLengthで制限される。
	 * @return 読み出したデータ長
	 */
	public int read(byte[] readBuffer, int length) {
		int count = 0;
		Iterator<Byte> it = pool.iterator();
		while (it.hasNext()) {
			readBuffer[count++] = it.next();
			it.remove();
			if (count == maxLength || count == length) {
				break;
			}
		}
		return count;
	}

	/**
	 * maxLength以上の読み出し可能なデータがあるかどうかを判定します。
	 * 
	 * @return 読み出し可能であればtrue
	 */
	public boolean isAvailable() {
		return (pool.size() >= maxLength);
	}

	/**
	 * すべてのデータの読み出しが完了したかどうかを判定します。
	 * 
	 * @return データの読み出しが完了して，読み出すデータが存在しない場合はtrue
	 */
	public boolean isFinished() {
		return (pool.size() == 0);
	}

	/**
	 * バッファに書き込まれたデータ数を取得します。
	 * 
	 * @return バッファに書き込まれたデータ数
	 */
	public int getTotalLength() {
		return totalLength;
	}
}
