/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.entity;

import java.io.File;
import java.util.TimeZone;

import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.utility.Utility;

/**
 * 暗号化されたファイルに関する情報を格納するためのクラスです。
 * @author Masanori Uozumi
 * @version 2017.01.14<br>
 * 2012.08.07 メソッド追加<br>
 * 2012.08.09 toStringメソッドをオーバーライド。暗号化の際，ファイルリストを作成するために使用。<br>
 * 2012.08.11 絶対パスが指定された場合のtoStringメソッドの動作を変更。<br>
 * 2013.06.04 パスのBase部分を削除するロジックを変更（Windows環境でgetFileNameOnlyメソッドがドライブ部分を削除することに対応）。<br>
 * 2017.01.14 サブクラス作成のため変数とメソッドのスコープを変更
 */
public class FileAttribute {
	protected String fileName;	// ファイル名
	protected int attribute;		// ファイル属性
	protected double modifiedDate;	// 変更日付 西暦1年1月1日からの暦日数
	protected double modifiedTime;	// 変更時間 午前 0 時から経過したミリ秒数
	protected double createDate;	// 作成日付 西暦1年1月1日からの暦日数
	protected double createTime;	// 作成時間 午前 0 時から経過したミリ秒数
	protected long fileSize;		// ファイルサイズ
	protected int index;			// 暗号化する際に作成するファイルリストに付加する番号
	protected String suffix = "Fn_";	// ファイルリスト作成時に付加するサフィックス
	protected String base;		// 起点となるディレクトリ
	protected String checksum = "";	// チェックサム
	
	/**
	 * FileAttributeクラスを構成します。
	 */
	public FileAttribute() {
		super();
	}

	/**
	 * FileAttributeクラスを構成します。
	 * @param s 各種情報を含んだ文字列（区切り文字はTAB）
	 */
	public FileAttribute(String s) {
		/*
		if (AttacheCaseConstant.DEBUG) {
			System.out.println(s);
		}
		*/
		String[] elements = s.split("\t");
		if (elements.length < 7) {
			return;
		}
		
		int i = 0;
		setFileName(elements[i++]);
		fileSize = parseInt(elements[i++]);
		attribute = parseInt(elements[i++]);
		modifiedDate = parseDouble(elements[i++]);
		modifiedTime = parseDouble(elements[i++]);
		createDate = parseDouble(elements[i++]);
		createTime = parseDouble(elements[i++]);
		
		// チェックサム
		if (elements.length > 7) {
			checksum = elements[i++];
		}
	}

	/**
	 * 文字列をint型の数値に変換します。
	 * @param s 文字列
	 * @return 文字列を変換した数値。変換できなかった場合は-1。
	 */
	protected int parseInt(String s) {
		int value = 0;
		try {
			value = Integer.parseInt(s);
		} catch (NumberFormatException e) {
			value = -1;
		}
		return value;
	}

	/**
	 * 文字列をdouble型の数値に変換します。
	 * @param s 文字列
	 * @return 文字列を変換した数値。変換できなかった場合は-1。
	 */
	protected double parseDouble(String s) {
		double value = 0;
		try {
			value = Double.parseDouble(s);
		} catch (NumberFormatException e) {
			value = -1.0;
		}
		return value;
	}

	public String getFileName() {
		return fileName;
	}

	/**
	 * ファイル名だけを取得します。
	 * @return ファイル名
	 */
	public String getFileNameOnly() {
		int pos = fileName.indexOf(':');
		// ":"が含まれていない場合は，文字列全体を取得
		if (pos == -1) {
			return Utility.rejectPathTraversal(fileName);
		}
		if (pos == fileName.length() - 1) {
			return "";
		}
		return Utility.rejectPathTraversal(fileName.substring(pos+1));
	}

	public void setFileName(String fileName) {
		fileName = fileName.replace('\\', File.separatorChar);	// パス区切り文字の変換 
		this.fileName = fileName;
	}

	public int getAttribute() {
		return attribute;
	}
	public void setAttribute(int attribute) {
		this.attribute = attribute;
	}
	public double getModifiedDate() {
		return modifiedDate;
	}
	public void setModifiedDate(double modifiedDate) {
		this.modifiedDate = modifiedDate;
	}
	public double getModifiedTime() {
		return modifiedTime;
	}
	public void setModifiedTime(double modifiedTime) {
		this.modifiedTime = modifiedTime;
	}
	public double getCreateDate() {
		return createDate;
	}
	public void setCreateDate(double createDate) {
		this.createDate = createDate;
	}
	public double getCreateTime() {
		return createTime;
	}
	public void setCreateTime(double createTime) {
		this.createTime = createTime;
	}

	public long getFileSize() {
		return fileSize;
	}

	public void setFileSize(long fileSize) {
		this.fileSize = fileSize;
	}

	public boolean isDirectory() {
		return fileSize == -1;
	}

	public boolean isFolder() {
		return isDirectory();
	}

	public String getBase() {
		return base;
	}

	public void setBase(String base) {
		this.base = base;
	}

	/**
	 * 変更日時を設定します。
	 * @param date 変更日時
	 */
	public void setModifyedTimeStamp(long date) {
		TimeZone tz = TimeZone.getDefault();
		date -= (24 * 3600 * 1000L + tz.getRawOffset()); // 1日 ＋ タイムゾーンを引く
		modifiedDate = (int)((date - AttacheCaseConstant.START_OF_DATE) / (1000L * 3600L * 24));
		modifiedTime = (int)((date - AttacheCaseConstant.START_OF_DATE) % (1000L * 3600L * 24));
	}

	/**
	 * 作成日時を設定します。
	 * @param date 作成日時
	 */
	public void setCreateTimeStamp(long date) {
		TimeZone tz = TimeZone.getDefault();
		date -= (24 * 3600 * 1000L + tz.getRawOffset()); // 1日 ＋ タイムゾーンを引く
		createDate = (int)((date - AttacheCaseConstant.START_OF_DATE) / (1000L * 3600L * 24));
		createTime = (int)((date - AttacheCaseConstant.START_OF_DATE) % (1000L * 3600L * 24));
	}

	/**
	 * 保持している情報をタブ区切り文字列として取得します。<br>
	 * ただし先頭の「Fn_*:」という文字列は付加しません。必要に応じて後から追加してください。
	 * @return タブで区切られた情報
	 */
	public String getAttributeByString() {
		StringBuilder sb = new StringBuilder();
		if (!fileName.startsWith("Fn_") && !fileName.startsWith("U_") && suffix != null) {
			sb.append(suffix);
		}
		sb.append(index);
		sb.append(':');
		String name = rejectPathTraversal(getFileNameOnly());
		// OSがWindowsでなければ，ファイル名のファイル区切り文字を¥に変更する
		if (!Utility.isWindows()) {
			name = name.replace(File.separatorChar, '\\');
			if (base != null) {
				base = base.replace(File.separatorChar, '\\');
			}
		}
		File f = new File(name);
		// base部分を削除
		if (base != null) {
			String baseWithOutDrive = "";
			if (Utility.isWindows() && base.indexOf(':') != -1) {
				baseWithOutDrive = base.substring(base.indexOf(':') + 1);
			} else {
				baseWithOutDrive = base;
			}
			if (name.startsWith(baseWithOutDrive)) {
				name = name.substring(baseWithOutDrive.length());
			}
		}
		if (f.isAbsolute()) {
			if (Utility.isWindows()) {
				int pos = name.indexOf('\\', 0);
				while (name.charAt(pos + 1) == '\\') {
					pos = name.indexOf('\\', pos + 1);
				}
				name = name.substring(pos + 1);
			} else {
				// Windows以外の場合は，先頭の1文字を除外
				name = name.substring(name.indexOf('/') + 1);
			}
		} else {
			// 先頭のパスを削除する
			if (name.startsWith("..\\")) {
				name = name.substring(3);
			} else if (name.startsWith(".\\")) {
				name = name.substring(2);
			}
		}
		// ディレクトリの場合は最後に¥を付ける
		sb.append(name);
		if (isDirectory()) {
			sb.append("\\\t*\t");	// ファイルサイズも一緒にくっつける
		} else {
			sb.append('\t');
			sb.append(fileSize);	// ファイルサイズ
			sb.append('\t');
		}
		// 属性
		sb.append((isDirectory()) ? 16 : attribute & 32);
		sb.append('\t');
		// 更新日時
		sb.append((int)modifiedDate);
		sb.append('\t');
		sb.append((int)modifiedTime);
		sb.append('\t');
		// 作成日時
		sb.append((int)createDate);
		sb.append('\t');
		sb.append((int)createTime);
		sb.append('\t');
		return sb.toString();
	}

	@Override
	public String toString() {
		return getAttributeByString();
	}

	public int getIndex() {
		return index;
	}

	public void setIndex(int index) {
		this.index = index;
	}

	public String getSuffix() {
		return suffix;
	}

	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}

	public String getChecksum() {
		return checksum;
	}

	/**
	 * パストラバーサルの脆弱性を防ぐため，先頭の"../" or "./"を削除する
	 * @param path頭に
	 * @return
	 */
	protected String rejectPathTraversal(String path) {
		while (path.startsWith("../") || path.startsWith("./")) {
			if (path.startsWith("../")) {
				path = path.substring("../".length());
			} else if (path.startsWith("./")) {
				path = path.substring("./".length());
			}
		}
		return path;
	}
}
