package org.hibara.attachecase.entity;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.hibara.attachecase.utility.Digest;
import org.hibara.attachecase.utility.Utility;

/**
 * 暗号化されたファイルに関する情報を格納するためのクラスです。<br>
 * アタッシェケース#3用
 * 
 * @author Masanori Uozumi
 * @version 2017.01.22<br>
 */

public class FileAttribute3 extends FileAttribute {

	public final static String FILE_SEPARATER = "" + File.separatorChar;
	
	/**
	 * FileAttributeクラスを構成します。
	 */
	public FileAttribute3() {
		super();
	}

	/**
	 * FileAttributeクラスを構成します。
	 * 
	 * @param s
	 *            各種情報を含んだ文字列（区切り文字はTAB）
	 */
	public FileAttribute3(String s) {
		super(s);
	}

	/**
	 * ディレクトリかどうか
	 * 
	 * @return ディレクトリの場合は true
	 */
	@Override
	public boolean isDirectory() {
		return this.fileName.endsWith(FILE_SEPARATER) && this.fileSize == 0;
	}

	/// ディレクトを設定
	public void setDirectory() {
		this.fileSize = 0;
		if (!this.fileName.endsWith(FILE_SEPARATER)) {
			this.fileName += FILE_SEPARATER;
		}
	}

	/**
	 * 変更日時を設定します。
	 * 
	 * @param date
	 *            変更日時
	 */
	@Override
	public void setModifyedTimeStamp(long date) {
		double[] dateAndTime = this.calcTimeStamp(date);
		modifiedDate = (int) dateAndTime[0];
		modifiedTime = dateAndTime[1];
	}

	/**
	 * 作成日時を設定します。
	 * 
	 * @param date
	 *            作成日時
	 */
	@Override
	public void setCreateTimeStamp(long date) {
		double[] dateAndTime = this.calcTimeStamp(date);
		createDate = (int) dateAndTime[0];
		createTime = dateAndTime[1];
	}

	/**
	 * 日付を計算します。
	 * 
	 * @param date
	 *            日付のlong値
	 * @return double型の配列。0番目の要素が日付，1番目の要素が時刻
	 */
	private double[] calcTimeStamp(long date) {
		Date d = new Date(date);
		double[] dateAndTime = { 0.0, 0.0 };
		try {
			SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
			String dayOnlyStr = df.format(d);
			Date dayOnly = df.parse(dayOnlyStr);
			double time = (date - dayOnly.getTime()) / 1000.0;
			double day = dayOnly.getTime() / (3600 * 24 * 1000) - Utility.get00010101Time() / (3600 * 24 * 1000) - 1;
			dateAndTime[0] = day;
			dateAndTime[1] = time;
		} catch (Exception e) {
			e.printStackTrace();
		}

		return dateAndTime;
	}

	/**
	 * 保持している情報をタブ区切り文字列として取得します。<br>
	 * 
	 * @return タブで区切られた情報
	 */
	@Override
	public String getAttributeByString() {
		StringBuilder sb = new StringBuilder();
		sb.append(index);
		sb.append(':');
		String name = getFileNameOnly();
		File f = new File(name);
		// OSがWindowsでなければ，ファイル名のファイル区切り文字を¥に変更する
		name = name.replace(File.separatorChar, '\\');
		if (base != null) {
			base = base.replace(File.separatorChar, '\\');
		}
		// System.out.println("name:" + name);
		// base部分を削除
		if (base != null) {
			String baseWithOutDrive = "";
			if (Utility.isWindows() && base.indexOf(':') != -1) {
				baseWithOutDrive = base.substring(base.indexOf(':') + 1);
			} else {
				baseWithOutDrive = base;
			}
			if (name.startsWith(baseWithOutDrive)) {
				name = name.substring(baseWithOutDrive.length());
			}
		}
		if (f.isAbsolute()) {
			if (Utility.isWindows()) {
				int pos = name.indexOf('\\', 0);
				while (name.charAt(pos + 1) == '\\') {
					pos = name.indexOf('\\', pos + 1);
				}
				name = name.substring(pos + 1);
			} else {
				// Windows以外の場合は，先頭の1文字を除外
				name = name.substring(name.indexOf('/') + 1);
			}
		} else {
			// 先頭のパスを削除する
			if (name.startsWith("..\\")) {
				name = name.substring(3);
			} else if (name.startsWith(".\\")) {
				name = name.substring(2);
			}
		}
		// ディレクトリの場合は最後に¥を付ける
		sb.append(name);
		if (isDirectory()) {
			sb.append("\t0\t"); // ファイルサイズも一緒にくっつける
		} else {
			sb.append('\t');
			sb.append(fileSize); // ファイルサイズ
			sb.append('\t');
		}
		// 属性
		sb.append((isDirectory()) ? 48 : attribute & 32);
		sb.append('\t');
		// 更新日時
		sb.append((int) modifiedDate);
		sb.append('\t');
		sb.append(modifiedTime);
		sb.append('\t');
		// 作成日時
		sb.append((int) createDate);
		sb.append('\t');
		sb.append(createTime);
		if (this.isDirectory()) {
			return sb.toString();
		}
		sb.append('\t');
		// チェックサム
		Digest digest = new Digest();
		checksum = digest.getSha256ByString(this.fileName);
		sb.append(checksum);

		return sb.toString();
	}

}
