/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.entity;

import java.io.IOException;

import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.exception.FileDestroyedException;
import org.hibara.attachecase.exception.NotEncryptFileException;
import org.hibara.attachecase.io.RandomAccessFile;

/**
 * 暗号化ファイル先頭の情報テーブルを扱うためのクラスです。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.16<br>
 *          2012.08.06 メンバ変数名とメソッド名を変更<br>
 *          2013.02.04 実行形式ファイルの判定部分のバグを修正（seekFromEndの引数間違い）。<br>
 *          2017.01.16 アタッシェケース#3対応のためのメソッド変更と追加
 */
public class InfoTable implements AttacheCaseConstant {
	private RandomAccessFile fs;

	private int planeHeaderSize; // 平文ヘッダサイズ（データサブバージョン、破壊設定など）
	private int encryptHeaderSize; // 暗号化部分のヘッダサイズ
	private byte token[] = new byte[16];
	private int dataVersion; // ver.2.00〜は "5", ver.2.70〜は "6"
	private int algorismType; //
	private long allTotalSize;
	private long totalSize;
	private int typeLimits; // パスワード入力回数の上限
	private boolean fileDestory; // パスワードを間違えたときにファイルを破壊するかどうか

	private byte[] randomSalt = new byte[8]; // ランダムソルト Ver3.0で使用

	private boolean executable = false; // 実行ファイル
	private boolean broken = false; // 破壊済み
	private boolean existToken = false; // トークンが存在しているかどうか

	private long headerTop = 0;
	private long lastPointer = 0;
	private int appVersion = 0;

	/**
	 * コンストラクタ
	 * 
	 * @param fs
	 *            ATCファイルを読み込むためのRandomAccessFile
	 * @throws Exception
	 *             ファイル操作に失敗した場合
	 */
	public InfoTable(RandomAccessFile fs) throws Exception {
		super();

		this.fs = fs;
		if (fs != null) {
			// ファイルの最終位置を取得
			long currentPointer = fs.getFilePointer();
			fs.seekFromEnd(0);
			this.lastPointer = fs.getFilePointer();
			fs.seek(currentPointer);

			// トークンの存在を確認
			this.checkFile(fs);
			fs.seek(currentPointer);
		}
	}

	public int getTypeLimits() {
		return typeLimits;
	}

	public int getPlaneHeaderSize() {
		return planeHeaderSize;
	}

	public void setPlaneHeaderSize(int planeHeaderSize) {
		this.planeHeaderSize = planeHeaderSize;
	}

	public int getEncryptHeaderSize() {
		return encryptHeaderSize;
	}

	public void setEncryptHeaderSize(int encryptHeaderSize) {
		this.encryptHeaderSize = encryptHeaderSize;
	}

	public byte[] getToken() {
		return token;
	}

	public void setToken(byte[] token) {
		this.token = token;
	}

	public int getDataVersion() {
		return dataVersion;
	}

	public void setDataVersion(int dataVersion) {
		this.dataVersion = dataVersion;
	}

	public int getAlgorismType() {
		return algorismType;
	}

	public void setAlgorismType(int algorismType) {
		this.algorismType = algorismType;
	}

	public long getAllTotalSize() {
		return allTotalSize;
	}

	public void setAllTotalSize(long allTotalSize) {
		this.allTotalSize = allTotalSize;
	}

	public long getTotalSize() {
		return totalSize;
	}

	public void setTotalSize(long totalSize) {
		this.totalSize = totalSize;
	}

	/**
	 * トークン配列が"_AttacheCaseData"に等しいかどうかをチェックします。
	 * 
	 * @return 等しい場合はtrue，そうでない場合はfalse
	 */

	public boolean isMatchToken() {
		return existToken;
	}

	/**
	 * トークン配列が指定した文字列に等しいかどうかをチェックします。
	 * 
	 * @param str チェックする文字列
	 * @return 等しい場合はtrue，そうでない場合はfalse
	 */
	public boolean isMatchToken(String str) {
		if (str == null || str.length() != token.length) {
			return false;
		}
		boolean match = true;
		for (int i = 0; i < token.length; i++) {
			byte ch = token[i];
			if (str.charAt(i) != (char) ch) {
				match = false;
				break;
			}
		}
		return match;
	}

	/**
	 * ヘッダ部分をファイルから読み込みます。
	 * 
	 * @throws Exception
	 *             ファイルが暗号化ファイルではない，あるいは破壊されている場合
	 */
	public void readFromFile() throws Exception {
		if (!existToken) {
			throw new NotEncryptFileException();
		}

		if (broken) {
			throw new FileDestroyedException();
		}

		// ヘッダ位置に合わせる
		fs.seek(headerTop);

		long work = fs.readIntFromByte();
		if (work > Integer.MAX_VALUE) {
			throw new NotEncryptFileException();
		}
		planeHeaderSize = (int) work;

		fs.read(this.token);

		// データバージョン 105はVer2.0 130はVer3.0
		work = fs.readIntFromByte();
		if (work > Integer.MAX_VALUE) {
			throw new NotEncryptFileException();
		}

		this.dataVersion = (int) work;
		if (work < ATC_DATA_FILE_VERSION_3) {
			// バージョン2
			this.readFromVersion2();
		} else if (work >= ATC_DATA_FILE_VERSION_3 && dataVersion < ATC_DATA_FILE_VERSION_MAX) {
			// バージョン3
			this.readFromVersion3();
		}
	}

	/**
	 * アタッシェケース#3のヘッダを読み込みます。
	 * 
	 * @throws NotEncryptFileException
	 * @throws IOException
	 */
	private void readFromVersion3() throws NotEncryptFileException, IOException {
		// ヘッダサイズを読み込む
		long work = fs.readIntFromByte();
		if (work > Integer.MAX_VALUE) {
			throw new NotEncryptFileException();
		}
		this.encryptHeaderSize = (int) work;

		// ランダムソルト
		fs.read(this.randomSalt, 0, 8);

		// パスワードの入力リミット
		this.typeLimits = this.getTypeLimits();
		// ファイル削除
		this.fileDestory = this.getDestoryFlagFromPlaneHeaderSize();
		// 有効範囲（1〜10）かチェック
		if (typeLimits < 1 || typeLimits > 10) {
			typeLimits = 3;
		}
		// アプリのバージョン情報
		this.getAppVersion();

	}

	private int getAppVersion() {
		this.appVersion = this.planeHeaderSize & 0xffff;
		return this.appVersion;

	}

	/**
	 * アタッシェケースVer.2のヘッダを読み込みます。
	 * 
	 * @throws NotEncryptFileException
	 * @throws IOException
	 */
	private void readFromVersion2() throws NotEncryptFileException, IOException {
		// アルゴリズムタイプ
		this.algorismType = (int) fs.readIntFromByte();
		// ヘッダサイズを読み込む
		long work = fs.readIntFromByte();
		if (work > Integer.MAX_VALUE) {
			throw new NotEncryptFileException();
		}
		this.encryptHeaderSize = (int) work;

		// データサブバージョンチェック（ver.2.70〜）
		if (this.getDataSubversionFromPlaneHeaderSize() == ATC_DATA_SUB_VERSION_6) {
			this.typeLimits = this.getTypeLimits();
			this.fileDestory = this.getDestoryFlagFromPlaneHeaderSize();
			// 有効範囲（1〜10）かチェック
			if (typeLimits < 1 || typeLimits > 10) {
				typeLimits = 3;
			}
		} else {
			this.typeLimits = 3;
			this.fileDestory = false;
		}
	}

	/**
	 * ファイルがATCファイルかどうかをチェックします。
	 * 
	 * @param fs
	 * @throws IOException
	 */
	private void checkFile(RandomAccessFile fs) throws IOException {
		byte data = fs.readByte();
		byte[] token = AttacheCaseConstant.TOKEN_STRING.getBytes();
		byte[] brokenToken = AttacheCaseConstant.DESTROY_TOKEN_STRING.getBytes();
		while (true) {
			if (data == token[0]) {
				this.existToken = true;
				for (int i = 1; i < token.length; i++) {
					data = fs.readByte();
					if (data != token[i]) {
						this.existToken = false;
						break;
					}
				}
				// 実行形式かどうか
				if (this.existToken) {
					if (fs.getFilePointer() > 20) {
						executable = true;
						this.headerTop = fs.getFilePointer() - 20;
					}
					break;
				}
			}
			// 削除データかどうか
			if (!existToken && data == brokenToken[0]) {
				existToken = true;
				for (int i = 1; i < brokenToken.length; i++) {
					data = fs.readByte();
					if (data != brokenToken[i]) {
						existToken = false;
						break;
					}
				}

				if (existToken) {
					if (fs.getFilePointer() > 20) {
						executable = true;
						headerTop = fs.getFilePointer() - 20;
					}
					broken = true;
				}
			}
			// ファイルポインターの制御
			long filePointer = fs.getFilePointer();
			if (filePointer > 0x200 && filePointer < 0x3600) {
				if (lastPointer > 0x36000) {
					fs.seek(0x36000);
				} else {
					break;
				}
			} else if (filePointer >= 0x37000 && filePointer < 0x7f600) {
				if (lastPointer > 0x7f600) {
					fs.seek(0x7f600);
				} else {
					break;
				}
			} else if (filePointer > 0x80000) {
				break;
			}
			data = fs.readByte();
		}
	}

	/**
	 * ATCファイルのヘッダーを読み込みます。<br>
	 * アタッシェケースVer.2専用であり，アタッシェケース#3には対応していません。<br>
	 * 
	 * @param fs
	 *            RandomAccessFile
	 * @throws IOException
	 *             ファイル操作に失敗した場合
	 * @deprecated 引数のないreadFromFileを使用してください。
	 */

	@Deprecated
	public void readFromFile(RandomAccessFile fs) throws IOException {
		if (fs == null) {
			return;
		}

		// 平文ヘッダサイズ（データサブバージョン、破壊設定など）
		planeHeaderSize = (int) (fs.readIntFromByte());
		// トークンの取得
		fs.read(token);
		// トークンのチェック
		if (!isMatchToken()) {
			// サイズを再取得
			fs.seekFromEnd(-SIZE_OF_LONG);
			allTotalSize = fs.readLongFromByte();
			// 位置を戻す
			fs.seekFromEnd(-(allTotalSize + SIZE_OF_LONG));
			// もう一度、平文ヘッダサイズを読み込む
			planeHeaderSize = (int) (fs.readIntFromByte());
			// もう一度、トークンを取得
			fs.read(token);
		}
		// データバージョン
		dataVersion = (int) (fs.readIntFromByte());
		// アルゴリズムタイプ
		algorismType = (int) (fs.readIntFromByte());
		// ヘッダサイズを読み込む
		encryptHeaderSize = (int) (fs.readIntFromByte());
		// System.out.println("encryptHeaderSize : " + encryptHeaderSize);
		// データサブバージョンチェック（ver.2.70〜）
		if (getDataSubversionFromPlaneHeaderSize() == ATC_DATA_SUB_VERSION_6) {
			typeLimits = getTypeLimitsFromPlaneHeaderSize();
			fileDestory = getDestoryFlagFromPlaneHeaderSize();
			// 有効範囲（1〜10）かチェック
			if (typeLimits < 1 || 10 < typeLimits) {
				typeLimits = 3; // デフォルト
			}
		} else {
			typeLimits = 3;
			fileDestory = false;
		}
	}

	public boolean isFileDestroy() {
		return fileDestory;
	}

	public void setFileDestroy(boolean fDestroy) {
		this.fileDestory = fDestroy;
	}

	public void setTypeLimits(int typeLimits) {
		this.typeLimits = typeLimits;
	}

	private int getTypeLimitsFromPlaneHeaderSize() {
		int work = planeHeaderSize & (0x00ff0000);
		work = (work >>> 16) & 0x000000ff;
		return work;
	}

	/**
	 * データサブバージョンの取得
	 * 
	 * @return int サブバージョン
	 */
	public int getDataSubversionFromPlaneHeaderSize() {
		int work = planeHeaderSize & (0xff);
		return work;
	}

	private boolean getDestoryFlagFromPlaneHeaderSize() {
		int work = planeHeaderSize & (0xff000000);
		work = (work >>> 24) & 0x000000ff;
		return work != 0;
	}

	public boolean isExecutable() {
		return executable;
	}

	public void setExecutable(boolean executable) {
		this.executable = executable;
	}

	public boolean isExistToken() {
		return existToken;
	}

	public void setExistToken(boolean existToken) {
		this.existToken = existToken;
	}

	public byte[] getRandomSalt() {
		return randomSalt;
	}

	public long getHeaderTop() {
		return headerTop;
	}
}
