/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.DialogConfirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.io.FileDeletion;
import org.hibara.attachecase.utility.Utility;

import com.jgoodies.binding.adapter.BasicComponentFactory;
import com.jgoodies.binding.beans.BeanAdapter;

/**
 * このクラスは，アタッシェケースのオプション設定用ダイアログです。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.20<br>
 *          2012.09.01 文字列をリソースファイルから取得するように変更。<br>
 *          2012.11.10 ラベルの間違いを訂正。<br>
 *          2014.07.24 パスワードを保存する場合の処理を修正。<br>
 *          2017.01.20 暗号化する際のバージョン設定用オプションを追加。<br>
 */

public class OptionDialog extends JDialog {
	private JFrame parent = null;
	private JDialog thisDialog = null;
	private static final long serialVersionUID = -4017491646731602178L;
	private Options options = null;
	private JButton okButton;
	private JPasswordField encryptPasswordField;
	private JPasswordField decryptPasswordField;
	private JTextField encodePathField;
	private JTextField decodePathField;
	private JTextField extentionField;
	private JTextField encryptPasswordFileField;
	private JTextField decryptPasswordFileField;
	private JTextField camouflageExtentionField;
	private JCheckBox chkbxDeleteOriginal;
	private JCheckBox chkbxDeleteEncrypt;

	private boolean canceled = false;

	/**
	 * OptionDialogを構成します。
	 */
	public OptionDialog() {
		this(null, null);
	}

	/**
	 * OptionDialogを構成します。
	 * 
	 * @param owner
	 *            親となるフレーム
	 * @param opt
	 *            オプション
	 */
	public OptionDialog(JFrame owner, Options opt) {
		super(owner);

		if (opt == null) {
			opt = new Options();
		}
		try {
			options = opt.clone(); // 渡されたオプションのクローンをいじる。
			options.addChangeListener();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			this.options = opt;
		}
		thisDialog = this;
		this.setModalityType(ModalityType.APPLICATION_MODAL);

		/*
		 * JGoodies Bindingを利用して，ダイアログの各部品とOptionオブジェクトの各プロパティの同期を取る
		 */
		BeanAdapter<Options> beanAdapter = new BeanAdapter<Options>(options, true);

		parent = owner;
		setName("optionDialog");
		Dimension dim = new Dimension(376, 580);
		setPreferredSize(dim);
		setMinimumSize(dim);
		setMaximumSize(dim);
		setTitle(Confirm.resources.getString("DIALOG_TITLE_OPTION")); // ダイアログのタイトル

		JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		getContentPane().add(tabbedPane, BorderLayout.CENTER);

		JPanel basicPanel = new JPanel();
		tabbedPane.addTab(Confirm.resources.getString("PANEL_TITLE_BASIC"), null, basicPanel, null);
		basicPanel.setLayout(null);

		JCheckBox chkbxEncryptPassword = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("memoryEncryptPassword"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_MEMORY_ENCRYPT_PASSWORD"));
		chkbxEncryptPassword.setBounds(6, 6, 201, 23);
		basicPanel.add(chkbxEncryptPassword);

		final JButton btnInputEncryptPassword = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnInputEncryptPassword.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Confirm confirm = new DialogConfirm(parent);
				String password = confirm.inputPassword(null, 2, false);
				if (password == null) {
					return;
				}
				options.setEncryptPassword(password);
			}
		});
		btnInputEncryptPassword.setBounds(0, 34, 53, 29);
		btnInputEncryptPassword.setEnabled(chkbxEncryptPassword.isSelected());
		basicPanel.add(btnInputEncryptPassword);

		encryptPasswordField = BasicComponentFactory.createPasswordField(beanAdapter.getValueModel("encryptPassword"));
		encryptPasswordField.setEditable(false);
		encryptPasswordField.setBounds(55, 33, 281, 28);
		basicPanel.add(encryptPasswordField);

		// チェックボックスの値の変化でボタンと入力欄の有効無効を切り替える
		chkbxEncryptPassword.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				encryptPasswordField.setEnabled(chkbox.isSelected());
				btnInputEncryptPassword.setEnabled(chkbox.isSelected());
			}
		});
		chkbxEncryptPassword.setSelected(options.isMemoryEncryptPassword());

		JCheckBox chkbxDecryptPassword = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("memoryDecryptPassword"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_MEMORY_DECRYPT_PASSWORD"));
		chkbxDecryptPassword.setBounds(6, 65, 201, 23);
		basicPanel.add(chkbxDecryptPassword);

		final JButton btnInputDecryptPassword = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnInputDecryptPassword.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Confirm confirm = new DialogConfirm(parent);
				String password = confirm.inputPassword(null, 2, false);
				if (password == null) {
					return;
				}
				options.setDecryptPassword(password);
			}
		});
		btnInputDecryptPassword.setBounds(0, 90, 53, 29);
		btnInputDecryptPassword.setEnabled(chkbxDecryptPassword.isSelected());
		basicPanel.add(btnInputDecryptPassword);

		decryptPasswordField = BasicComponentFactory.createPasswordField(beanAdapter.getValueModel("decryptPassword"));
		decryptPasswordField.setEditable(false);
		decryptPasswordField.setBounds(55, 89, 281, 28);
		basicPanel.add(decryptPasswordField);

		// チェックボックスの値の変化でボタンと入力欄の有効無効を切り替える
		chkbxDecryptPassword.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				decryptPasswordField.setEnabled(chkbox.isSelected());
				btnInputDecryptPassword.setEnabled(chkbox.isSelected());
			}
		});
		chkbxDecryptPassword.setSelected(options.isMemoryDecryptPassword());

		JCheckBox chkbxWithoutConfirmation = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("encryptWithoutConfirmation"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_WITHOUT_CONFIRMATION"));
		chkbxWithoutConfirmation.setEnabled(false);
		chkbxWithoutConfirmation.setBounds(6, 131, 330, 23);
		basicPanel.add(chkbxWithoutConfirmation);

		JCheckBox chkbxOpenDirectory = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("openDirectoryAfterDecrypt"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_OPEN_DIR"));
		chkbxOpenDirectory.setEnabled(false);
		chkbxOpenDirectory.setBounds(6, 166, 330, 23);
		basicPanel.add(chkbxOpenDirectory);

		JCheckBox chkbxOpenFile = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("openFile"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_OPEN_FILE"));
		chkbxOpenFile.setEnabled(false);
		chkbxOpenFile.setBounds(6, 201, 330, 23);
		basicPanel.add(chkbxOpenFile);

		JCheckBox chkbxQuit = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("quitAfterEncrypt"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_QUIT"));
		chkbxQuit.setBounds(6, 236, 330, 23);
		basicPanel.add(chkbxQuit);

		JCheckBox chkbxAlwaysOnTop = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("alwaysOnTop"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_ALWAYS_TOP"));
		chkbxAlwaysOnTop.setBounds(6, 271, 330, 23);
		basicPanel.add(chkbxAlwaysOnTop);

		JCheckBox chkbxShowPassword = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("showPassword"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_SHOW_PASSWORD"));
		chkbxShowPassword.setToolTipText("パスワードの表示");
		chkbxShowPassword.setBounds(6, 306, 330, 23);
		basicPanel.add(chkbxShowPassword);

		JCheckBox chkbxAskEncryptOrDecrypt = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("askEncryptOrDecrypt"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_ASK_ENCRYPT_DECRYPT"));
		chkbxAskEncryptOrDecrypt.setToolTipText("処理の確認の有無");
		chkbxAskEncryptOrDecrypt.setBounds(6, 341, 330, 23);
		basicPanel.add(chkbxAskEncryptOrDecrypt);

		JCheckBox chkbxNotMulipleStarts = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("notMulipleStarts"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_NOT_MULTIPLE_STARTS"));
		chkbxNotMulipleStarts.setToolTipText("複数起動の抑制");
		chkbxNotMulipleStarts.setBounds(6, 376, 330, 23);
		basicPanel.add(chkbxNotMulipleStarts);

		JCheckBox chkbxIncludeHiddenFile = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("includeHidden"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_INCLUDE_HIDDEN"));
		chkbxIncludeHiddenFile.setToolTipText("隠しファイルの扱い隠しファイルの扱い");
		chkbxIncludeHiddenFile.setBounds(6, 411, 295, 23);
		basicPanel.add(chkbxIncludeHiddenFile);

		JCheckBox chckbxUseVersion2 = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("useVersion2"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_ENCRYPT_VERSION"));
		chckbxUseVersion2.setToolTipText("暗号化の形式を形式を選択");
		chckbxUseVersion2.setBounds(6, 446, 281, 23);
		basicPanel.add(chckbxUseVersion2);

		JPanel savePanel = new JPanel();
		tabbedPane.addTab(Confirm.resources.getString("PANEL_TITLE_SAVE"), null, savePanel, null);
		savePanel.setLayout(null);

		JCheckBox chkbxEncodeToSameFolder = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("encodeToSameFolder"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_ENCODE_TO_SAME"));
		chkbxEncodeToSameFolder.setBounds(6, 6, 343, 23);
		savePanel.add(chkbxEncodeToSameFolder);

		final JButton btnInputEncodePath = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnInputEncodePath.setBounds(0, 34, 53, 29);
		btnInputEncodePath.setEnabled(chkbxEncodeToSameFolder.isSelected());
		savePanel.add(btnInputEncodePath);
		btnInputEncodePath.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = (encodePathField.getText().length() != 0)
						? new JFileChooser(new File(encodePathField.getText()).getParentFile()) : new JFileChooser();
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); // ディレクトリのみ
				fileChooser.setDialogTitle(Confirm.resources.getString("DIALOG_TITLE_DIR_FOR_ENCODE"));
				fileChooser.setMultiSelectionEnabled(false);
				int result = fileChooser.showDialog(thisDialog, Confirm.resources.getString("SELECT_OPTION_SELECT"));
				if (result == JFileChooser.APPROVE_OPTION) {
					try {
						options.setEncodeFolderPath(fileChooser.getSelectedFile().getCanonicalPath());
					} catch (IOException ex) {
						ex.printStackTrace();
					}
				}
			}
		});

		encodePathField = BasicComponentFactory.createTextField(beanAdapter.getValueModel("encodeFolderPath"));
		encodePathField.setBounds(55, 33, 281, 28);
		encodePathField.setEditable(false);
		savePanel.add(encodePathField);
		encodePathField.setColumns(10);

		// チェックボックスの値の変化でボタンと入力欄の有効無効を切り替える
		chkbxEncodeToSameFolder.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				encodePathField.setEnabled(chkbox.isSelected());
				btnInputEncodePath.setEnabled(chkbox.isSelected());
			}
		});
		chkbxEncodeToSameFolder.setSelected(options.isEncodeToSameFolder());

		JCheckBox chkbxDecodeToSameFolder = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("decodeToSameFolder"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_DECODE_TO_SAME"));
		chkbxDecodeToSameFolder.setBounds(6, 75, 343, 23);
		savePanel.add(chkbxDecodeToSameFolder);

		final JButton btnInputDecodePath = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnInputDecodePath.setBounds(0, 103, 53, 29);
		btnInputDecodePath.setEnabled(chkbxDecodeToSameFolder.isSelected());
		savePanel.add(btnInputDecodePath);
		btnInputDecodePath.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = (decodePathField.getText().length() != 0)
						? new JFileChooser(new File(decodePathField.getText()).getParentFile()) : new JFileChooser();
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); // ディレクトリのみ
				fileChooser.setDialogTitle(Confirm.resources.getString("DIALOG_TITLE_DIR_FOR_DECODE"));
				fileChooser.setMultiSelectionEnabled(false);
				int result = fileChooser.showDialog(thisDialog, Confirm.resources.getString("SELECT_OPTION_SELECT"));
				if (result == JFileChooser.APPROVE_OPTION) {
					try {
						options.setDecodeFolderPath(fileChooser.getSelectedFile().getCanonicalPath());
					} catch (IOException ex) {
						ex.printStackTrace();
					}
				}
			}
		});

		decodePathField = BasicComponentFactory.createTextField(beanAdapter.getValueModel("decodeFolderPath"));
		decodePathField.setBounds(55, 102, 281, 28);
		decodePathField.setEditable(false);
		savePanel.add(decodePathField);
		decodePathField.setColumns(10);

		// チェックボックスの値の変化でボタンと入力欄の有効無効を切り替える
		chkbxDecodeToSameFolder.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				decodePathField.setEnabled(chkbox.isSelected());
				btnInputDecodePath.setEnabled(chkbox.isSelected());
			}
		});
		chkbxDecodeToSameFolder.setSelected(options.isDecodeToSameFolder());

		JCheckBox chkbxConfirmOverWrite = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("confirmOverwirte"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_CONFIRM_OVER_WRITE"));
		chkbxConfirmOverWrite.setBounds(6, 142, 343, 23);
		savePanel.add(chkbxConfirmOverWrite);

		JPanel panel = new JPanel();
		panel.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null),
				Confirm.resources.getString("BORDER_TITLE_FILES_OPERATION"), TitledBorder.LEADING, TitledBorder.TOP,
				null, new Color(0, 0, 0)));
		panel.setMaximumSize(new Dimension(350, 60));
		panel.setMinimumSize(new Dimension(350, 60));
		panel.setSize(new Dimension(344, 60));
		panel.setPreferredSize(new Dimension(344, 80));
		panel.setBounds(6, 177, 343, 80);
		savePanel.add(panel);
		panel.setLayout(null);

		// こちらがチェックされた場合はfalseを設定
		JRadioButton rdBtnNotpack = BasicComponentFactory.createRadioButton(beanAdapter.getValueModel("allFilePacked"),
				false, Confirm.resources.getString("OPTION_DIALOG_RDBTN_NOT_PACKED"));
		rdBtnNotpack.setBounds(6, 19, 316, 23);
		panel.add(rdBtnNotpack);

		// こちらがチェックされた場合はtrueを設定
		JRadioButton rdBtnAllPack = BasicComponentFactory.createRadioButton(beanAdapter.getValueModel("allFilePacked"),
				true, Confirm.resources.getString("OPTION_DIALOG_RDBTN_PACKED"));
		rdBtnAllPack.setBounds(6, 51, 316, 23);
		panel.add(rdBtnAllPack);

		JCheckBox chkbxNotMakeParentFolder = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("notMakeParentFolder"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_NOT_MAKE_PARENT"));
		chkbxNotMakeParentFolder.setEnabled(false); // まだ未実装
		chkbxNotMakeParentFolder.setBounds(6, 269, 343, 23);
		savePanel.add(chkbxNotMakeParentFolder);

		JCheckBox chkbxKeepTimestamp = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("keepTimestamp"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_KEEP_TIMESTAMP"));
		chkbxKeepTimestamp.setBounds(6, 304, 343, 23);
		savePanel.add(chkbxKeepTimestamp);

		JCheckBox chkbxTimestampNow = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("timestampNow"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_TIMESTAMP_NOW"));
		chkbxTimestampNow.setBounds(6, 339, 343, 23);
		savePanel.add(chkbxTimestampNow);

		JCheckBox chkbxIncludeExtension = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("includeExtension"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_INCLUDE_EXTENSION"));
		chkbxIncludeExtension.setBounds(6, 374, 343, 23);
		savePanel.add(chkbxIncludeExtension);

		JCheckBox chkbxAddName = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("addFileName"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_ADD_NAME"));
		chkbxAddName.setEnabled(false);
		chkbxAddName.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent arg0) {
				JCheckBox chkbox = (JCheckBox) arg0.getSource();
				if (extentionField != null) {
					extentionField.setEnabled(chkbox.isSelected());
				}
			}
		});
		chkbxAddName.setBounds(6, 407, 343, 23);
		savePanel.add(chkbxAddName);

		extentionField = BasicComponentFactory.createTextField(beanAdapter.getValueModel("autoNameFormatText"));
		extentionField.setEnabled(false);
		extentionField.setBounds(6, 428, 330, 28);
		savePanel.add(extentionField);
		extentionField.setColumns(10);

		JPanel deletePanel = new JPanel();
		tabbedPane.addTab(Confirm.resources.getString("PANEL_TITLE_DELETE"), null, deletePanel, null);
		deletePanel.setLayout(null);

		chkbxDeleteOriginal = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("deleteOriginalFile"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_DELETE_ORIGINAL"));
		chkbxDeleteOriginal.setBounds(6, 6, 343, 23);
		deletePanel.add(chkbxDeleteOriginal);

		chkbxDeleteEncrypt = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("deleteEncryptoFile"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_DELETE_ENCRYPT"));
		chkbxDeleteEncrypt.setBounds(6, 41, 343, 23);
		boolean delete = options.isDeleteEncryptoFile();
		chkbxDeleteEncrypt.setSelected(true);
		deletePanel.add(chkbxDeleteEncrypt);
		JPanel panel_1 = new JPanel();
		panel_1.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null),
				Confirm.resources.getString("BORDER_TITLE_DELETE_SETTING"), TitledBorder.LEADING, TitledBorder.TOP,
				null, new Color(0, 0, 0)));
		panel_1.setBounds(6, 76, 343, 192);
		deletePanel.add(panel_1);
		panel_1.setLayout(null);

		final JRadioButton rdbtnNormalDelete = BasicComponentFactory.createRadioButton(
				beanAdapter.getValueModel("deleteMode"), FileDeletion.NORMA_DELETE,
				Confirm.resources.getString("OPTION_DIALOG_RDBTN_DELETE_NORMAL"));
		rdbtnNormalDelete.setBounds(6, 26, 141, 23);
		panel_1.add(rdbtnNormalDelete);

		final JRadioButton rdbtnTrushDelete = BasicComponentFactory.createRadioButton(
				beanAdapter.getValueModel("deleteMode"), FileDeletion.TRASH_DELETE,
				Confirm.resources.getString("OPTION_DIALOG_RDBTN_DELETE_TO_TRUSH"));
		rdbtnTrushDelete.setBounds(6, 61, 220, 23);
		panel_1.add(rdbtnTrushDelete);

		final JRadioButton rdbtnThoroughDelete = BasicComponentFactory.createRadioButton(
				beanAdapter.getValueModel("deleteMode"), FileDeletion.THOROUGH_DELETE,
				Confirm.resources.getString("OPTION_DIALOG_RDBTN_DELETE_THOROUGH"));
		rdbtnThoroughDelete.setBounds(6, 96, 220, 23);
		panel_1.add(rdbtnThoroughDelete);

		final JLabel lblRandomCount = new JLabel(Confirm.resources.getString("OPTION_DIALOG_LABEL_RANDOM"));
		lblRandomCount.setHorizontalAlignment(SwingConstants.LEFT);
		lblRandomCount.setBounds(35, 131, 127, 16);
		panel_1.add(lblRandomCount);

		final JSpinner spinnerRandomCount = new JSpinner(
				new SpinnerNumberModel(options.getRandomWriteCount(), 0, null, 1));
		spinnerRandomCount.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSpinner spinner = (JSpinner) e.getSource();
				int value = (Integer) spinner.getValue();
				options.setRandomWriteCount(value);
			}
		});
		spinnerRandomCount.setBounds(35, 148, 120, 28);
		panel_1.add(spinnerRandomCount);

		final JLabel lblZeroCount = new JLabel(Confirm.resources.getString("OPTION_DIALOG_LABEL_ZERO"));
		lblZeroCount.setHorizontalAlignment(SwingConstants.LEFT);
		lblZeroCount.setBounds(201, 131, 127, 16);
		panel_1.add(lblZeroCount);

		final JSpinner spinnerZeroCount = new JSpinner(new SpinnerNumberModel(options.getZeroWriteCount(), 0, null, 1));
		spinnerZeroCount.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSpinner spinner = (JSpinner) e.getSource();
				int value = (Integer) spinner.getValue();
				options.setZeroWriteCount(value);
			}
		});
		spinnerZeroCount.setBounds(201, 148, 120, 28);
		panel_1.add(spinnerZeroCount);

		chkbxDeleteOriginal.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				boolean selected = isEnableDeleteSetting();
				rdbtnNormalDelete.setEnabled(selected);
				rdbtnTrushDelete.setEnabled(selected);
				rdbtnThoroughDelete.setEnabled(selected);
				lblRandomCount.setEnabled(selected);
				spinnerRandomCount.setEnabled(selected);
				lblZeroCount.setEnabled(selected);
				spinnerZeroCount.setEnabled(selected);
			}
		});
		chkbxDeleteOriginal.setSelected(options.isDeleteOriginalFile());

		chkbxDeleteEncrypt.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				boolean selected = isEnableDeleteSetting();
				rdbtnNormalDelete.setEnabled(selected);
				rdbtnTrushDelete.setEnabled(selected);
				rdbtnThoroughDelete.setEnabled(selected);
				lblRandomCount.setEnabled(selected);
				spinnerRandomCount.setEnabled(selected);
				lblZeroCount.setEnabled(selected);
				spinnerZeroCount.setEnabled(selected);
			}
		});

		chkbxDeleteEncrypt.setSelected(delete);

		JPanel advancedPanel = new JPanel();
		tabbedPane.addTab(Confirm.resources.getString("PANEL_TITLE_ADVANCED"), null, advancedPanel, null);
		advancedPanel.setLayout(null);

		JPanel panel_2 = new JPanel();
		panel_2.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null),
				Confirm.resources.getString("BORDER_TITLE_PASSWORD_FILE"), TitledBorder.LEADING, TitledBorder.TOP, null,
				new Color(0, 0, 0)));
		panel_2.setBounds(6, 6, 343, 158);
		advancedPanel.add(panel_2);
		panel_2.setLayout(null);

		JCheckBox chkbxPasswordFile = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("usePasswordFile"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_PASSWORD_FILE"));
		chkbxPasswordFile.setBounds(6, 16, 331, 23);
		panel_2.add(chkbxPasswordFile);
		final boolean usePasswordFile = options.isUsePasswordFile();
		chkbxPasswordFile.setSelected(true);

		final JButton btnEncryptPasswordFile = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnEncryptPasswordFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setMultiSelectionEnabled(false);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fileChooser.setDialogTitle(Confirm.resources.getString("DIALOG_TITLE_PASSWORD_FOR_ENCRYPT"));
				int result = fileChooser.showOpenDialog(parent);
				if (result == JFileChooser.APPROVE_OPTION) {
					options.setEncryptPasswordFileName(fileChooser.getSelectedFile().getAbsolutePath());
				}
			}
		});
		btnEncryptPasswordFile.setBounds(6, 60, 53, 29);
		panel_2.add(btnEncryptPasswordFile);

		final JButton btnDecryptPasswordFile = new JButton(Confirm.resources.getString("OPTION_DIALOG_BTN_SETTING"));
		btnDecryptPasswordFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setMultiSelectionEnabled(false);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				fileChooser.setDialogTitle(Confirm.resources.getString("DIALOG_TITLE_PASSWORD_FOR_DECRYPT"));
				int result = fileChooser.showOpenDialog(parent);
				if (result == JFileChooser.APPROVE_OPTION) {
					options.setDecryptPasswordFileName(fileChooser.getSelectedFile().getAbsolutePath());
				}
			}
		});
		btnDecryptPasswordFile.setBounds(6, 116, 53, 29);
		panel_2.add(btnDecryptPasswordFile);

		encryptPasswordFileField = BasicComponentFactory
				.createTextField(beanAdapter.getValueModel("encryptPasswordFileName"));
		encryptPasswordFileField.setColumns(10);
		encryptPasswordFileField.setEditable(false);
		encryptPasswordFileField.setBounds(59, 59, 278, 28);
		panel_2.add(encryptPasswordFileField);

		decryptPasswordFileField = BasicComponentFactory
				.createTextField(beanAdapter.getValueModel("decryptPasswordFileName"));
		decryptPasswordFileField.setColumns(10);
		decryptPasswordFileField.setEditable(false);
		decryptPasswordFileField.setBounds(59, 116, 278, 28);
		panel_2.add(decryptPasswordFileField);

		final JLabel lblEncryptPasswordFile = new JLabel(
				Confirm.resources.getString("OPTION_DIALOG_LABEL_PASSWORD_FILE_FOR_ENCRYPT"));
		lblEncryptPasswordFile.setBounds(16, 45, 184, 16);
		panel_2.add(lblEncryptPasswordFile);

		final JLabel lblDecryptPasswordFile = new JLabel(
				Confirm.resources.getString("OPTION_DIALOG_LABEL_PASSWORD_FILE_FOR_DECRYPT"));
		lblDecryptPasswordFile.setBounds(16, 101, 184, 16);
		panel_2.add(lblDecryptPasswordFile);

		chkbxPasswordFile.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				boolean selected = chkbox.isSelected();
				btnDecryptPasswordFile.setEnabled(selected);
				btnEncryptPasswordFile.setEnabled(selected);
				decryptPasswordFileField.setEnabled(selected);
				encryptPasswordFileField.setEnabled(selected);
				lblEncryptPasswordFile.setEnabled(selected);
				lblDecryptPasswordFile.setEnabled(selected);
			}
		});
		chkbxPasswordFile.setSelected(usePasswordFile);

		JPanel panel_3 = new JPanel();
		panel_3.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null),
				Confirm.resources.getString("BORDER_TITLE_EXT_CAMOUFLAGE"), TitledBorder.LEADING, TitledBorder.TOP,
				null, new Color(0, 0, 0)));
		panel_3.setBounds(6, 176, 343, 75);
		advancedPanel.add(panel_3);
		panel_3.setLayout(null);

		JCheckBox chkbxCamouflageExtention = BasicComponentFactory.createCheckBox(
				beanAdapter.getValueModel("camouflageExtention"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_EXT_CAMOUFLAGE"));
		chkbxCamouflageExtention.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JCheckBox chkbox = (JCheckBox) e.getSource();
				boolean selected = chkbox.isSelected();
				camouflageExtentionField.setEnabled(selected);
			}
		});
		chkbxCamouflageExtention.setBounds(6, 16, 331, 23);
		panel_3.add(chkbxCamouflageExtention);

		camouflageExtentionField = BasicComponentFactory.createTextField(beanAdapter.getValueModel("extensionString"));
		camouflageExtentionField.setBounds(16, 39, 223, 28);
		panel_3.add(camouflageExtentionField);
		camouflageExtentionField.setColumns(10);
		camouflageExtentionField.setEnabled(chkbxCamouflageExtention.isSelected());

		JPanel panel_4 = new JPanel();
		panel_4.setBorder(new TitledBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null),
				Confirm.resources.getString("BORDER_TITLE_LIMIT_PASSWORD_INPUT"), TitledBorder.LEADING,
				TitledBorder.TOP, null, new Color(0, 0, 0)));
		panel_4.setBounds(6, 263, 343, 110);
		advancedPanel.add(panel_4);
		panel_4.setLayout(null);

		JLabel lblNewLabel = new JLabel(Confirm.resources.getString("OPTION_DIALOG_LABEL_CAUTION"));
		lblNewLabel.setFont(new Font("Lucida Grande", Font.BOLD, 13));
		lblNewLabel.setForeground(Color.RED);
		lblNewLabel.setBounds(6, 20, 331, 21);
		panel_4.add(lblNewLabel);

		JLabel lblNewLabel_1 = new JLabel(Confirm.resources.getString("OPTION_DIALOG_LABEL_LIMIT_PASSWORD_INPUT"));
		lblNewLabel_1.setBounds(16, 53, 200, 16);
		panel_4.add(lblNewLabel_1);

		JSpinner spinnerMaxMisstake = new JSpinner(new SpinnerNumberModel(options.getMaxMistake(), 0, null, 1));
		spinnerMaxMisstake.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSpinner spinner = (JSpinner) e.getSource();
				options.setMaxMistake((Integer) spinner.getValue());
			}
		});
		spinnerMaxMisstake.setBounds(224, 40, 69, 28);
		panel_4.add(spinnerMaxMisstake);

		JCheckBox chkbxFileBreak = BasicComponentFactory.createCheckBox(beanAdapter.getValueModel("fileBrakeOption"),
				Confirm.resources.getString("OPTION_DIALOG_CHK_FILE_BREAK"));
		chkbxFileBreak.setBounds(6, 74, 320, 23);
		panel_4.add(chkbxFileBreak);

		JPanel panel_5 = new JPanel();
		panel_5.setBorder(new TitledBorder(null, Confirm.resources.getString("BORDER_TITLE_COMPRESS_RATE"),
				TitledBorder.LEADING, TitledBorder.TOP, null, null));
		panel_5.setBounds(6, 385, 343, 71);
		advancedPanel.add(panel_5);
		panel_5.setLayout(null);

		JSlider compressRateSlider = new JSlider();
		compressRateSlider.setMaximum(9);
		compressRateSlider.setPaintTicks(true);
		compressRateSlider.setSnapToTicks(true);
		compressRateSlider.setMajorTickSpacing(1);
		compressRateSlider.setLabelTable(compressRateSlider.createStandardLabels(1));
		compressRateSlider.setPaintLabels(true);
		compressRateSlider.setValue(options.getCompressRate());
		compressRateSlider.setBounds(6, 16, 331, 38);
		compressRateSlider.setPaintTicks(true);
		compressRateSlider.setToolTipText(Confirm.resources.getString("BORDER_TITLE_COMPRESS_RATE"));
		compressRateSlider.setPaintLabels(true);
		panel_5.add(compressRateSlider);
		compressRateSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider slider = (JSlider) e.getSource();
				options.setCompressRate(slider.getValue());
			}
		});

		JPanel buttonPanel = new JPanel();
		okButton = new JButton(Confirm.resources.getString("LABEL_OK"));
		okButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				canceled = false;
				close();
			}
		});
		JButton cancelButton = new JButton(Confirm.resources.getString("LABEL_CANCEL"));
		cancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int result = 0;
				if (options.isChanged()) {
					result = JOptionPane.showConfirmDialog(parent,
							Confirm.resources.getString("MSG_CONFIRM_SETTING_CANCEL"),
							Confirm.resources.getString("DIALOG_TITLE_SETTING_CANCEL"), JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE, new ImageIcon(getClass().getResource("/icon_help.png")));
				}
				if (result == JOptionPane.YES_OPTION) {
					canceled = true;
					close();
				}
			}
		});
		// Macとその他の場合で，ボタンの順番を入れ替える
		if (Utility.isMac()) {
			buttonPanel.add(cancelButton);
			buttonPanel.add(okButton);
		} else {
			buttonPanel.add(okButton);
			buttonPanel.add(cancelButton);
		}
		buttonPanel.setPreferredSize(new Dimension(376, 30));
		buttonPanel.setInheritsPopupMenu(true);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 2));

		this.setLocationRelativeTo(parent);
		this.addWindowListener(new WindowAdapter() {

			@Override
			public void windowActivated(WindowEvent e) {
				super.windowActivated(e);
				okButton.requestFocusInWindow();
			}
		});
		this.setResizable(false); // 大きさが変更できないように。
		options.setChanged(false);
	}

	/**
	 * オプションを取得します。
	 * 
	 * @return オプション
	 */
	public Options getOptions() {
		return options;
	}

	/**
	 * オプションを設定します。
	 * 
	 * @param options
	 *            オプション
	 */
	public void setOptions(Options options) {
		this.options = options;
	}

	/**
	 * ダイアログを閉じます。
	 */
	private void close() {
		this.setVisible(false);
	}

	/**
	 * キャンセルされたかどうかを取得します。
	 * 
	 * @return キャンセルされた場合はtrue
	 */
	public boolean isCanceled() {
		return canceled;
	}

	/**
	 * canceledを設定します。
	 * 
	 * @param canceled
	 *            設定する値
	 */
	public void setCanceled(boolean canceled) {
		this.canceled = canceled;
	}

	/**
	 * 「元ファイルを削除する」または「暗号化ファイルを削除する」のいずれかがチェックされているかどうかを取得します。
	 * 
	 * @return どちらかがチェックされている場合はtrue
	 */
	private boolean isEnableDeleteSetting() {
		return chkbxDeleteEncrypt.isSelected() || chkbxDeleteOriginal.isSelected();
	}
}
