/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.io;

import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.Confirm;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.FileAttribute;
import org.hibara.attachecase.utility.AbstractProgressListener;

/**
 * ファイルまたはフォルダの削除を行います。
 * 
 * <pre>
 * 削除の方法は次の三つから選択することができます。
 * (1) 通常削除
 *     ファイルまたはフォルダを削除します。
 * (2) 完全削除
 *     乱数と0を書き込んでからファイルまたはフォルダを削除します。
 *     フォルダが指定された場合は，フォルダ内のすべてのファイルに乱数と0を書き込みます。
 * (3) ゴミ箱へ移動
 *     削除せずにゴミ箱へ移動します。Windows上ではこの機能は利用できません。
 * </pre>
 * 
 * @author Masanori Uozumi
 * @version 2012.08.10<br>
 *          2012.08.02 ゴミ箱に同名ファイルが存在する場合の処理を追加。デフォルト動作は削除無しに変更。<br>
 *          2012.08.10 進捗状況の情報追加。引数有りコンストラクタを追加。<br>
 * 
 */
public class FileDeletion implements AttacheCaseConstant {

	/**
	 * 削除しない
	 */
	public static final int NO_DELETE = -1; // 削除しない

	/**
	 * 通常削除
	 */
	public static final int NORMA_DELETE = 0; // 通常削除

	/**
	 * 完全削除
	 */
	public static final int THOROUGH_DELETE = 1; // 完全削除

	/**
	 * ゴミ箱へ移動
	 */
	public static final int TRASH_DELETE = 2; // ゴミ箱へ移動。Windowsでは無理かも…

	/*
	 * ゴミ箱のフォルダ名。 Windowsには未対応。
	 */
	private static final String TRASH_PATH = ".Trash/"; // Unix系OSの場合

	private int randomCount; // 乱数の書き込み回数
	private int zeroCount; // 0の書き込み回数
	private int deleteMode = NO_DELETE; // 消去モード(デフォルトは削除無し)
	private int progress; // 進捗状況%
	private String currentFileName; // 処理中のファイル名
	private long allSize = 0;

	private List<String> fileList; // 削除対象のファイル

	private AbstractProgressListener listener = null;

	/**
	 * FileDeletionを構築します。
	 */
	public FileDeletion() {
		zeroCount = 1;
		randomCount = 1;
		deleteMode = NORMA_DELETE;
	}

	/**
	 * FileDeletionを構築します。
	 * 
	 * @param options
	 *            オプション
	 */
	public FileDeletion(Options options) {
		randomCount = options.getRandomWriteCount();
		zeroCount = options.getZeroWriteCount();
		deleteMode = options.getDeleteMode();
	}

	/**
	 * 削除ファイルのリストを取得します。
	 * 
	 * @return 削除ファイルリスト
	 */
	public List<String> getFileList() {
		return fileList;
	}

	/**
	 * 削除ファイルを設定します。
	 * 
	 * @param fileList
	 *            ファイル属性の文字列表現のリスト
	 */
	public void setFileList(List<String> fileList) {
		this.fileList = fileList;
	}

	/**
	 * 削除ファイルを設定します。
	 * 
	 * @param list
	 *            ファイル属性オブジェクトのリスト
	 */
	public void setFileListByAttribute(List<FileAttribute> list) {
		fileList = new ArrayList<String>();
		for (FileAttribute attr : list) {
			fileList.add(attr.getFileNameOnly());
		}
	}

	/**
	 * <pre>
	 * 削除ファイルが一つだけの場合に利用します。
	 * fileListは新たに作り直されます。要素は指定したファイルだけになります。
	 * </pre>
	 * 
	 * @param fileName
	 *            削除するファイル。
	 */
	public void setDeleteFile(String fileName) {
		fileList = new ArrayList<String>();
		fileList.add(fileName);
	}

	public int getRandomCount() {
		return randomCount;
	}

	public void setRandomCount(int randomCount) {
		if (randomCount < 0) {
			randomCount = 1;
		}
		this.randomCount = randomCount;
	}

	public int getZeroCount() {
		return zeroCount;
	}

	public void setZeroCount(int zeroCount) {
		if (zeroCount < 0) {
			zeroCount = 1;
		}
		this.zeroCount = zeroCount;
	}

	public int getDeleteMode() {
		return deleteMode;
	}

	public int getProgress() {
		return progress;
	}

	public String getCurrentFileName() {
		return currentFileName;
	}

	/**
	 * 削除モードを設定します。
	 * 
	 * <pre>
	 * 乱数による削除回数，また0による削除回数が0以下の場合，それぞれを1に設定します。
	 * </pre>
	 * 
	 * @param deleteMode
	 *            削除モード
	 */
	public void setDeleteMode(int deleteMode) {
		this.deleteMode = deleteMode;
		if (this.deleteMode == THOROUGH_DELETE) {
			randomCount = (randomCount <= 0) ? 1 : randomCount;
			zeroCount = (zeroCount <= 0) ? 1 : zeroCount;
		}
	}

	/**
	 * ゴミ箱用ディレクトリのパスを取得します。
	 * 
	 * @return ゴミ箱用ディレクトリのパス
	 */
	private String getTrashPath() {
		File f = new File(System.getProperty("user.home"), TRASH_PATH);
		return (f.exists()) ? f.getAbsolutePath() : null;
	}

	/**
	 * 完全削除を行います。
	 * 
	 * @param path
	 *            削除するフォルダまたはディレクトリ
	 * @return 削除結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteThoroughMode(String path) {
		return deleteThoroughMode(new File(path));
	}

	/**
	 * 完全削除を行います。
	 * 
	 * @param f
	 *            削除するフォルダまたはディレクトリ
	 * @return 削除結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteThoroughMode(File f) {
		if (!f.exists()) {
			return false;
		}

		boolean result = true;
		if (f.isDirectory()) {
			File[] files = f.listFiles();
			for (File file : files) {
				result = deleteThoroughMode(file);
				if (!result) {
					return result;
				}
			}
		} else {
			if (listener != null) {
				listener.publishFileName(Confirm.resources.getString("MSG_DELETE_CONTINUE") + f.getName());
			}
			RandomAccessFile fs = null;
			try {
				fs = new RandomAccessFile(f, "rw");
				deleteByRandomData(fs);
				deleteByZeroData(fs);
				result = true;
			} catch (IOException e) {
				result = false;
				e.printStackTrace();
			} finally {
				if (fs != null) {
					try {
						fs.close();
					} catch (IOException e) {
						// 何もしない
					}
				}
			}
		}
		return result;
	}

	/**
	 * 通常削除（ファイル，またはディレクトリをそのまま削除）を行います。
	 * 
	 * @param path
	 *            削除するフォルダまたはディレクトリ
	 * @return 削除結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteNormalMode(String path) {
		return deleteNormalMode(new File(path));
	}

	/**
	 * 通常削除（ファイル，またはディレクトリをそのまま削除）を行います。
	 * 
	 * @param f
	 *            削除するフォルダまたはディレクトリ
	 * @return 削除結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteNormalMode(File f) {
		if (!f.exists()) {
			return false;
		}
		if (listener != null) {
			listener.publishFileName(Confirm.resources.getString("MSG_DELETE_CONTINUE") + f.getName());
		}
		f.setWritable(true);
		return FileUtils.deleteQuietly(f);
	}

	/**
	 * 指定したファイルまたはディレクトリをゴミ箱へ移動します。
	 * 
	 * <pre>
	 * 今のところ，この機能が利用できるのは，OSがunix系の場合のみとなります。
	 * </pre>
	 * 
	 * @param path
	 *            移動するフォルダまたはディレクトリ
	 * @return 移動結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteTrashMode(String path) {
		return deleteTrashMode(new File(path));
	}

	/**
	 * 指定したファイルまたはディレクトリをゴミ箱へ移動します。
	 * 
	 * <pre>
	 * 今のところ，この機能が利用できるのは，OSがunix系の場合のみとなります。
	 * </pre>
	 * 
	 * @param f
	 *            移動するフォルダまたはディレクトリ
	 * @return 移動結果。成功した場合は true，失敗した場合は false。
	 */
	private boolean deleteTrashMode(File f) {
		if (!f.exists()) {
			// System.out.println("ファイルが存在しない");
			return false;
		}

		String trash = getTrashPath();
		if (trash == null) {
			// System.out.println("ゴミ箱が存在しない");
			return false;
		}

		boolean result = false;
		try {
			if (listener != null) {
				listener.publishFileName(Confirm.resources.getString("MSG_DELETE_CONTINUE") + f.getName());
			}
			// ゴミ箱に同じ名前のファイルが存在していないかどうかをチェック
			String fileName = getTrashFileName(f, new File(trash));
			// System.out.println(fileName);
			if (!fileName.equals(f.getName())) {
				File newFile = new File(f.getParent(), fileName);
				result = f.renameTo(newFile);
				if (!result) {
					return result;
				}
				f = newFile;
			}
			FileUtils.moveToDirectory(f, new File(trash), false);
			FileUtils.deleteQuietly(f);
			result = true;
		} catch (IOException e) {
			result = false;
			// e.printStackTrace();
		}
		return result;
	}

	/**
	 * 
	 * @param listener
	 *            GUI
	 */
	public void run(AbstractProgressListener listener) {
		this.listener = listener;
		run();
	}

	/**
	 * ファイルまたはディレクトリの削除を実行します。
	 * 
	 * <pre>
	 * このメソッドが呼び出される前に，以下のデータが設定されている必要があります。
	 * ● 削除するファイル名またはディレクトリ名（複数化）
	 * ● 削除モード
	 * </pre>
	 */
	public void run() {
		allSize = getAllSize();
		long totalSize = 0;
		if (listener != null) {
			listener.publishProgress(0);
		}
		// 削除するファイルが設定されていない場合はそのまま終了
		if (deleteMode == NO_DELETE || fileList == null || fileList.size() == 0) {
			return;
		}
		boolean result = false;
		for (String fName : fileList) {
			// 現在処理中のファイル
			currentFileName = fName;
			File f = new File(fName);
			if (f.exists() && f.isFile()) {
				totalSize += f.length();
				progress = (int) (totalSize * 100 / allSize);
			}
			if (listener != null) {
				listener.publishFileName(
						ResourceBundle.getBundle(RESOURCE).getString("MSG_DELETE_CONTINUE") + f.getName());
			}
			switch (deleteMode) {
			case TRASH_DELETE:
				result = deleteTrashMode(fName);
				break;
			case NORMA_DELETE:
				result = deleteNormalMode(fName);
				break;
			default:
				result = deleteThoroughMode(fName);
				result = deleteNormalMode(fName);
				break;
			}
			if (!result) {
				break;
			}
			// キャンセルされていれば中止
			if (listener != null && listener.isCancel()) {
				return;
			}
		}
		// System.out.println(result);
	}

	/**
	 * 指定したファイルにランダムなデータを書き込みます。
	 * 
	 * @param f
	 *            データを書き込むファイル
	 * @throws IOException
	 *             書き込みに失敗した場合
	 */
	private void deleteByRandomData(RandomAccessFile f) throws IOException {
		long size = f.length();
		for (int i = 0; i < randomCount; i++) {
			f.seek(0L); // ポインタを先頭に戻す
			long count = size; // 書き込むバイト数
			while (count > 0) {
				byte[] buffer = getRandomByteArray((int) ((count >= FILE_BUFFER_SIZE) ? FILE_BUFFER_SIZE : count));
				f.write(buffer);
				count -= FILE_BUFFER_SIZE;
			}
		}
	}

	/**
	 * 乱数が格納された指定した要素数のバイト配列を取得します。
	 * 
	 * @param length
	 *            取得するバイト配列の要素数
	 * @return 乱数が格納されたバイト配列
	 */
	private byte[] getRandomByteArray(int length) {
		byte[] buffer = new byte[length];
		for (int i = 0; i < length; i++) {
			buffer[i] = (byte) (Math.random() * 256);
		}
		return buffer;
	}

	/**
	 * 指定したファイルに0を書き込みます。
	 * 
	 * @param f
	 *            データを書き込むファイル
	 * @throws IOException
	 *             書き込みに失敗した場合
	 */
	private void deleteByZeroData(RandomAccessFile f) throws IOException {
		long size = f.length();
		byte[] buffer = new byte[FILE_BUFFER_SIZE];
		for (int i = 0; i < zeroCount; i++) {
			f.seek(0L); // ポインタを先頭に戻す
			long count = size; // 書き込むバイト数
			while (count > 0) {
				f.write(buffer, 0, (int) ((count >= FILE_BUFFER_SIZE) ? FILE_BUFFER_SIZE : count));
				count -= FILE_BUFFER_SIZE;
			}
		}
	}

	/**
	 * ゴミ箱に移動する際のファイル名（フォルダ名）を取得します。
	 * 
	 * <pre>
	 * ゴミ箱にすでに同じ名前のファイル（フォルダ）が存在する場合，異なる名前を返します。
	 * 異なる名前は，元の名前の後に5桁の数字を追加したものとなります。
	 * </pre>
	 * 
	 * @param file
	 *            ファイル名
	 * @param trash
	 *            ゴミ箱
	 * @return ゴミ箱に移動後のファイル名
	 */
	private String getTrashFileName(File file, File trash) {
		File dir = file.getParentFile();
		File f1 = new File(trash, file.getName());
		// ゴミ箱に同じ名前のファイル（フォルダ）が存在しない場合はそのまま名前を返す。
		if (!f1.exists()) {
			return file.getName();
		}
		// ゴミ箱に同じ名前のファイル（フォルダ）が存在する場合
		File f2 = new File(dir, file.getName());
		int count = 0;
		NumberFormat nf = NumberFormat.getInstance();
		nf.setParseIntegerOnly(true);
		nf.setGroupingUsed(false);
		nf.setMinimumIntegerDigits(5);
		while (f1.exists() || f2.exists()) {
			count++;
			String fileName = FilenameUtils.getBaseName(file.getName()) + "_" + nf.format(count) + "."
					+ FilenameUtils.getExtension(file.getName());
			f1 = new File(trash, fileName);
			f2 = new File(dir, fileName);
		}
		return f2.getName();
	}

	/**
	 * 削除対象になっているファイルのサイズの総計を求めます。
	 * 
	 * @return 削除対象ファイルのサイズ総計
	 */
	private long getAllSize() {
		long size = 0;
		for (String s : fileList) {
			File f = new File(s);
			if (f.exists() && f.isFile()) {
				size += f.length();
			}
		}
		return size;
	}
}
