/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */
package org.hibara.attachecase.io;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * java.io.RandomAccessFileにいくつかのメソッドを追加しています。
 * 
 * <pre>
 * 追加したメソッドは以下のとおりです。
 * ●シークに関するもの
 * ●基本データ型を読み込むもの（従来のものはビックエンディアンなので，リトルエンディアンのものを追加）
 * </pre>
 * 
 * @author Masanori Uozumi
 * @version 2012.07.30
 */
public class RandomAccessFile extends java.io.RandomAccessFile {

	public RandomAccessFile(String name, String mode) throws FileNotFoundException {
		super(name, mode);
	}

	public RandomAccessFile(File file, String mode) throws FileNotFoundException {
		super(file, mode);
	}

	/**
	 * リソースの末尾からシークします。
	 * 
	 * @param pos
	 *            末尾からの位置。マイナスで指定。
	 * @throws IOException
	 *             I/Oエラー発生時
	 */
	public void seekFromEnd(long pos) throws IOException {
		if (pos > 0 || Math.abs(pos) > length()) {
			return;
		}
		seek(length() + pos);
	}

	/**
	 * 現在のポインターから指定したバイト数だけポインタをシークさせます。
	 * 
	 * @param pos
	 *            プラスの値の場合は後，マイナスの値の場合は前にシーク
	 * @throws IOException
	 *             I/Oエラー発生時
	 */
	public void seekFromCurrent(long pos) throws IOException {
		long current = this.getFilePointer();
		seek(current + pos);
	}

	/**
	 * 現在のポインターからlongの値を読み込みます。
	 * 
	 * @return 読み込んだ long の値
	 * @throws IOException
	 *             I/Oエラー発生時
	 */
	public long readLongFromByte() throws IOException {
		return readFromByte(8);
	}

	/**
	 * 現在のポインターからintの値を読み込みます。
	 * 
	 * @return 読み込んだ int の値
	 * @throws IOException
	 *             I/Oエラー発生時
	 */
	public long readIntFromByte() throws IOException {
		return readFromByte(4);
	}

	/**
	 * 現在のポインターから指定したbyte分の値を読み込みます。
	 * 
	 * @param digit
	 *            読み込むバイト数
	 * @throws IOException
	 *             I/Oエラー発生時
	 */
	private long readFromByte(int digit) throws IOException {
		long value = 0;
		int mul = 1;
		for (int i = 0; i < digit; i++) {
			value += read() * mul;
			mul *= 256;
		}
		return value;
	}
}
