package org.hibara.attachecase.io;

import java.io.IOException;
import java.util.zip.Deflater;
import java.util.zip.DeflaterOutputStream;

/**
 * データを圧縮して保持するバッファとして機能します。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.23
 *
 */
public class ZipDeflateOutputStream extends ZipOutputStream {
	private DeflaterOutputStream deflateStream;

	/**
	 * コンストラクタ
	 * 
	 * @param level
	 *            圧縮レベル
	 * @param nowrap
	 *            ヘッダを付加するかどうかを指定。trueの場合はヘッダを付けないrawデータとなる
	 * @throws IOException
	 *             ファイル操作で失敗した場合
	 */
	public ZipDeflateOutputStream(int level, boolean nowrap) throws IOException {
		super();
		Deflater deflater = new Deflater(level, nowrap);
		deflateStream = new DeflaterOutputStream(byteStream, deflater);
	}

	/**
	 * オブジェクト破棄時にストリームをクローズします
	 */
	@Override
	protected void finalize() throws Throwable {
		this.close();
		super.finalize();
	}

	/**
	 * 渡されたデータを圧縮してストリームに書き込みます。
	 * 
	 * @param buffer
	 *            圧縮前のデータを格納したバッファ
	 * @throws IOException
	 *             データの書き込みに失敗した場合
	 */
	public void write(byte[] buffer) throws IOException {
		deflateStream.write(buffer, 0, buffer.length);
	}

	/**
	 * 渡されたデータを圧縮してストリームに書き込みます。
	 * 
	 * @param buffer
	 *            圧縮前のデータを格納したバッファ
	 * @param length
	 *            書き込むデータ長
	 * @throws IOException
	 *             データの書き込みに失敗した場合
	 */
	public void write(byte[] buffer, int length) throws IOException {
		deflateStream.write(buffer, 0, length);
	}

	/**
	 * データが終わりであることを指定します。
	 * 
	 * @throws IOException
	 *             データ操作に失敗
	 */
	public void finish() throws IOException {
		deflateStream.flush();
		deflateStream.finish();
	}

	/**
	 * データを読み出し可能かどうかを判定します。
	 */
	public boolean canRead() {
		try {
			deflateStream.flush();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return super.canRead();
	}

	/**
	 * ストリームを閉じます。
	 */
	public void close() {
		try {
			if (deflateStream != null) {
				deflateStream.close();
			}
			super.close();
		} catch (Exception e) {
			// 何もしない
		} finally {
			deflateStream = null;
		}
	}
}
