package org.hibara.attachecase.io;

import java.io.IOException;
import java.util.zip.Inflater;
import java.util.zip.InflaterOutputStream;

/**
 * データを伸張して保持するバッファとして機能します。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.23
 *
 */
public class ZipInflateOutputStream extends ZipOutputStream {
	private InflaterOutputStream inflateStream;

	/**
	 * コンストラクタ
	 * 
	 * @param nowrap
	 *            読み込ませるデータにヘッダが付加されているかどうかを指定。trueの場合はヘッダを付けないrawデータとなる
	 * @throws IOException
	 *             ファイル操作に失敗した場合
	 */
	public ZipInflateOutputStream(boolean nowrap) throws IOException {
		super();
		Inflater inflater = new Inflater(nowrap);
		inflateStream = new InflaterOutputStream(byteStream, inflater);
	}

	/**
	 * オブジェクト破棄時にストリームをクローズします
	 */
	@Override
	protected void finalize() throws Throwable {
		this.close();
		super.finalize();
	}

	/**
	 * 渡されたデータを圧縮してストリームに書き込みます。
	 * 
	 * @param buffer
	 *            圧縮前のデータを格納したバッファ
	 * @throws IOException
	 *             データの書き込みに失敗した場合
	 */
	public void write(byte[] buffer) throws IOException {
		inflateStream.write(buffer, 0, buffer.length);
	}

	/**
	 * 渡されたデータを圧縮してストリームに書き込みます。
	 * 
	 * @param buffer
	 *            圧縮前のデータを格納したバッファ
	 * @param length
	 *            書き込むデータ長
	 * @throws IOException
	 *             データの書き込みに失敗した場合
	 */
	public void write(byte[] buffer, int length) throws IOException {
		inflateStream.write(buffer, 0, length);
	}

	/**
	 * データが終わりであることを指定します。
	 * 
	 * @throws IOException
	 *             ファイル操作に失敗した場合
	 */
	public void finish() throws IOException {
		inflateStream.flush();
		inflateStream.finish();
	}

	/**
	 * ストリームを閉じます。
	 */
	public void close() {
		try {
			if (inflateStream != null) {
				inflateStream.close();
			}
			super.close();
		} catch (Exception e) {
			// 何もしない
		} finally {
			inflateStream = null;
		}
	}

}
