package org.hibara.attachecase.io;

import java.io.IOException;

/**
 * 圧縮・伸張したデータを格納するためのストリームです。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.23
 */
public class ZipOutputStream {
	// データを格納するためのストリーム
	protected ByteArrayOutputStream byteStream;

	/**
	 * コンストラクタ
	 */
	public ZipOutputStream() {
		byteStream = new ByteArrayOutputStream();
	}

	/**
	 * オブジェクト破棄時にストリームをクローズします
	 */
	@Override
	protected void finalize() throws Throwable {
		this.close();
		super.finalize();
	}

	/**
	 * 引数として渡されたバイト配列に蓄えられたデータを読み出します。<br>
	 * 読み出したデータはストリームからは削除されます<br>
	 * 
	 * @param buffer
	 *            読み出し用バッファ
	 * @return 読み出したバイト数
	 * @throws IOException
	 *             データの読み出しに失敗した場合
	 */
	public int read(byte[] buffer) throws IOException {
		int readLength = 0;
		if (byteStream.size() > 0) {
			readLength = Math.min(byteStream.size(), buffer.length);
			byte[] byteBuffer = byteStream.toByteArray();
			for (int i = 0; i < readLength; i++) {
				buffer[i] = byteBuffer[i];
			}
			// 読み出したデータを削除
			byteStream.reset();
			if (byteStream.size() - readLength > 0) {
				byteStream.write(byteBuffer, readLength, byteStream.size() - readLength);
			}
		}

		return readLength;
	}

	/**
	 * すべてのデータを読み出します。ストリームは空になります。<br>
	 * 
	 * @return ストリームから読み出されたデータ
	 * @throws IOException
	 *             データの読み出しに失敗した場合
	 */
	public byte[] readAll() throws IOException {
		if (byteStream.size() == 0) {
			return null;
		}
		byte[] byteBuffer = byteStream.toByteArray();
		byteStream.reset();
		return byteBuffer;
	}

	/**
	 * ストリームからデータを読み出し可能かどうかを判定します。
	 * 
	 * @return 1バイトでも読み出し可能であればtrue
	 */
	public boolean canRead() {
		return (byteStream.size() > 0);
	}

	/**
	 * 指定したバイト数が読み出し可能かを判定します。<br>
	 * lengthとして0が渡された場合は，canRead()と同じ動作になります。<br>
	 * 
	 * @param length
	 *            データ数
	 * @return 指定されたデータ数を読み出し可能かどうか
	 */
	public boolean canRead(int length) {
		if (length == 0) {
			return this.canRead();
		}
		return (byteStream.size() >= length);
	}

	/**
	 * ストリームを閉じます。
	 */
	public void close() {
		try {
			if (byteStream != null) {
				byteStream.close();
			}
		} catch (Exception e) {
			// 何もしない
		} finally {
			byteStream = null;
		}
	}

}
