/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */

package org.hibara.attachecase.utility;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.entity.ShaEntity;

/**
 * SHAハッシュを求めます。
 * 
 * @author Masanori Uozumi
 * @version 2017.01.20<br>
 *          2012.08.31
 *          パスワード用データとして利用するデータを160ビット＋96ビット(12バイト）とする。12バイト分は最後に読み込んだデータを利用。
 *          2017.01.20 アタッシェケース#3用にSHA-256のメソッドを追加。
 */
public class Digest implements AttacheCaseConstant {
	private final static String SHA1_ALGOLITHM = "SHA1";
	private final static String SHA256_ALGOLITHM = "SHA-256";
	private byte[] fileHeader = new byte[BUF_HEADER_SIZE];
	private byte[] lastBufferData = new byte[BUF_HEADER_SIZE];

	/**
	 * 指定したファイルのSHA-256のハッシュ値を連結した文字列を取得します。<br>
	 * ハッシュ値は10進数表記で連結します。<br>
	 * 
	 * @param path
	 *            ファイルパス
	 * @return ハッシュ値は10進数表記にして連結
	 */
	public String getSha256ByString(String path) {
		StringBuilder sb = new StringBuilder();
		try {
			byte[] hash = getSha256(path);
			for (byte value : hash) {
				int intValue = value;
				if (intValue < 0) {
					intValue = 255 - (byte) (value ^ 0xff);
				}
				sb.append(intValue);
			}
		} catch (IOException e) {
			// 何もしない
		}
		return sb.toString();
	}

	/**
	 * 指定したファイルのSHA-256ハッシュ値を取得する
	 * 
	 * @param path
	 *            ファイルパス
	 * @return SHA-256ハッシュ
	 * @throws IOException
	 *             ファイル操作に失敗した場合
	 */
	public byte[] getSha256(String path) throws IOException {
		File f = new File(path);
		byte[] hash = null;
		if (!f.exists()) {
			throw new FileNotFoundException();
		}
		try {
			// Hashの取得
			hash = getFileDigest(f, SHA256_ALGOLITHM); // shaHashは20バイト
			if (DEBUG)
				System.out.printf("\nSHA HASH\n");
			// 160bit（20バイト）のハッシュを32バイトの領域にコピー
			for (int i = 0; i < hash.length; i++) {
				if (DEBUG)
					System.out.printf("%02x ", hash[i]);
			}
			if (DEBUG)
				System.out.printf("\n");
		} catch (NoSuchAlgorithmException e) {
			// 何もしない
			if (DEBUG)
				e.printStackTrace();
		} catch (IOException e) {
			if (DEBUG)
				e.printStackTrace();
			throw e;
		}
		return hash;
	}

	/**
	 * 指定したファイルのSHA-1ハッシュを取得します。
	 * 
	 * @param path
	 *            パスワードとして利用するファイルのパス
	 * @return SHA-1を格納したオブジェクト
	 * @throws IOException
	 *             パスワードの読み込みでエラーが発生した場合
	 */
	public ShaEntity getHash(String path) throws IOException {
		File f = new File(path);
		ShaEntity shaEntity = null;
		if (!f.exists()) {
			throw new FileNotFoundException();
		}
		try {
			shaEntity = new ShaEntity();
			// Hashの取得
			byte[] shaHash = getFileDigest(f, SHA1_ALGOLITHM); // shaHashは20バイト
			byte[] hash = new byte[BUF_SHA1_SIZE]; // hashは32バイト
			if (DEBUG)
				System.out.printf("\nSHA HASH\n");
			// 160bit（20バイト）のハッシュを32バイトの領域にコピー
			for (int i = 0; i < shaHash.length; i++) {
				hash[i] = shaHash[i];
				if (DEBUG)
					System.out.printf("%02x ", shaHash[i]);
			}
			if (DEBUG)
				System.out.printf("\n");
			// 残りの12バイトにはファイルのデータを入れておく
			// 入れるデータは最後に読み込んだバッファのデータ
			for (int i = 0; i < (hash.length - shaHash.length); i++) {
				hash[shaHash.length + i] = lastBufferData[i];
			}
			if (DEBUG) {
				for (int i = 0; i < hash.length; i++) {
					System.out.printf("%02x ", hash[i]);
				}
			}
			shaEntity.setHashArray(hash);
			shaEntity.setHashString(shaHash);
			// HeaderStringの取得
			shaEntity.setHeaderArray(getHeaderByte(fileHeader));
			shaEntity.setHeaderString(getHeaderString(shaEntity.getHeaderArray()));
		} catch (NoSuchAlgorithmException e) {
			// 何もしない
			if (DEBUG)
				e.printStackTrace();
		} catch (IOException e) {
			if (DEBUG)
				e.printStackTrace();
			throw e;
		}
		return shaEntity;
	}

	/**
	 * 指定したファイルのSHA-1ハッシュを計算します。
	 * 
	 * @param f
	 *            SHA-1ハッシュを求めるファイル
	 * @return SHA-1ハッシュが格納された配列
	 * @throws IOException
	 * @throws NoSuchAlgorithmException
	 */
	private byte[] getFileDigest(File f, String algolithm) throws IOException, NoSuchAlgorithmException {
		BufferedInputStream is = null;
		MessageDigest md = null;
		try {
			is = new BufferedInputStream(new FileInputStream(f));
			byte[] buffer = new byte[BUF_HEADER_SIZE];
			md = MessageDigest.getInstance(algolithm);
			int length = 0;
			int count = 0;
			while ((length = is.read(buffer)) != -1) {
				if (count == 0) {
					fileHeader = Arrays.copyOfRange(buffer, 0, length);
				}
				count++;
				md.update(buffer, 0, length);
			}
			// 12バイト分のデータを追加する必要があるので，最後に読み込んだデータを取っておく
			lastBufferData = buffer.clone();
		} catch (IOException e) {
			throw e;
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// 何もしない
				}
			}

		}
		return md.digest();
	}

	/**
	 * 与えられたバイト配列を文字列に変換します。
	 * 
	 * <pre>
	 * バイト配列を文字列に変換します。ただしバイト配列に0があるとそこで打ち切ります。
	 * つまり0をC言語のヌル文字と解釈し，そこまでを文字列に変換します。
	 * </pre>
	 * 
	 * @param array
	 * @return
	 */
	private String getHeaderString(byte[] array) {
		int pos = indexOf(array, (byte) 0);
		if (pos == -1) {
			pos += 256;
		}
		int length = (array.length < pos) ? array.length : pos;
		byte[] buffer = new byte[pos + 1];
		System.arraycopy(array, 0, buffer, 0, length);
		return new String(buffer);
	}

	/**
	 * 与えられたバイト配列の先頭から最長256バイト分を取得。
	 * 
	 * @param array
	 *            バイト配列
	 * @return 先頭から最長256バイト分（与えられた配列の要素数が256より小さい場合はその長さ分）
	 */
	private byte[] getHeaderByte(byte[] array) {
		int length = (array.length > 256) ? 256 : array.length;
		byte[] buffer = new byte[length];
		System.arraycopy(array, 0, buffer, 0, length);
		return buffer;
	}

	/**
	 * バイト配列を16進数の文字列に変換します。
	 * 
	 * <pre>
	 * 配列{af,07,45,1f} を 文字列 "af07451f" に変換します。
	 * </pre>
	 * 
	 * @param bytes
	 *            バイト配列
	 * @return 16進数の文字列
	 */
	public String toEncryptedString(byte[] bytes) {
		StringBuilder sb = new StringBuilder();
		for (byte b : bytes) {
			String hex = String.format("%02x", b);
			sb.append(hex);
		}
		return sb.toString();
	}

	/**
	 * 配列の中から，指定した値の位置を取得します。
	 * 
	 * @param array
	 *            配列
	 * @param value
	 *            探索する値
	 * @return 値の位置。見つからなかった場合の戻り値は-1
	 */
	private int indexOf(byte[] array, byte value) {
		int pos = -1;
		for (int i = 0; i < array.length; i++) {
			if (value == array[i]) {
				pos = i;
				break;
			}
		}
		return pos;
	}
}
