/*
 * Copyright (C) 2012-2017 Masanori Uozumi.
 */

package org.hibara.attachecase.utility;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.FileTime;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.engines.BlowfishEngine;
import org.bouncycastle.crypto.paddings.PaddedBufferedBlockCipher;
import org.bouncycastle.crypto.params.KeyParameter;
import org.hibara.attachecase.AttacheCaseConstant;
import org.hibara.attachecase.Options;
import org.hibara.attachecase.entity.FileAttribute;
import org.hibara.attachecase.entity.FileAttribute3;
import org.hibara.attachecase.entity.InfoTable;
import org.hibara.attachecase.io.ByteArrayOutputStream;
import org.hibara.attachecase.io.RandomAccessFile;

/**
 * アタッシェケース用ユーティリティ・クラス。
 * 
 * <pre>
 * アタッシェケースで利用するユーティリティ・メソッドを集めたクラスです。
 * メソッドはすべてstaticになっています。
 * </pre>
 * 
 * @author Masanori Uozumi
 * @version 2017.01.18<br>
 *          2012.08.31 暗号化，復号用メソッドを追加<br>
 *          2013.01.09 C++のTTimeStampの計算で1日ずれるバグを修正<br>
 *          2014.01.28 getDiskFreeSpaceメソッドで例外を送出<br>
 *          2014.07.24 暗号化，復号用メソッドでエラーが発生していたバグを修正<br>
 *          2017.01.18 アタッシェケース#3用メソッドの追加<br>
 */

public class Utility implements AttacheCaseConstant {

	/**
	 * 実行環境のOS名.
	 */
	private static String osName = System.getProperty("os.name");

	/**
	 * 指定したパス名がパス区切り文字で終わるように調整します。
	 * 
	 * @param path
	 *            パス区切り文字で終わっているかどうかをチェックするパス名
	 * @return パス区切り文字が追加されたパス名
	 */
	public static String includeTrailingPathDelimiter(String path) {
		if (!path.endsWith(File.separator)) {
			path = path + File.separator;
		}
		return path;
	}

	/**
	 * 先頭の . と / を削除する
	 * 
	 * @param path
	 *            パス名
	 * @return 先頭の . と / を削除したパス名
	 */
	public static String rejectPathTraversal(String path) {
		String work = path;
		while (work.startsWith("../") || work.startsWith("./")) {
			if (work.startsWith("../")) {
				work = work.substring("../".length());
			} else if (work.startsWith("./")) {
				work = work.substring("./".length());
			}
		}

		return work;
	}

	/**
	 * 指定したパスが存在するディスクの空き容量を取得します。<br>
	 * 2014/01/28 SecurityExceptionをthrowするように変更
	 * 
	 * @param path
	 *            空き容量を取得するパス名
	 * @return ディスクの空き容量
	 * @throws SecurityException
	 *             セキュリティ例外
	 */
	public static long getDiskFreeSpace(String path) throws SecurityException {
		long freeSpace = -1;
		File f = new File(path);
		if (!f.isFile()) {
			f = f.getParentFile();
		}
		freeSpace = f.getUsableSpace();
		return freeSpace;
	}

	public static boolean setFileAttribute(String path, FileAttribute attr) {
		boolean result = false;
		File f = new File(path);
		if (!f.exists()) {
			return result;
		}
		// リードオンリー属性
		if ((attr.getAttribute() & AttacheCaseConstant.READ_ONLY_FILE_ATTR) != 0) {
			result = f.setWritable(false);
		}
		// アーカイブ属性は設定不能…
		return result;
	}

	/**
	 * ディレクトリまたはファイルの更新日付を指定した日付に設定します。
	 * 
	 * @param path
	 *            更新日付を設定するパス
	 * @param attr
	 *            ファイル情報
	 * @param now
	 *            trueを指定すると現在時刻
	 */
	public static void setFileTimeStamp(String path, FileAttribute attr, boolean now) {
		Date modified = null;
		Date create = null;

		if (attr instanceof FileAttribute3) {
			modified = convetToDate(attr.getModifiedDate(), attr.getModifiedTime());
			create = convetToDate(attr.getCreateDate(), attr.getCreateTime());
		} else {
			modified = convetToDate((int) attr.getModifiedDate(), (int) attr.getModifiedTime());
			create = convetToDate((int) attr.getCreateDate(), (int) attr.getCreateTime());
		}
		FileTime creationTime = FileTime.fromMillis(create.getTime());
		FileTime modifiedTime = FileTime.fromMillis(modified.getTime());

		TimeZone tz = TimeZone.getTimeZone("UTC");
		Calendar cal = Calendar.getInstance(tz);
		FileTime currentTime = FileTime.fromMillis(cal.getTimeInMillis());

		if (now) {
			creationTime = modifiedTime = currentTime;
		}

		// ファイルの日付を変更
		if (Double.parseDouble(System.getProperty("java.specification.version")) >= 1.7) {
			// Java7以上であれば
			try {
				Path p = Paths.get(path);
				Files.setAttribute(p, "creationTime", creationTime);
				Files.setAttribute(p, "lastModifiedTime", modifiedTime);
				Files.setAttribute(p, "lastAccessTime", currentTime);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			// Java6以下であれば
			setFileTimeStamp(path, modified.getTime(), create.getTime(), now);
		}
	}

	/**
	 * ディレクトリまたはファイルの更新日付を指定した日付に設定します。
	 * 
	 * @param path
	 *            - 更新日付を設定するパス
	 * @param modified
	 *            - 設定する更新日付
	 * @param create
	 *            - 設定する作成日付(Javaで設定不能）
	 * @param now
	 *            - 現在の日時を設定する。true の場合，更新日付，作成日付に現在の日時を設定する。
	 */
	private static void setFileTimeStamp(String path, long modified, long create, boolean now) {
		File f = new File(path);
		if (!f.exists())
			return; // ファイルが存在しなければ何もしない
		// 協定世界時で時刻を取得
		if (now) {
			TimeZone tz = TimeZone.getTimeZone("UTC");
			Calendar cal = Calendar.getInstance(tz);
			create = modified = cal.getTimeInMillis();
		}
		// 更新日時を変更
		f.setLastModified(modified);
		// 作成日時はJavaからは変更できない…
	}

	/**
	 * OSがWindowsの場合にtrueを返します。
	 * 
	 * @return OS が Windows の場合は true，それ以外の場合は false
	 */
	public static boolean isWindows() {
		return osName.toLowerCase().indexOf("windows") != -1;
	}

	/**
	 * プレースホルダを指定した値で置き換えます。
	 * 
	 * <pre>
	 * プレースホルダは {0} の形で与えます。初期値は 0 から始まります。
	 * </pre>
	 * 
	 * @param s
	 *            - プレースホルダーを含む文字列
	 * @param obj
	 *            - 置き換える値。Objectクラスの配列として与える
	 * @return プレースホルダーが指定した値で置き換えられた文字列
	 */
	public static String replacePlaceHolder(String s, Object[] obj) {
		MessageFormat mf = new MessageFormat(s);
		return mf.format(obj);
	}

	/**
	 * C++のTTimeStamp型をDate型に変更します。
	 * 
	 * <pre>
	 * TTimeStamp型については次の情報を参考にしてください。<a href=
	"http://docwiki.embarcadero.com/Libraries/ja/System.SysUtils.TTimeStamp">System.SysUtils.TTimeStamp</a>
	 * </pre>
	 * 
	 * @param date
	 *            - 西暦1年1月1日からの暦日数（1/1/0001 から経過した日数に 1 日追加）を示します。
	 * @param time
	 *            - 午前 0 時から経過したミリ秒数を表します。
	 * @return 日付
	 */
	public static Date convetToDate(int date, int time) {
		long day = (date) * 1000L * 3600L * 24 + time + AttacheCaseConstant.START_OF_DATE;
		TimeZone tz = TimeZone.getDefault();
		day += 24 * 3600 * 1000L + tz.getRawOffset(); // 1日 ＋ タイムゾーン
		return new Date(day);
	}

	/**
	 * doubleをDate型に変更します。
	 * 
	 * @param date
	 *            西暦1年1月1日からの暦日数を示します。
	 * @param time
	 *            午前 0 時から経過したミリ秒数を表します。
	 * @return 日付
	 */
	public static Date convetToDate(double date, double time) {
		double day = date * 3600 * 24 * 1000 + Utility.get00010101Time() + 3600 * 24 * 1000 * 2 + time * 1000;
		return new Date((long) day);
	}

	/**
	 * Date型オブジェクトを日時文字列に変換します。
	 * 
	 * @param date
	 *            日付
	 * @return 日付文字列
	 */
	public static String getDateTime(Date date) {
		SimpleDateFormat df = new SimpleDateFormat(AttacheCaseConstant.DATE_FORMAT_PATTERN);
		return df.format(date);
	}

	/**
	 * Max上で動作しているかどうかを確認します。
	 * 
	 * @return Mac上で動作している場合はtrue
	 */
	public static boolean isMac() {
		String osName = System.getProperty("os.name").toLowerCase();
		return osName.startsWith("mac os x");
	}

	/**
	 * アタッシェケースで暗号化されたファイルかどうかを確認します。
	 * 
	 * @param f
	 *            確認するファイル
	 * @return アタッシェケースで暗号化されたファイルの場合はtrue
	 */
	public static boolean isAtcFile(File f) {
		boolean result = false;
		RandomAccessFile fis = null;
		try {
			fis = new RandomAccessFile(f, "r");
			InfoTable infoTable = new InfoTable(fis);
			infoTable.readFromFile(fis);
			result = infoTable.isMatchToken();
		} catch (Exception e) {
			result = false;
		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (IOException ex) {
					// 何もしない
				}
			}
		}
		return result;
	}

	/**
	 * アタッシェケースファイルのバージョンを取得します
	 * 
	 * @param f
	 *            チェックするファイル
	 * @return Version2の場合は2，Version3の場合は3
	 */
	public static int getAtcVersion(File f) {
		int version = -1;
		RandomAccessFile fis = null;
		try {
			fis = new RandomAccessFile(f, "r");
			InfoTable info = new InfoTable(fis);
			info.readFromFile();
			if (!info.isExistToken()) {
				return version;
			}

			if (info.getDataVersion() <= ATC_DATA_FILE_VERSION) {
				version = 2;
			} else if (info.getDataVersion() > ATC_DATA_FILE_VERSION
					&& info.getDataVersion() < ATC_DATA_FILE_VERSION_MAX) {
				version = 3;
			}
		} catch (Exception e) {
			// e.printStackTrace();
			version = -1;
		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (IOException ex) {
					// 何もしない
				}
			}
		}

		return version;
	}

	/**
	 * 文字列を暗号化します。<br>
	 * 暗号化したデータはBase64でエンコードした文字列として返ります。<br>
	 * 
	 * @param key
	 *            暗号化用キー
	 * @param text
	 *            暗号化する文字列
	 * @return 暗号化されBase64でエンコードされた文字列
	 */
	public static String encrypt(String key, String text) {
		try {
			BlockCipher engine = new BlowfishEngine();
			PaddedBufferedBlockCipher cipher = new PaddedBufferedBlockCipher(engine);
			KeyParameter keyParameter = new KeyParameter(key.getBytes());
			cipher.init(true, keyParameter);
			byte input[] = text.getBytes();
			byte output[] = new byte[cipher.getOutputSize(input.length)];
			int length = cipher.processBytes(input, 0, input.length, output, 0);
			cipher.doFinal(output, length);
			String s = new String(org.bouncycastle.util.encoders.Base64.encode(output));
			return s;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 文字列を復号します。<br>
	 * 与える文字列はBase64でエンコードした文字列です。
	 * 
	 * @param key
	 *            復号用キー
	 * @param encryptText
	 *            復号する文字列（Base64でエンコード）
	 * @return 復号された文字列
	 */
	public static String decrypt(String key, String encryptText) {
		if (key == null || encryptText == null) {
			return null;
		}
		if (key.length() == 0 || encryptText.length() == 0) {
			return "";
		}

		try {
			BlockCipher engine = new BlowfishEngine();
			PaddedBufferedBlockCipher cipher = new PaddedBufferedBlockCipher(engine);
			StringBuilder result = new StringBuilder();
			KeyParameter keyParameter = new KeyParameter(key.getBytes());
			cipher.init(false, keyParameter);
			byte base64[] = org.bouncycastle.util.encoders.Base64.decode(encryptText);
			byte output[] = new byte[cipher.getOutputSize(base64.length)];
			int length = cipher.processBytes(base64, 0, base64.length, output, 0);
			cipher.doFinal(output, length);
			String s2 = new String(output);
			for (int i = 0; i < s2.length(); i++) {
				char c = s2.charAt(i);
				if (c != 0) {
					result.append(c);
				}
			}
			return result.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * 起動パスを取得します。
	 * 
	 * @return 起動パス
	 */
	public static String getStartupPath() {
		String path = includeTrailingPathDelimiter(System.getProperty("user.dir"));
		File dir = null;
		URL url = Options.class.getResource("Options.class");
		String protocol = url.getProtocol();
		if (protocol.equals("jar")) {
			Pattern p = Pattern.compile("(file:/.*)!/");
			Matcher m = p.matcher(url.toString());
			if (!m.find()) {
				return path;
			}

			File jarFile = null;
			try {
				jarFile = new File(new URI(m.group(1)));
			} catch (URISyntaxException e) {
				// e.printStackTrace();
				return path;
			}
			dir = jarFile.getParentFile();
		} else if (protocol.equals("file")) {
			File classFile = null;
			try {
				classFile = new File(new URI(url.toString()));
			} catch (URISyntaxException e) {
				// e.printStackTrace();
				return path;
			}

			Package pack = Options.class.getPackage();
			if (pack == null) {
				dir = classFile.getParentFile();
			} else {
				String packName = pack.getName();
				String[] dirs = packName.split("\\.");
				dir = classFile.getParentFile();
				for (int i = 0; i < dirs.length; i++) {
					dir = dir.getParentFile();
				}
			}
		} else {
			return path;
		}
		return dir.getAbsolutePath();
	}

	/**
	 * アタッシェケース#3で用いるランダムソルトを取得する
	 * 
	 * @param length
	 *            ランダムソルトの長さ
	 * @return ランダムソルトを格納したバイト配列
	 */
	public static byte[] getRandomSalt(int length) {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		for (int i = 0; i < length; i++) {
			os.write((byte) (Math.floor(Math.random() * 256) - 128));
		}
		byte[] salt = os.toByteArray();
		try {
			os.close();
		} catch (IOException e) {
			// 何もしない
		}
		return salt;
	}

	/**
	 * 0001年1月1日のlong値を求める
	 * 
	 * @return 0001年1月1日のlong値
	 */
	public static long get00010101Time() {
		Date date00010101 = null;
		// 変換対象の日付文字列
		String dateStr = "00010101 00:00:00";
		try {
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HH:mm:ss");
			// Date型変換
			date00010101 = sdf.parse(dateStr);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (date00010101 != null) {
			return date00010101.getTime();
		}
		return 0;
	}

}
